@@ -1,22 +1,22 @@
-/* $Header: /cvsroot/nco/nco/src/nco/libnco.h,v 1.27 2002/12/30 02:56:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/libnco.h,v 1.31 2004/02/09 07:54:41 zender Exp $ */
 
-/* Purpose: Prototypes, typedefs, and global variables for libcsm_c++ */
+/* Purpose: netCDF Operator (NCO) library */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* libnco.h headers depend on netcdf.h and nco_netcdf.h headers */
 
 /* Usage:
-   #include "libnco.h" *//* netCDF operator library */
+   #include "libnco.h" *//* netCDF Operator (NCO) library */
 
 #ifndef LIBNCO_H /* Contents have not yet been inserted in current source file */
 #define LIBNCO_H
 
 /* Core headers */
-#include "nco_netcdf.h" /* Wrappers for netCDF 3.X C-library */
-#include "nco.h" /* netCDF operator definitions */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
+#include "nco.h" /* netCDF Operator (NCO) definitions */
 
 /* Library headers */
 #include "nco_att_utl.h" /* Attribute utilities */
@@ -25,7 +25,7 @@
 #include "nco_cnf_dmn.h" /* Conform dimensions */
 #include "nco_cnf_typ.h" /* Conform variable types */
 #include "nco_cnv_arm.h" /* ARM conventions */
-#include "nco_cnv_csm.h" /* CSM conventions */
+#include "nco_cnv_csm.h" /* CCSM conventions */
 #include "nco_ctl.h" /* Program flow control functions */
 #include "nco_dbg.h" /* Debugging */
 #include "nco_dmn_utl.h" /* Dimension utilities */
