@@ -1,12 +1,33 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncap.c,v 1.108 2003/01/20 17:46:02 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncap.c,v 1.139 2004/09/07 04:31:19 zender Exp $ */
 
 /* ncap -- netCDF arithmetic processor */
 
 /* Purpose: Compute user-defined derived fields using forward algebraic notation applied to netCDF files */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2004 Charlie Zender
+
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
+   The full license text is at http://www.gnu.ai.mit.edu/copyleft/gpl.html 
+   and in the file nco/doc/LICENSE in the NCO source distribution.
+   
+   As a special exception to the terms of the GPL, you are permitted 
+   to link the NCO source code with the DODS, HDF, netCDF, and UDUnits
+   libraries and to distribute the resulting executables under the terms 
+   of the GPL, but in addition obeying the extra stipulations of the 
+   DODS, HDF, netCDF, and UDUnits licenses.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+   See the GNU General Public License for more details.
+   
+   The original author of this software, Charlie Zender, wants to improve it
+   with the help of your suggestions, improvements, bug-reports, and patches.
+   Please contact the NCO project at http://nco.sf.net or by writing
+   Charlie Zender
+   Department of Earth System Science
+   University of California at Irvine
+   Irvine, CA 92697-3100 */
 
 /* Usage:
    ncap -O -D 1 -S ${HOME}/nco/data/ncap.in ${HOME}/nco/data/in.nc ${HOME}/nco/data/foo.nc
@@ -27,29 +48,31 @@
 #include <sys/stat.h> /* stat() */
 #include <time.h> /* machine time */
 #include <unistd.h> /* all sorts of POSIX stuff */
-#ifdef HAVE_GETOPT_H 
-/* GNU getopt() is independent system header on FREEBSD, LINUX, LINUXALPHA, WIN32
+/* GNU getopt() is independent system header on FREEBSD, LINUX, LINUXALPHA, LINUXAMD, LINUXARM, WIN32
    AT&T getopt() is in unistd.h or stdlib.h on AIX, CRAY, NECSX, SUNMP, SUN4SOL2
    fxm: I'm not sure what ALPHA and SGI do */
-#include <getopt.h> /* getopt_long() */
-#else
+#ifndef HAVE_GETOPT_LONG
 #include "nco_getopt.h"
-#endif /* !HAVE_GETOPT_H */
+#else /* !NEED_GETOPT_LONG */ 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif /* !HAVE_GETOPT_H */ 
+#endif /* HAVE_GETOPT_LONG */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
+#include <netcdf.h> /* netCDF definitions and C library */
 #include "nco_netcdf.h"  /* NCO wrappers for libnetcdf.a */
 
 /* Personal headers */
-/* #define MAIN_PROGRAM_FILE MUST precede #include nco.h */
+/* #define MAIN_PROGRAM_FILE MUST precede #include libnco.h */
 #define MAIN_PROGRAM_FILE
-#include "ncap.h" /* ncap-specific definitions */
-#include "libnco.h" /* netCDF operator library */
+#include "ncap.h" /* netCDF arithmetic processor-specific definitions (symbol table, ...) */
+#include "libnco.h" /* netCDF Operator (NCO) library */
 
 /* Global variables */
-size_t ncl_dpt_crr=0UL; /* [nbr] Depth of current #include file (incremented in ncap.l) */
-size_t *ln_nbr_crr; /* [cnt] Line number (incremented in ncap.l) */
-char **fl_spt_glb; /* [fl] Script file */
+size_t ncap_ncl_dpt_crr=0UL; /* [nbr] Depth of current #include file (incremented in ncap.l) */
+size_t *ncap_ln_nbr_crr; /* [cnt] Line number (incremented in ncap.l) */
+char **ncap_fl_spt_glb; /* [fl] Script file */
 
 int 
 main(int argc,char **argv)
@@ -64,16 +87,17 @@ main(int argc,char **argv)
   
   bool EXCLUDE_INPUT_LIST=False; /* Option c */
   bool FILE_RETRIEVED_FROM_REMOTE_LOCATION;
+  bool FL_LST_IN_FROM_STDIN=False; /* [flg] fl_lst_in comes from stdin */
   bool FORCE_APPEND=False; /* Option A */
   bool FORCE_OVERWRITE=False; /* Option O */
-  bool FORTRAN_STYLE=False; /* Option F */
+  bool FORTRAN_IDX_CNV=False; /* Option F */
   bool HISTORY_APPEND=True; /* Option h */
-  bool NCAR_CSM_FORMAT;
+  bool NCAR_CCSM_FORMAT;
   bool PROCESS_ALL_COORDINATES=False; /* Option c */
   bool PROCESS_ASSOCIATED_COORDINATES=True; /* Option C */
   bool REMOVE_REMOTE_FILES_AFTER_PROCESSING=True; /* Option R */
-  bool PROCESS_ALL_VARS=True;     /* option v */  
-  bool PRINT_FUNCTION_TABLE=False;
+  bool PROCESS_ALL_VARS=True; /* Option v */  
+  bool PRN_FNC_TBL=False; /* Option f */  
   
   char **var_lst_in=NULL_CEWI;
   char **fl_lst_abb=NULL; /* Option n */
@@ -85,39 +109,44 @@ main(int argc,char **argv)
 #define NCAP_SPT_NBR_MAX 100
   char *spt_arg[NCAP_SPT_NBR_MAX]; /* fxm: Arbitrary size, should be dynamic */
   char *spt_arg_cat=NULL; /* [sng] User-specified script */
-  char *opt_sng;
-  char *fl_out;
+  char *fl_out=NULL; /* Option o */
   char *fl_out_tmp;
   char *fl_pth=NULL; /* Option p */
   char *time_bfr_srt;
   char *cmd_ln;
-  char CVS_Id[]="$Id: ncap.c,v 1.108 2003/01/20 17:46:02 zender Exp $"; 
-  char CVS_Revision[]="$Revision: 1.108 $";
-  
-  dmn_sct **dmn_in=NULL_CEWI;  /* holds ALL DIMS in the input file */
-  dmn_sct **dmn_out=NULL_CEWI; /* Holds DIMS that have been written to OUTPUT */
-  dmn_sct **dmn_new=NULL_CEWI; /* Temporary dim to reduce referencing */
-  int nbr_dmn_out=0;           /* Number of dims in list dmn_out */
+
+  const char * const CVS_Id="$Id: ncap.c,v 1.139 2004/09/07 04:31:19 zender Exp $"; 
+  const char * const CVS_Revision="$Revision: 1.139 $";
+  const char * const opt_sng="ACcD:d:Ffhl:n:Oo:p:Rrs:S:vx-:"; /* [sng] Single letter command line options */
+
+  dmn_sct **dmn_in=NULL_CEWI;  /* [lst] Dimensions in input file */
+  dmn_sct **dmn_out=NULL_CEWI; /* [lst] Dimensions written to output file */
+  dmn_sct **dmn_new=NULL_CEWI; /* [lst] Temporary dimensions to reduce referencing */
+
+  int nbr_dmn_out=0; /* [nbr] Number of dimensions in list dmn_out */
   
   extern char *optarg;
   extern int optind;
   
   /* Math float prototypes required by AIX, Solaris, but not by Linux, IRIX */
-  /* Basic math: acos, asin, atan, cos, exp, log, log10, sin, sqrt, tan */
+  /* Basic math: acos, asin, atan, cos, exp, fabs, log, log10, sin, sqrt, tan */
   extern float acosf(float);
   extern float asinf(float);
   extern float atanf(float);
   extern float cosf(float);
-  extern float erff(float);
-  extern float erfcf(float);
   extern float expf(float);
-  extern float gammaf(float);
+  extern float fabsf(float); /* 20040629: Only AIX may need this */
   extern float logf(float);
   extern float log10f(float);
   extern float sinf(float);
   extern float sqrtf(float);
   extern float tanf(float);
   
+  /* Advanced math: erf, erfc, gamma */
+  extern float erff(float);
+  extern float erfcf(float);
+  extern float gammaf(float);
+
   /* Hyperbolic trigonometric: acosh, asinh, atanh, cosh, sinh, tanh */
   extern float acoshf(float);
   extern float asinhf(float);
@@ -141,8 +170,7 @@ main(int argc,char **argv)
   int jdx;
   int in_id;  
   int out_id;  
-  int nbr_abb_arg=0;
-  int nbr_dmn_fl;
+  int abb_arg_nbr=0;
   int lmt_nbr=0; /* Option d. NB: lmt_nbr gets incremented */
   int nbr_spt=0; /* Option s. NB: nbr_spt gets incremented */
   int nbr_var_fl;/* number of vars in a file */
@@ -151,29 +179,27 @@ main(int argc,char **argv)
   int nbr_xtr=0; /* nbr_xtr will not otherwise be set for -c with no -v */
   int nbr_dmn_in=int_CEWI; /* number of dims in dim_in */
   int nbr_dmn_ass=int_CEWI;/* number of dims in temporary list */
-  int nbr_fl=0;
+  int fl_nbr=0;
   int nbr_lst_a=0; /* size of xtr_lst_a */
   int opt;
-  int rec_dmn_id=NCO_REC_DMN_UNDEFINED;
   int rcd=NC_NOERR; /* [rcd] Return code */
-  int sng_lng;
   int var_id;
-  int spt_arg_lng=int_CEWI;
   int nbr_att=0; /* [nbr] Number of attributes in script */
   
   sym_sct **sym_tbl; /* [fnc] Symbol table for functions */
   int sym_tbl_nbr; /* [nbr] Size of symbol table */
   int sym_idx=0; /* [idx] Counter for symbols */
   
-  
-  
   lmt_sct *lmt=NULL_CEWI;
   
   nm_id_sct *dmn_lst=NULL;       
   nm_id_sct *xtr_lst=NULL;   /* The list of non-processed variables which get copied to OUTPUT */
   nm_id_sct *xtr_lst_a=NULL; /* initialized to ALL the vars in the OUTPUT file */
   
-  time_t clock;
+  size_t sng_lng;
+  size_t spt_arg_lng=size_t_CEWI;
+
+  time_t time_crr_time_t;
   
   var_sct **var;
   var_sct **var_fix;
@@ -187,26 +213,37 @@ main(int argc,char **argv)
   prs_sct prs_arg; /* [sct] Global information required in parser routines */
   
   static struct option opt_lng[]=
-    {
+    { /* Structure ordered by short option key if possible */
       {"append",no_argument,0,'A'},
       {"coords",no_argument,0,'c'},
-      {"nocoords",no_argument,0,'C'},
+      {"crd",no_argument,0,'c'},
+      {"no-coords",no_argument,0,'C'},
+      {"no-crd",no_argument,0,'C'},
       {"debug",required_argument,0,'D'},
       {"dbg_lvl",required_argument,0,'D'},
       {"dimension",required_argument,0,'d'},
       {"dmn",required_argument,0,'d'},
-      {"ftn",no_argument,0,'F'},
+      {"fnc_tbl",no_argument,0,'f'},
+      {"prn_fnc_tbl",no_argument,0,'f'},
       {"ftn",no_argument,0,'F'},
       {"history",no_argument,0,'h'},
       {"hst",no_argument,0,'h'},
       {"local",required_argument,0,'l'},
+      {"lcl",required_argument,0,'l'},
       {"nintap",required_argument,0,'n'},
       {"overwrite",no_argument,0,'O'},
+      {"ovr",no_argument,0,'O'},
+      {"output",required_argument,0,'o'},
+      {"fl_out",required_argument,0,'o'},
       {"path",required_argument,0,'p'},
       {"retain",no_argument,0,'R'},
+      {"rtn",no_argument,0,'R'},
       {"revision",no_argument,0,'r'},
-      {"script",required_argument,0,'s'},
       {"file",required_argument,0,'S'},
+      {"script-file",required_argument,0,'S'},
+      {"fl_spt",required_argument,0,'S'},
+      {"spt",required_argument,0,'s'},
+      {"script",required_argument,0,'s'},
       {"units",no_argument,0,'u'},
       {"variable",no_argument,0,'v'},
       {"version",no_argument,0,'r'},
@@ -220,14 +257,13 @@ main(int argc,char **argv)
   
   /* Start clock and save command line */ 
   cmd_ln=nco_cmd_ln_sng(argc,argv);
-  clock=time((time_t *)NULL);
-  time_bfr_srt=ctime(&clock); time_bfr_srt=time_bfr_srt; /* Avoid compiler warning until variable is used for something */
+  time_crr_time_t=time((time_t *)NULL);
+  time_bfr_srt=ctime(&time_crr_time_t); time_bfr_srt=time_bfr_srt; /* Avoid compiler warning until variable is used for something */
   
   /* Get program name and set program enum (e.g., prg=ncra) */
   prg_nm=prg_prs(argv[0],&prg);
   
   /* Parse command line arguments */
-  opt_sng="ACcD:d:Fhl:n:Op:Rrs:S:vx-:";
   while((opt = getopt_long(argc,argv,opt_sng,opt_lng,&opt_idx)) != EOF){
     switch(opt){
     case 'A': /* Toggle FORCE_APPEND */
@@ -247,7 +283,10 @@ main(int argc,char **argv)
       lmt_nbr++;
       break;
     case 'F': /* Toggle index convention. Default is 0-based arrays (C-style). */
-      FORTRAN_STYLE=!FORTRAN_STYLE;
+      FORTRAN_IDX_CNV=!FORTRAN_IDX_CNV;
+      break;
+    case 'f': /* Print function table */
+      PRN_FNC_TBL=True;
       break;
     case 'h': /* Toggle appending to history global attribute */
       HISTORY_APPEND=!HISTORY_APPEND;
@@ -258,9 +297,9 @@ main(int argc,char **argv)
     case 'n': /* NINTAP-style abbreviation of files to process */
       /* Currently not used in ncap but should be to allow processing multiple input files by same script */
       (void)fprintf(stderr,"%s: ERROR %s does not currently implement -n option\n",prg_nm_get(),prg_nm_get());
-      fl_lst_abb=lst_prs(optarg,",",&nbr_abb_arg);
-      if(nbr_abb_arg < 1 || nbr_abb_arg > 3){
-	(void)fprintf(stdout,"%s: ERROR Incorrect abbreviation for file list\n",prg_nm);
+      fl_lst_abb=lst_prs(optarg,",",&abb_arg_nbr);
+      if(abb_arg_nbr < 1 || abb_arg_nbr > 3){
+	(void)fprintf(stderr,"%s: ERROR Incorrect abbreviation for file list\n",prg_nm);
 	(void)nco_usg_prn();
 	nco_exit(EXIT_FAILURE);
       } /* end if */
@@ -268,6 +307,9 @@ main(int argc,char **argv)
     case 'O': /* Toggle FORCE_OVERWRITE */
       FORCE_OVERWRITE=!FORCE_OVERWRITE;
       break;
+    case 'o': /* Name of output file */
+      fl_out=(char *)strdup(optarg);
+      break;
     case 'p': /* Common file path */
       fl_pth=optarg;
       break;
@@ -276,8 +318,8 @@ main(int argc,char **argv)
       break;
     case 'r': /* Print CVS program information and copyright notice */
       (void)copyright_prn(CVS_Id,CVS_Revision);
-      (void)nco_lib_vrs_prn();
-      PRINT_FUNCTION_TABLE=True;
+      (void)nco_lbr_vrs_prn();
+      nco_exit(EXIT_SUCCESS);
       break;
     case 's': /* Copy command script for later processing */
       spt_arg[nbr_spt++]=(char *)strdup(optarg);
@@ -293,12 +335,12 @@ main(int argc,char **argv)
       break;
     case 'x': /* Exclude rather than extract variables specified with -v */
       EXCLUDE_INPUT_LIST=True;
-      (void)fprintf(stderr,"%s: ERROR %s does not currently implement -x option\n",prg_nm_get(),prg_nm_get());
+      if(EXCLUDE_INPUT_LIST) (void)fprintf(stderr,"%s: ERROR %s does not currently implement -x option\n",prg_nm_get(),prg_nm_get());
       nco_exit(EXIT_FAILURE);
       break;
     case '?': /* Print proper usage */
       (void)nco_usg_prn();
-      nco_exit(EXIT_FAILURE);
+      nco_exit(EXIT_SUCCESS);
       break;
     case '-': /* Long options are not allowed */
       (void)fprintf(stderr,"%s: ERROR Long options are not available in this build. Use single letter options instead.\n",prg_nm_get());
@@ -329,7 +371,8 @@ main(int argc,char **argv)
   
   /* Create function table */
   sym_tbl_nbr= /* fxm: Make this dynamic */
-    +10 /* Basic math: acos, asin, atan, cos, exp, log, log10, sin, sqrt, tan */
+    +11 /* Basic math: acos, asin, atan, cos, exp, fabs, log, log10, sin, sqrt, tan */
+    +1 /* Basic math synonyms: ln */
     +6 /* Hyperbolic trigonometric: acosh, asinh, atanh, cosh, sinh, tanh */
     +2 /* Basic Rounding: ceil, floor */
     +4 /* Advanced Rounding: nearbyint, rint, round, trunc */
@@ -343,11 +386,15 @@ main(int argc,char **argv)
   sym_tbl[sym_idx++]=ncap_sym_init("atan",atan,atanf);
   sym_tbl[sym_idx++]=ncap_sym_init("cos",cos,cosf);  
   sym_tbl[sym_idx++]=ncap_sym_init("exp",exp,expf);
+  sym_tbl[sym_idx++]=ncap_sym_init("fabs",fabs,fabsf);
   sym_tbl[sym_idx++]=ncap_sym_init("log",log,logf);
   sym_tbl[sym_idx++]=ncap_sym_init("log10",log10,log10f);
   sym_tbl[sym_idx++]=ncap_sym_init("sin",sin,sinf);
   sym_tbl[sym_idx++]=ncap_sym_init("sqrt",sqrt,sqrtf);
   sym_tbl[sym_idx++]=ncap_sym_init("tan",tan,tanf);
+
+  /* Basic math synonyms: ln */
+  sym_tbl[sym_idx++]=ncap_sym_init("ln",log,logf); /* ln() is synonym for log() */
   
   /* Basic Rounding: ceil, floor */
   sym_tbl[sym_idx++]=ncap_sym_init("ceil",ceil,ceilf); /* Round up to nearest integer */
@@ -388,28 +435,23 @@ main(int argc,char **argv)
 #endif /* not AIX || CRAY || SGI* || WIN32 */
   assert(sym_idx == sym_tbl_nbr);
   
-  
-  if(PRINT_FUNCTION_TABLE){
-    
-    printf("\n  Available Maths functions:\n");
-    printf("  ----------------------\n");
-    printf("  NAME    FLOAT   DOUBLE\n");          
-    for(idx = 0;idx < sym_tbl_nbr; idx++)
-      printf("  %-7s %-7c %-7c\n", sym_tbl[idx]->nm, (sym_tbl[idx]->fnc_flt ? 'y' : 'n'), (sym_tbl[idx]->fnc_dbl ? 'y' : 'n'));
-    printf("  ----------------------\n");
+  if(PRN_FNC_TBL){
+    /* ncap TODO #43: alphabetize this list */ 
+    (void)fprintf(stdout,"Maths functions available in %s:\n",prg_nm_get());
+    (void)fprintf(stdout,"Name\tFloat\tDouble\n");          
+    for(idx=0;idx<sym_tbl_nbr;idx++)
+      (void)fprintf(stdout,"%s\t%c\t%c\n",sym_tbl[idx]->nm,(sym_tbl[idx]->fnc_flt ? 'y' : 'n'),(sym_tbl[idx]->fnc_dbl ? 'y' : 'n'));
     nco_exit(EXIT_SUCCESS);
-    
-  }
-  
+  } /* end if PRN_FNC_TBL */
   
   /* Process positional arguments and fill in filenames */
-  fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&nbr_fl,&fl_out);
+  fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&fl_nbr,&fl_out,&FL_LST_IN_FROM_STDIN);
   
   /* Make uniform list of user-specified dimension limits */
   if(lmt_nbr > 0) lmt=nco_lmt_prs(lmt_nbr,lmt_arg);
   
   /* Parse filename */
-  fl_in=nco_fl_nm_prs(fl_in,0,&nbr_fl,fl_lst_in,nbr_abb_arg,fl_lst_abb,fl_pth);
+  fl_in=nco_fl_nm_prs(fl_in,0,&fl_nbr,fl_lst_in,abb_arg_nbr,fl_lst_abb,fl_pth);
   /* Make sure file is on local system and is readable or die trying */
   fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
   /* Open file for reading */
@@ -448,13 +490,13 @@ main(int argc,char **argv)
   prs_arg.nbr_dmn_out=&nbr_dmn_out; /* number of dims in above list */
   prs_arg.sym_tbl=sym_tbl; /* [fnc] Symbol table for functions */
   prs_arg.sym_tbl_nbr=sym_tbl_nbr; /* [nbr] Number of functions in table */
-  prs_arg.ntl_scn=False; /* No longer do an initial scan */
+  prs_arg.ntl_scn=False; /* [flg] Initial scan of script */
   prs_arg.var_LHS=NULL; /* [var] LHS cast variable */
   prs_arg.nco_op_typ=nco_op_nil; /* [enm] Operation type */
   
   /* Initialize line counter */
-  ln_nbr_crr=(size_t *)nco_realloc(ln_nbr_crr,ncl_dpt_crr+1UL); 
-  ln_nbr_crr[ncl_dpt_crr]=1UL; /* [cnt] Line number incremented in ncap.l */
+  ncap_ln_nbr_crr=(size_t *)nco_realloc(ncap_ln_nbr_crr,ncap_ncl_dpt_crr+1UL); 
+  ncap_ln_nbr_crr[ncap_ncl_dpt_crr]=1UL; /* [cnt] Line number incremented in ncap.l */
   if(fl_spt_usr == NULL){
     /* No script file specified, look for command-line scripts */
     if(nbr_spt == 0){
@@ -463,28 +505,26 @@ main(int argc,char **argv)
       nco_exit(EXIT_FAILURE);
     } /* end if */
     
-    /* Print all command line scripts */
+    /* Print all command-line scripts */
     if(dbg_lvl_get() > 0){
       for(idx=0;idx<nbr_spt;idx++) (void)fprintf(stderr,"spt_arg[%d] = %s\n",idx,spt_arg[idx]);
     } /* endif debug */
     
-    /* Run parser on command line scripts */
+    /* Parse command line scripts */
     fl_spt_usr=(char *)strdup("Command-line script");
     yy_scan_string(spt_arg_cat);
     
   }else{ /* ...endif command-line scripts, begin script file... */
-    
     /* Open script file for reading */
     if((yyin=fopen(fl_spt_usr,"r")) == NULL){
       (void)fprintf(stderr,"%s: ERROR Unable to open script file %s\n",prg_nm_get(),fl_spt_usr);
       nco_exit(EXIT_FAILURE);
     } /* end if */
-    
-  } /* end else */
+  } /* end else script file */
   
   /* Copy script file name to global variable */
-  fl_spt_glb=(char **)nco_realloc(fl_spt_glb,ncl_dpt_crr+1UL); 
-  fl_spt_glb[ncl_dpt_crr]=fl_spt_usr;
+  ncap_fl_spt_glb=(char **)nco_realloc(ncap_fl_spt_glb,ncap_ncl_dpt_crr+1UL); 
+  ncap_fl_spt_glb[ncap_ncl_dpt_crr]=fl_spt_usr;
   
   /* Invoke parser */
   rcd=yyparse((void *)&prs_arg);
@@ -504,12 +544,12 @@ main(int argc,char **argv)
   } /* endif */
   
   if(!PROCESS_ALL_VARS){
-    /* Make list of vars of new attributes whose parent var is ONLY in input file */
+    /* Make list of variables of new attributes whose parent variable is ONLY in input file */
     xtr_lst=nco_att_lst_mk(in_id,out_id,att_lst,nbr_att,&nbr_xtr);
   } /* endif */
-    /* Find dims associated with xtr_lst */
+    /* Find dimensions associated with xtr_lst */
     /* Write to O only new dims
-       Add apropriate co-ordinate vars to extraction list 
+       Add apropriate coordinate variables to extraction list 
        options -c      -process all cordinates 
        i.e add  co-ords to var list 
        Also add their dims
@@ -519,53 +559,53 @@ main(int argc,char **argv)
        
        options -C         no co-ordinates   Do nothing */
   
-    /* Subtract list a again */
-    /* Finally extract vars on list */
+    /* Subtract list A again */
+    /* Finally extract variables on list */
   
-  /* Subtract list a */
-  if(nbr_lst_a > 0) xtr_lst=nco_var_lst_sub(xtr_lst, &nbr_xtr,xtr_lst_a,nbr_lst_a);
+  /* Subtract list A */
+  if(nbr_lst_a > 0) xtr_lst=nco_var_lst_sub(xtr_lst,&nbr_xtr,xtr_lst_a,nbr_lst_a);
   
   (void)nco_redef(out_id);
   
-  /* Make list of dims of vars in xtr_lst */
+  /* Make list of dimensions of variables in xtr_lst */
   if(nbr_xtr > 0) dmn_lst=nco_dmn_lst_ass_var(in_id,xtr_lst,nbr_xtr,&nbr_dmn_ass);
   
-  /* Find and add any new dims to output */
+  /* Find and add any new dimensions to output */
   for(idx=0;idx<nbr_dmn_ass;idx++)
     for(jdx=0;jdx<nbr_dmn_in;jdx++){
-      /* if dimension in list and it hasn't been defined yet */
-      if(!strcmp(dmn_lst[idx].nm, dmn_in[jdx]->nm) && !dmn_in[jdx]->xrf){     
-	/* add dim to output list dmn_prc */
+      /* If dimension is in list and is not yet defined */
+      if(!strcmp(dmn_lst[idx].nm,dmn_in[jdx]->nm) && !dmn_in[jdx]->xrf){     
+	/* Add dimension to output list dmn_prc */
 	dmn_new=nco_dmn_out_grow((void *)&prs_arg);
 	*dmn_new=nco_dmn_dpl(dmn_in[jdx]);
 	(void)nco_dmn_xrf(*dmn_new,dmn_in[jdx]);
-	/* write dim to output */
+	/* Write dimension to output */
 	(void)nco_dmn_dfn(fl_out,out_id,dmn_new,1);
 	break;
       } /* endif */
     } /* end loop over jdx */
   
-  /* All dims for all vars are now in output */
-  /* Need to add co-ordinate vars to extraction list */
-  /* if PROCESS_ALL_COORDINATES then the associated DIM needs to written to output */
+  /* All dimensions for all variables are now in output
+     Add coordinate variables to extraction list
+     If PROCESS_ALL_COORDINATES then write associated dimension to output */
   if(PROCESS_ASSOCIATED_COORDINATES){
     for(idx=0; idx <nbr_dmn_in; idx++){
       if(!dmn_in[idx]->is_crd_dmn) continue;
       
       if(PROCESS_ALL_COORDINATES && !dmn_in[idx]->xrf){
-	/* Add dim to output list dmn_out */
+	/* Add dimensions to output list dmn_out */
 	dmn_new=nco_dmn_out_grow((void *)&prs_arg);
 	*dmn_new=nco_dmn_dpl(dmn_in[idx]);
 	(void)nco_dmn_xrf(*dmn_new,dmn_in[idx]);
-	/* write dim to output */
+	/* Write dimension to output */
 	(void)nco_dmn_dfn(fl_out,out_id,dmn_new,1);
       } /* end if */
-      /* Add co-ordinate var to extraction list, dim has already been output */
+      /* Add coordinate variable to extraction list, dimension has already been output */
       if(dmn_in[idx]->xrf){
 	for(jdx=0;jdx<nbr_xtr;jdx++)
 	  if(!strcmp(xtr_lst[jdx].nm,dmn_in[idx]->nm)) break;
 	if(jdx != nbr_xtr) continue;
-	/* If co-ord is not on list then add it to extraction list */
+	/* If coordinate is not on list then add it to extraction list */
 	xtr_lst=(nm_id_sct *)nco_realloc(xtr_lst,(nbr_xtr+1)*sizeof(nm_id_sct));     
 	xtr_lst[nbr_xtr].nm=(char *)strdup(dmn_in[idx]->nm);
 	xtr_lst[nbr_xtr++].id=dmn_in[idx]->cid;
@@ -573,14 +613,14 @@ main(int argc,char **argv)
     } /* end loop over idx */	      
   } /* end if */ 
   
-    /* Subtract list a again (it may contain re-defined co-ordinates) */
+  /* Subtract list A again (it may contain re-defined coordinates) */
   if(nbr_xtr >0) xtr_lst=nco_var_lst_sub(xtr_lst, &nbr_xtr,xtr_lst_a,nbr_lst_a);
   
   /* Sort extraction list for faster I/O */
-  if(nbr_xtr > 1) xtr_lst=nco_lst_srt(xtr_lst,nbr_xtr,False);
+  if(nbr_xtr > 1) xtr_lst=nco_lst_srt_nm_id(xtr_lst,nbr_xtr,False);
   
-  /* Is this an NCAR CSM-format history tape? */
-  NCAR_CSM_FORMAT=nco_ncar_csm_inq(in_id);
+  /* Is this an NCAR CCSM-format history tape? */
+  NCAR_CCSM_FORMAT=nco_ncar_csm_inq(in_id);
   
   /* Write "fixed" variables */
   var=(var_sct **)nco_malloc(nbr_xtr*sizeof(var_sct *));
@@ -592,13 +632,13 @@ main(int argc,char **argv)
     (void)nco_xrf_dmn(var_out[idx]);
   } /* end loop over idx */
   
-  /* NB: ncap is not suited for nco_var_lst_dvd() */
+  /* NB: ncap is not well-suited for nco_var_lst_dvd() */
   /* Divide variable lists into lists of fixed variables and variables to be processed */
-  (void)nco_var_lst_dvd(var,var_out,nbr_xtr,NCAR_CSM_FORMAT,(dmn_sct **)NULL,(int)0,&var_fix,&var_fix_out,&nbr_var_fix,&var_prc,&var_prc_out,&nbr_var_prc);
+  (void)nco_var_lst_dvd(var,var_out,nbr_xtr,NCAR_CCSM_FORMAT,nco_pck_plc_nil,nco_pck_map_nil,(dmn_sct **)NULL,(int)0,&var_fix,&var_fix_out,&nbr_var_fix,&var_prc,&var_prc_out,&nbr_var_prc);
   
-  /* Why not call this with var_fix */
+  /* Why not call this with var_fix? */
   /* Define non-processed vars */
-  (void)nco_var_dfn(in_id,fl_out,out_id,var_out,nbr_xtr,(dmn_sct **)NULL,(int)0);
+  (void)nco_var_dfn(in_id,fl_out,out_id,var_out,nbr_xtr,(dmn_sct **)NULL,(int)0,nco_pck_plc_nil,nco_pck_map_nil);
   
   (void)nco_enddef(out_id);
   
