@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncap_utl.c,v 1.83 2003/01/20 17:46:02 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncap_utl.c,v 1.103 2004/09/03 23:06:46 zender Exp $ */
 
 /* Purpose: netCDF arithmetic processor */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include "ncap.h" /* netCDF arithmetic processor */
@@ -13,8 +13,10 @@ ncap_var_init(const char * const var_nm,prs_sct *prs_arg)
 {
   /* Purpose: Initialize variable structure, retrieve variable values from disk
      Parser calls ncap_var_init() when it encounters a new RHS variable */
-  int i;
-  int j;
+  /* const char fnc_nm[]="ncap_var_init()"; *//* [sng] Function name */
+
+  int idx;
+  int jdx;
   int nbr_dmn_var;
   int *dim_id=NULL;
   int var_id;
@@ -34,13 +36,14 @@ ncap_var_init(const char * const var_nm,prs_sct *prs_arg)
     /* Check input file for ID */
     rcd=nco_inq_varid_flg(prs_arg->in_id,var_nm,&var_id);
     if(rcd != NC_NOERR){
-      /* return null if var not in input or output file */
+      /* Return NULL if variable not in input or output file */
       (void)fprintf(stderr,"WARNING unable to find %s in %s or %s\n",var_nm,prs_arg->fl_in,prs_arg->fl_out);    
       return (var_sct *)NULL;
     } /* end if */
     
-    /* Find dimensions used in var and add any new ones not already in
-       output list prs_arg->dmn_out and output file */
+    /* Find dimensions used in var
+       Learn which are not already in output list prs_arg->dmn_out and output file
+       Add these to output list and output file */
     (void)nco_redef(prs_arg->out_id);
     fl_id=prs_arg->in_id;
     
@@ -49,39 +52,42 @@ ncap_var_init(const char * const var_nm,prs_sct *prs_arg)
       dim_id=(int *)nco_malloc(nbr_dmn_var*sizeof(int));
       
       (void)nco_inq_vardimid(fl_id,var_id,dim_id);
-      for(i=0;i<nbr_dmn_var;i++) 
-	for(j=0;j<prs_arg->nbr_dmn_in;j++){
+      for(idx=0;idx<nbr_dmn_var;idx++) 
+	for(jdx=0;jdx<prs_arg->nbr_dmn_in;jdx++){
 	  
-	  /* de-reference */
-	  dmn_in=prs_arg->dmn_in[j];
-	  if(dim_id[i] != dmn_in->id || dmn_in->xrf) continue;
+	  /* De-reference */
+	  dmn_in=prs_arg->dmn_in[jdx];
+	  if(dim_id[idx] != dmn_in->id || dmn_in->xrf) continue;
 	  
-	  /* define dimension  in (prs_arg->dmn_out) */ 
+	  /* Define new dimension in (prs_arg->dmn_out) */ 
 	  dim_new=nco_dmn_out_grow(prs_arg);
 	  *dim_new=nco_dmn_dpl(dmn_in);
 	  (void)nco_dmn_xrf(*dim_new,dmn_in);
-	  /* write dimension to output file */
+	  /* Write new dimension to output file */
 	  (void)nco_dmn_dfn(prs_arg->fl_out,prs_arg->out_id,dim_new,1);
-	  /* printf("Dimension %s defined\n",(*dim_new)->nm); */
+	  if(dbg_lvl_get() > 2) (void)fprintf(stderr,"%s: DEBUG Found new dimension %s in input variable %s in file %s. Defining dimension %s in output file %s\n",prg_nm_get(),(*dim_new)->nm,var_nm,prs_arg->fl_in,(*dim_new)->nm,prs_arg->fl_out);
 	  break;
-	} /* end for j */
+	} /* end loop over dimensions in current output dimension list */
       (void)nco_free(dim_id);
-    } 
+    } /* end loop over dimension in current input variable */
     (void)nco_enddef(prs_arg->out_id); 
     
   } /* end else */
   
   if(dbg_lvl_get() > 2) (void)fprintf(stderr,"%s: parser VAR action called ncap_var_init() to retrieve %s from disk\n",prg_nm_get(),var_nm);
   var=nco_var_fll(fl_id,var_id,var_nm,*(prs_arg->dmn_out),*(prs_arg->nbr_dmn_out));
-  var->nm=(char *)nco_malloc((strlen(var_nm)+1UL)*sizeof(char));
-  (void)strcpy(var->nm,var_nm);
-  var->tally=(long *)nco_malloc(var->sz*sizeof(long));
-  (void)nco_zero_long(var->sz,var->tally);
-  var->val.vp=(void *)nco_malloc(var->sz*nco_typ_lng(var->type));
+  /*  var->nm=(char *)nco_malloc((strlen(var_nm)+1UL)*sizeof(char));
+  (void)strcpy(var->nm,var_nm); */
+
+  /* Tally is not required yet since ncap does not perform cross-file operations (yet) */
+  /* var->tally=(long *)nco_malloc_dbg(var->sz*sizeof(long),"Unable to malloc() tally buffer in variable initialization",fnc_nm);
+      (void)nco_zero_long(var->sz,var->tally); */
+  var->tally=(long *)NULL;
+
   /* Retrieve variable values from disk into memory */
   (void)nco_var_get(fl_id,var);
   /* (void)nco_var_free(var_nm);*/
-  /* free(var_nm->nm);*/
+  /* (void)nco_free(var_nm->nm);*/
   /* var=nco_var_upk(var); */
   return var;
 } /* end ncap_var_init() */
@@ -105,6 +111,9 @@ ncap_var_write
  prs_sct *prs_arg)
 {
   /* Purpose: Define variable in output file and write variable */
+  const char mss_val_sng[]="missing_value"; /* [sng] Unidata standard string for missing value */
+  const char add_fst_sng[]="add_offset"; /* [sng] Unidata standard string for add offset */
+  const char scl_fct_sng[]="scale_factor"; /* [sng] Unidata standard string for scale factor */
   int var_out_id;
   
   (void)nco_redef(prs_arg->out_id);
@@ -112,12 +121,12 @@ ncap_var_write
   /* Define variable */   
   (void)nco_def_var(prs_arg->out_id,var->nm,var->type,var->nbr_dim,var->dmn_id,&var_out_id);
   /* Put missing value */  
-  if(var->has_mss_val) (void)nco_put_att(prs_arg->out_id,var_out_id,"missing_value",var->type,1,var->mss_val.vp);
+  if(var->has_mss_val) (void)nco_put_att(prs_arg->out_id,var_out_id,mss_val_sng,var->type,1,var->mss_val.vp);
   
   /* Write/overwrite scale_factor and add_offset attributes */
   if(var->pck_ram){ /* Variable is packed in memory */
-    if(var->has_scl_fct) (void)nco_put_att(prs_arg->out_id,var_out_id,"scale_factor",var->typ_upk,1,var->scl_fct.vp);
-    if(var->has_add_fst) (void)nco_put_att(prs_arg->out_id,var_out_id,"add_offset",var->typ_upk,1,var->add_fst.vp);
+    if(var->has_scl_fct) (void)nco_put_att(prs_arg->out_id,var_out_id,scl_fct_sng,var->typ_upk,1,var->scl_fct.vp);
+    if(var->has_add_fst) (void)nco_put_att(prs_arg->out_id,var_out_id,add_fst_sng,var->typ_upk,1,var->add_fst.vp);
   } /* endif pck_ram */
   (void)nco_enddef(prs_arg->out_id);
   
@@ -133,7 +142,7 @@ ncap_var_write
 
 sym_sct *
 ncap_sym_init
-(char *sym_nm,
+(const char * const sym_nm,
  double (*fnc_dbl)(double),
  float (*fnc_flt)(float))
 { 
@@ -177,38 +186,34 @@ ncap_var_var_add /* [fnc] Add two variables */
  var_sct *var_2) /* I [sct] Input variable structure containing second operand */
 {
   /* Purpose: Add two variables */
-  var_sct *var_nsw;
+  /* Store result in var_2 */
   
   (void)ncap_var_retype(var_1,var_2);
-  var_nsw=nco_var_dpl(var_2);
-  (void)ncap_var_cnf_dmn(&var_1,&var_nsw);
-  /* fxm: bug in nco_var_add. missing_value is not carried over to var_nsw in result when var_1->has_mss_val is true */
+  (void)ncap_var_cnf_dmn(&var_1,&var_2);
+  /* fxm: bug in nco_var_add()? missing_value is not carried over to var_2 in result when var_1->has_mss_val is true */
   if(var_1->has_mss_val){
-    (void)nco_var_add_no_tally(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_nsw->val);
+    (void)nco_var_add(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
   }else{
-    (void)nco_var_add_no_tally(var_1->type,var_1->sz,var_nsw->has_mss_val,var_nsw->mss_val,var_1->val,var_nsw->val);
+    (void)nco_var_add(var_1->type,var_1->sz,var_2->has_mss_val,var_2->mss_val,var_1->val,var_2->val);
   } /* end if */
-  return var_nsw;
+  return var_2;
 } /* end ncap_var_var_add() */
 
-var_sct * /* O [frc] Remainder of input variables (var_2-var_1) */
-ncap_var_var_sub /* [fnc] Subtract two variables (var_2-var_1) */ 
-(var_sct *var_2, /* I [sct] Variable structure containing second operand */ /* fxm TODO: 19 non-standard argument order */
- var_sct *var_1) /* I [sct] Variable structure containing first operand */
+var_sct * /* O [sct] Quotient of input variables (var_2/var_1) */
+ncap_var_var_dvd /* [fnc] Divide two variables (var_2/var_1) */ 
+(var_sct *var_1, /* I [sct] Variable structure containing denominator */
+ var_sct *var_2) /* I [sct] Variable structure containing numerator */
 {
-  /* Purpose: Subtract a variable from another variable */
-  var_sct *var_nsw;
-  
+  /* Purpose: Divide two variables (var_2/var_1) */
   (void)ncap_var_retype(var_1,var_2);
-  var_nsw=nco_var_dpl(var_2);
-  (void)ncap_var_cnf_dmn(&var_1,&var_nsw);
+  (void)ncap_var_cnf_dmn(&var_1,&var_2);
   if(var_1->has_mss_val){
-    (void)nco_var_sbt(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_nsw->val);
+    (void)nco_var_dvd(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
   }else{
-    (void)nco_var_sbt(var_1->type,var_1->sz,var_nsw->has_mss_val,var_nsw->mss_val,var_1->val,var_nsw->val);
-  }/* end else */
-  return var_nsw;
-} /* end ncap_var_var_sub() */
+    (void)nco_var_dvd(var_1->type,var_1->sz,var_2->has_mss_val,var_2->mss_val,var_1->val,var_2->val);
+  } /* end else */
+  return var_2;
+} /* end ncap_var_var_dvd() */
 
 var_sct * /* O [sct] Product of input variables (var_1*var_2) */
 ncap_var_var_mlt /* [fnc] Multiply two variables */ 
@@ -216,35 +221,64 @@ ncap_var_var_mlt /* [fnc] Multiply two variables */
  var_sct *var_2) /* I [sct] Variable structure containing second operand */
 {
   /* Purpose: Multiply two variables variables (var_1*var_2) */
-  var_sct *var_nsw;
   (void)ncap_var_retype(var_1,var_2);
-  var_nsw=nco_var_dpl(var_2);
-  (void)ncap_var_cnf_dmn(&var_1,&var_nsw);
+  (void)ncap_var_cnf_dmn(&var_1,&var_2);
   if(var_1->has_mss_val){
-    (void)nco_var_mlt(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_nsw->val);
+    (void)nco_var_mlt(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
   }else{
-    (void)nco_var_mlt(var_1->type,var_1->sz,var_nsw->has_mss_val,var_nsw->mss_val,var_1->val,var_nsw->val);
-  }
-   return var_nsw;
+    (void)nco_var_mlt(var_1->type,var_1->sz,var_2->has_mss_val,var_2->mss_val,var_1->val,var_2->val);
+  } /* end else */
+   return var_2;
 } /* end ncap_var_var_mlt() */
 
-var_sct * /* O [sct] Quotient of input variables (var_2/var_1) */
-ncap_var_var_dvd /* [fnc] Divide two variables (var_2/var_1) */ 
-(var_sct *var_1, /* I [sct] Variable structure containing first operand */
- var_sct *var_2) /* I [sct] Variable structure containing second operand */
+var_sct * /* O [sct] Remainder of modulo operation of input variables (var_1%var_2) */
+ncap_var_var_mod /* [fnc] Remainder (modulo) operation of two variables */
+(var_sct *var_1, /* I [sct] Variable structure containing field */
+ var_sct *var_2) /* I [sct] Variable structure containing divisor */
 {
-  /* Purpose: Divide two variables (var_2/var_1) */
-  var_sct *var_nsw;
+  /* Purpose: Remainder (modulo) operation of two variables (var_1%var_2) */
   (void)ncap_var_retype(var_1,var_2);
-  var_nsw=nco_var_dpl(var_2);
-  (void)ncap_var_cnf_dmn(&var_1,&var_nsw);
+  (void)ncap_var_cnf_dmn(&var_1,&var_2);
   if(var_1->has_mss_val){
-    (void)nco_var_dvd(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_nsw->val);
+    (void)nco_var_mod(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
   }else{
-    (void)nco_var_dvd(var_1->type,var_1->sz,var_nsw->has_mss_val,var_nsw->mss_val,var_1->val,var_nsw->val);
-  } /* end else */ 
-  return var_nsw;
-} /* end ncap_var_var_dvd() */
+    (void)nco_var_mod(var_1->type,var_1->sz,var_2->has_mss_val,var_2->mss_val,var_1->val,var_2->val);
+  } /* end else */
+   return var_2;
+} /* end ncap_var_var_mod() */
+
+var_sct * /* O [sct] Empowerment of input variables (var_1^var_2) */
+ncap_var_var_pwr /* [fnc] Empowerment of two variables */ 
+(var_sct *var_1, /* I [sct] Variable structure containing base */
+ var_sct *var_2) /* I [sct] Variable structure containing exponent */
+{
+  /* Purpose: Empower two variables (var_1^var_2) */
+  (void)ncap_var_retype(var_1,var_2);
+  (void)ncap_var_cnf_dmn(&var_1,&var_2);
+  if(var_1->has_mss_val){
+    (void)nco_var_pwr(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
+  }else{
+    (void)nco_var_pwr(var_1->type,var_1->sz,var_2->has_mss_val,var_2->mss_val,var_1->val,var_2->val);
+  } /* end else */
+   return var_2;
+} /* end ncap_var_var_pwr() */
+
+var_sct * /* O [frc] Remainder of input variables (var_2-var_1) */
+ncap_var_var_sub /* [fnc] Subtract two variables (var_2-var_1) */ 
+(var_sct *var_2, /* I [sct] Variable structure containing second operand */ /* fxm TODO: 19 non-standard argument order */
+ var_sct *var_1) /* I [sct] Variable structure containing first operand */
+{
+  /* Purpose: Subtract a variable from another variable */
+  
+  (void)ncap_var_retype(var_1,var_2);
+  (void)ncap_var_cnf_dmn(&var_1,&var_2);
+  if(var_1->has_mss_val){
+    (void)nco_var_sbt(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
+  }else{
+    (void)nco_var_sbt(var_1->type,var_1->sz,var_2->has_mss_val,var_2->mss_val,var_1->val,var_2->val);
+  }/* end else */
+  return var_2;
+} /* end ncap_var_var_sub() */
 
 var_sct *
 ncap_var_fnc(var_sct *var_in,sym_sct *app)
@@ -254,23 +288,21 @@ ncap_var_fnc(var_sct *var_in,sym_sct *app)
   long idx;
   long sz;
   ptr_unn op1;
-  var_sct *var;
   
   /* Promote variable to NC_FLOAT */
   if(var_in->type < NC_FLOAT) var_in=nco_var_cnf_typ((nc_type)NC_FLOAT,var_in);
-  var=nco_var_dpl(var_in);
   
-  op1=var->val;
-  sz=var->sz;
-  (void)cast_void_nctype(var->type,&op1);
-  if(var->has_mss_val) (void)cast_void_nctype(var->type,&(var->mss_val));
+  op1=var_in->val;
+  sz=var_in->sz;
+  (void)cast_void_nctype(var_in->type,&op1);
+  if(var_in->has_mss_val) (void)cast_void_nctype(var_in->type,&(var_in->mss_val));
   
-  switch(var->type){ 
+  switch(var_in->type){ 
   case NC_DOUBLE: {
-    if(!var->has_mss_val){
+    if(!var_in->has_mss_val){
       for(idx=0;idx<sz;idx++) op1.dp[idx]=(*(app->fnc_dbl))(op1.dp[idx]);
     }else{
-      double mss_val_dbl=*(var->mss_val.dp); /* Temporary variable reduces dereferencing */
+      double mss_val_dbl=*(var_in->mss_val.dp); /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
         if(op1.dp[idx] != mss_val_dbl) op1.dp[idx]=(*(app->fnc_dbl))(op1.dp[idx]);
       } /* end for */
@@ -278,10 +310,10 @@ ncap_var_fnc(var_sct *var_in,sym_sct *app)
     break;
   }
   case NC_FLOAT: {
-    if(!var->has_mss_val){
+    if(!var_in->has_mss_val){
       for(idx=0;idx<sz;idx++) op1.fp[idx]=(*(app->fnc_flt))(op1.fp[idx]);
     }else{
-      float mss_val_flt=*(var->mss_val.fp); /* Temporary variable reduces dereferencing */
+      float mss_val_flt=*(var_in->mss_val.fp); /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
         if(op1.fp[idx] != mss_val_flt) op1.fp[idx]=(*(app->fnc_flt))(op1.fp[idx]);
       } /* end for */
@@ -291,143 +323,123 @@ ncap_var_fnc(var_sct *var_in,sym_sct *app)
   default: nco_dfl_case_nc_type_err(); break;
   }/* end switch */
   
-  if(var->has_mss_val) (void)cast_nctype_void(var->type,&(var->mss_val));
-  return var;
+  if(var_in->has_mss_val) (void)cast_nctype_void(var_in->type,&(var_in->mss_val));
+  return var_in;
 } /* end ncap_var_fnc() */
 
 var_sct *
 ncap_var_scv_add(var_sct *var,scv_sct scv)
 {
   /* Purpose: add the value in scv to each element of var */
-  var_sct *var_nsw;
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
-  var_nsw=nco_var_dpl(var);
-  (void)var_scv_add(var->type,var->sz,var->has_mss_val,var->mss_val,var_nsw->val,&scv);
-  
-  return var_nsw;
+  (void)var_scv_add(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
+  return var;
 } /* end ncap_var_scv_add() */
 
 var_sct *
 ncap_var_scv_sub(var_sct *var,scv_sct scv)
 {
   /* Purpose: Subtract value in scv from each element of var */
-  var_sct *var_nsw;
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
-  var_nsw=nco_var_dpl(var);
   (void)ncap_scv_minus(&scv);
-  (void)var_scv_add(var->type,var->sz,var->has_mss_val,var->mss_val,var_nsw->val,&scv);
+  (void)var_scv_add(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
   
-  return var_nsw;
+  return var;
 } /* end ncap_var_scv_sub() */
 
 var_sct *
 ncap_var_scv_mlt(var_sct *var,scv_sct scv)
 {
   /* Purpose: Multiply variable by value in scv */
-  var_sct *var_nsw;
-  /* fxm: 20020421 Old method was to always convert scv to var precision 
-     (void)scv_conform_type(var->type,&scv); */
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
-  var_nsw=nco_var_dpl(var);
-  (void)var_scv_mlt(var->type,var->sz,var->has_mss_val,var->mss_val,var_nsw->val,&scv);
-  
-  return var_nsw;
+  (void)var_scv_mlt(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
+  return var;
 } /* end ncap_var_scv_mlt */
 
 var_sct *
 ncap_var_scv_dvd(var_sct *var,scv_sct scv)
 {
   /* Purpose: Divide each element of var by value in scv */
-  var_sct *var_nsw;
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
-  var_nsw=nco_var_dpl(var);
-  (void)var_scv_dvd(var->type,var->sz,var->has_mss_val,var->mss_val,var_nsw->val,&scv);
-  
-  return var_nsw;
+  (void)var_scv_dvd(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
+  return var;
 } /* end ncap_var_scv_dvd */
 
+var_sct *
+ncap_scv_var_dvd(scv_sct scv,var_sct *var)
+{
+  /* Purpose: Divide scv by value of each element in var */
+  (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
+  (void)scv_var_dvd(var->type,var->sz,var->has_mss_val,var->mss_val,&scv,var->val);
+  return var;
+} /* end ncap_scv_var_dvd */
+
 var_sct *
 ncap_var_scv_mod(var_sct *var,scv_sct scv)
 {
-  /* Purpose: var % scv, take modulus of each element of var with value in scv */
-  
-  var_sct *var_nsw;
+  /* Purpose: Modulus of each element of var with scv */
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
-  var_nsw=nco_var_dpl(var);
-  (void)var_scv_mod(var->type,var->sz,var->has_mss_val,var->mss_val,var_nsw->val,&scv);
-  
-  return var_nsw;
+  (void)var_scv_mod(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
+  return var;
 } /* ncap_var_scv_mod */
 
+var_sct *
+ncap_scv_var_mod(scv_sct scv,var_sct *var)
+{
+  /* Purpose: Modulus of scv with each element of var */
+  (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
+  (void)scv_var_mod(var->type,var->sz,var->has_mss_val,var->mss_val,&scv,var->val);
+  return var;
+} /* ncap_var_scv_mod */
+
+var_sct *
+ncap_var_scv_pwr(var_sct *var,scv_sct scv)
+{
+  /* Purpose: Empower each element in var by scv */
+  /* Promote scv and var to NC_FLOAT if necessary since C has no integer empowerment 
+     This reduces type conversion warnings (it is not done to avoid overflow) */
+  if(var->type < NC_FLOAT) var=nco_var_cnf_typ((nc_type)NC_FLOAT,var);
+  (void)nco_scv_cnf_typ(var->type,&scv);
+  (void)var_scv_pwr(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
+  return var;
+} /* end ncap_var_scv_pwr */
+
+var_sct *
+ncap_scv_var_pwr(scv_sct scv,var_sct *var)
+{
+  /* Purpose: Empower each element in scv by var */
+  /* Promote scv and var to NC_FLOAT if necessary since C has no integer empowerment 
+     This reduces type conversion warnings (it is not done to avoid overflow) */
+  if(var->type < NC_FLOAT) var=nco_var_cnf_typ((nc_type)NC_FLOAT,var);
+  (void)nco_scv_cnf_typ(var->type,&scv);
+  (void)scv_var_pwr(var->type,var->sz,var->has_mss_val,var->mss_val,&scv,var->val);
+  return var;
+} /* end ncap_var_scv_pwr */
+
 var_sct *
 ncap_var_abs(var_sct *var)
 {
   /* Purpose: Find absolute value of each element of var */
-  var_sct *var_nsw;
-  var_nsw=nco_var_dpl(var);
-  (void)nco_var_abs(var->type,var->sz,var->has_mss_val,var->mss_val,var_nsw->val);
-  return var_nsw;
+  (void)nco_var_abs(var->type,var->sz,var->has_mss_val,var->mss_val,var->val);
+  return var;
 } /* end ncap_var_abs */
 
-var_sct *
-ncap_var_scv_pwr(var_sct *var_in,scv_sct scv)
-{
-  /* Purpose: Raise var to power in scv
-     All values converted to type double before operation */
-  long idx;
-  long sz;
-  ptr_unn op1;
-  var_sct *var;
-  
-  /* Promote scv and var to NC_FLOAT */
-  if(var_in->type < NC_FLOAT) var_in=nco_var_cnf_typ((nc_type)NC_FLOAT,var_in);
-  var=nco_var_dpl(var_in);
-  (void)scv_conform_type(var->type,&scv);
-  
-  op1=var->val;
-  sz=var->sz;
-  (void)cast_void_nctype(var->type,&op1);
-  if(var->has_mss_val) (void)cast_void_nctype(var->type,&(var->mss_val));
-  
-  switch(var->type){ 
-  case NC_DOUBLE: {
-    double scv_dpl=scv.val.d;
-    if(!var->has_mss_val){
-      for(idx=0;idx<sz;idx++) op1.dp[idx]=pow(op1.dp[idx],scv_dpl);
-    }else{
-      double mss_val_dbl=*(var->mss_val.dp); /* Temporary variable reduces dereferencing */
-      for(idx=0;idx<sz;idx++){
-        if(op1.dp[idx] != mss_val_dbl) op1.dp[idx]=pow(op1.dp[idx],scv_dpl);
-      } /* end for */
-    } /* end else */
-    break;
-  } /* end NC_DOUBLE */
-  case NC_FLOAT: {
-    float scv_flt=scv.val.f;
-    if(!var->has_mss_val){
-      for(idx=0;idx<sz;idx++) op1.fp[idx]=powf(op1.fp[idx],scv_flt);
-    }else{
-      float mss_val_flt=*(var->mss_val.fp); /* Temporary variable reduces dereferencing */
-      for(idx=0;idx<sz;idx++){
-        if(op1.fp[idx] != mss_val_flt) op1.fp[idx]=powf(op1.fp[idx],scv_flt);
-      } /* end for */
-    } /* end else */
-    break;
-  } /* end NC_FLOAT */
-  default: nco_dfl_case_nc_type_err(); break;
-  }/* end switch */
-  
-  if(var->has_mss_val) (void)cast_nctype_void(var->type,&(var->mss_val));
-  return var;
-} /* end ncap_var_scv_pwr */
-
 scv_sct  
-ncap_scv_clc(scv_sct scv_1, char op, scv_sct scv_2)
+ncap_scv_clc
+(scv_sct scv_1,
+ const char op,
+ scv_sct scv_2)
 {
   /* Purpose: Calculate (scv_1 op scv_2) NB: Scalar values must be of same type */
   
+  /* http://gcc.gnu.org/onlinedocs/cpp/Common-Predefined-Macros.html
+     __GNUC__ : Defined by gcc 
+     __GNUG__ : Defined by g++, equivalent to (__GNUC__ && __cplusplus) */
+
+  /* #ifndef __GNUG__ */
   extern float fmodf(float,float); /* Cannot insert fmodf in ncap_sym_init() because it takes two arguments TODO #20 */
   extern float fabsf(float); /* Sun math.h does not include fabsf() prototype */
+  /* #endif */ /* __GNUG__ */
   
   scv_sct scv_out;
   scv_out.type=scv_1.type;
@@ -751,7 +763,7 @@ ncap_var_stretch /* [fnc] Stretch variables */
      If variables conform, then ncap_var_stretch() will broadcast
      If variables share no dimensions, then ncap_var_stretch() will convolve
      
-     Terminology: 
+     Terminology--- 
      Broadcast: Inflate smaller conforming variable to larger variable
      Conform: Dimensions of one variable are subset of other variable
      Convolve: Construct array which is product of ranks of two variables
@@ -774,17 +786,16 @@ ncap_var_stretch /* [fnc] Stretch variables */
      var_lsr_out plays role of wgt_out
      var_lsr_out=var_lsr only if variables already conform
      var_gtr_out is required since both variables may change
-     var_gtr_out=var_gtr unless convolution is required
-  */
+     var_gtr_out=var_gtr unless convolution is required */
   
-  bool CONFORMABLE=False; /* Whether var_lsr can be made to conform to var_gtr */
+  bool CONFORMABLE=False; /* [flg] Whether var_lsr can be made to conform to var_gtr */
   bool CONVOLVE=False; /* [flg] var_1 and var_2 had to be convolved */
   bool DO_CONFORM; /* [flg] Did var_1 and var_2 conform? */
   bool MUST_CONFORM=False; /* [flg] Must var_1 and var_2 conform? */
   
   int idx;
   int idx_dmn;
-  int var_lsr_var_gtr_dmn_shr_nbr=0; /* Number of dimensions shared by var_lsr and var_gtr */
+  int var_lsr_var_gtr_dmn_shr_nbr=0; /* [nbr] Number of dimensions shared by var_lsr and var_gtr */
   
   var_sct *var_gtr=NULL; /* [ptr] Pointer to variable structure of greater rank */
   var_sct *var_lsr=NULL; /* [ptr] Pointer to variable structure to lesser rank */
@@ -813,7 +824,7 @@ ncap_var_stretch /* [fnc] Stretch variables */
       for(idx=0;idx<var_lsr->nbr_dim;idx++){
         for(idx_dmn=0;idx_dmn<var_gtr->nbr_dim;idx_dmn++){
 	  /* Compare names, not dimension IDs */
-	  if(strstr(var_lsr->dim[idx]->nm,var_gtr->dim[idx_dmn]->nm)){
+	  if(!strcmp(var_lsr->dim[idx]->nm,var_gtr->dim[idx_dmn]->nm)){
 	    var_lsr_var_gtr_dmn_shr_nbr++; /* var_lsr and var_gtr share this dimension */
 	    break;
 	  } /* endif */
@@ -850,7 +861,7 @@ ncap_var_stretch /* [fnc] Stretch variables */
 	  /* var_gtr and var_lsr conform and are same rank */
 	  /* Test whether all var_lsr and var_gtr dimensions match in sequence */
 	  for(idx=0;idx<var_gtr->nbr_dim;idx++){
-	    if(!strstr(var_lsr->dim[idx]->nm,var_gtr->dim[idx]->nm)) break;
+	    if(strcmp(var_lsr->dim[idx]->nm,var_gtr->dim[idx]->nm)) break;
 	  } /* end loop over dimensions */
 	  /* If so, take shortcut and copy var_lsr to var_lsr_out */
 	  if(idx == var_gtr->nbr_dim) DO_CONFORM=True;
@@ -859,15 +870,15 @@ ncap_var_stretch /* [fnc] Stretch variables */
 	  DO_CONFORM=False;
 	} /* end else */
       } /* endif CONFORMABLE */
-    }else{
+    }else{ /* nbr_dmn == 0 */
       /* var_gtr is scalar, if var_lsr is also then set flag to copy var_lsr to var_lsr_out else proceed to generic conform routine */
       if(var_lsr->nbr_dim == 0) DO_CONFORM=True; else DO_CONFORM=False;
-    } /* end else */
+    } /* end else nbr_dmn == 0 */
     if(CONFORMABLE && DO_CONFORM){
       var_lsr_out=nco_var_dpl(var_lsr);
       (void)nco_xrf_var(var_lsr,var_lsr_out);
     } /* end if */
-  } /* end if */
+  } /* endif var_lsr_out == NULL */
   
   if(var_lsr_out == NULL && CONVOLVE){
     /* Convolve variables by returned stretched variables with minimum possible number of dimensions */
@@ -875,12 +886,13 @@ ncap_var_stretch /* [fnc] Stretch variables */
     if(dbg_lvl_get() >= 1) (void)fprintf(stdout,"\n%s: WARNING Convolution not yet implemented, results of operation between %s and %s are unpredictable\n",prg_nm_get(),var_lsr->nm,var_gtr->nm);
     /* Dimensions in convolution are union of dimensions in variables */
     dmn_nbr=var_lsr->nbr_dim+var_gtr->nbr_dim-var_lsr_var_gtr_dmn_shr_nbr; /* Number of dimensions in convolution */
+    dmn_nbr=dmn_nbr; /* CEWI: Avert compiler warning that variable is set but never used */
     /* fxm: these should go away soon */
     var_lsr_out=nco_var_dpl(var_lsr);
     var_gtr_out=nco_var_dpl(var_gtr);
-    for(idx_dmn=0;idx_dmn<var_gtr->nbr_dim;idx_dmn++){;}
-    
-    if(var_lsr_var_gtr_dmn_shr_nbr == 0); else;
+
+    /* for(idx_dmn=0;idx_dmn<var_gtr->nbr_dim;idx_dmn++){;}
+       if(var_lsr_var_gtr_dmn_shr_nbr == 0); else; */
     
     /* Free calling variables */
     var_lsr=nco_var_free(var_lsr);
@@ -889,6 +901,7 @@ ncap_var_stretch /* [fnc] Stretch variables */
   
   if(var_lsr_out == NULL){
     /* Expand lesser variable (var_lsr) to match size of greater variable */
+    const char fnc_nm[]="ncap_var_stretch()"; /* [sng] Function name */
     char *var_lsr_cp;
     char *var_lsr_out_cp;
     
@@ -910,20 +923,18 @@ ncap_var_stretch /* [fnc] Stretch variables */
     var_lsr_out=nco_var_dpl(var_gtr);
     (void)nco_xrf_var(var_lsr,var_lsr_out);
     
-    /* Modify a few elements of lesser variable array */
+    /* Modify elements of lesser variable array */
     var_lsr_out->nm=var_lsr->nm;
     var_lsr_out->id=var_lsr->id;
     var_lsr_out->type=var_lsr->type;
-    var_lsr_out->val.vp=(void *)nco_malloc(var_lsr_out->sz*nco_typ_lng(var_lsr_out->type));
+    var_lsr_out->val.vp=(void *)nco_malloc_dbg(var_lsr_out->sz*nco_typ_lng(var_lsr_out->type),"Unable to malloc() value buffer in variable stretching",fnc_nm);
     var_lsr_cp=(char *)var_lsr->val.vp;
     var_lsr_out_cp=(char *)var_lsr_out->val.vp;
     var_lsr_typ_sz=nco_typ_lng(var_lsr_out->type);
     
     if(var_lsr_out->nbr_dim == 0){
       /* Variables are scalars, not arrays */
-      
       (void)memcpy(var_lsr_out_cp,var_lsr_cp,var_lsr_typ_sz);
-      
     }else{
       /* Variables are arrays, not scalars */
       
@@ -951,7 +962,7 @@ ncap_var_stretch /* [fnc] Stretch variables */
       for(idx=0;idx<var_lsr->nbr_dim;idx++){
 	for(idx_dmn=0;idx_dmn<var_gtr->nbr_dim;idx_dmn++){
 	  /* Compare names, not dimension IDs */
-	  if(strstr(var_gtr->dim[idx_dmn]->nm,var_lsr->dim[idx]->nm)){
+	  if(!strcmp(var_gtr->dim[idx_dmn]->nm,var_lsr->dim[idx]->nm)){
 	    idx_var_lsr_var_gtr[idx]=idx_dmn;
 	    /*	    idx_var_gtr_var_lsr[idx_dmn]=idx;*/
 	    break;
