@@ -1,44 +1,34 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncks.c,v 1.79 2003/01/09 00:27:37 rorik Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncks.c,v 1.109 2004/09/24 00:54:14 zender Exp $ */
 
 /* ncks -- netCDF Kitchen Sink */
 
 /* Purpose: Extract (subsets of) variables from a netCDF file 
    Print them to screen, or copy them to a new file, or both */
 
-/* Copyright (C) 1995--2003 Charlie Zender
+/* Copyright (C) 1995--2004 Charlie Zender
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
-   of the License, or (at your option) any later version.
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
+   The full license text is at http://www.gnu.ai.mit.edu/copyleft/gpl.html 
+   and in the file nco/doc/LICENSE in the NCO source distribution.
    
    As a special exception to the terms of the GPL, you are permitted 
-   to link the NCO source code with the NetCDF and HDF libraries 
-   and distribute the resulting executables under the terms of the GPL, 
-   but in addition obeying the extra stipulations of the netCDF and 
-   HDF library licenses.
+   to link the NCO source code with the DODS, HDF, netCDF, and UDUnits
+   libraries and to distribute the resulting executables under the terms 
+   of the GPL, but in addition obeying the extra stipulations of the 
+   DODS, HDF, netCDF, and UDUnits licenses.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+   See the GNU General Public License for more details.
    
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-
-   The file LICENSE contains the GNU General Public License, version 2
-   It may be viewed interactively by typing, e.g., ncks -L
-
-   The author of this software, Charlie Zender, would like to receive
-   your suggestions, improvements, bug-reports, and patches for NCO.
-   Please contact the project at http://sourceforge.net/projects/nco or by writing
-
+   The original author of this software, Charlie Zender, wants to improve it
+   with the help of your suggestions, improvements, bug-reports, and patches.
+   Please contact the NCO project at http://nco.sf.net or by writing
    Charlie Zender
    Department of Earth System Science
    University of California at Irvine
-   Irvine, CA 92697-3100
- */
+   Irvine, CA 92697-3100 */
 
 /* Usage:
    ncks in.nc 
@@ -51,8 +41,7 @@
    ncks -H -d fl_dim,1 -d char_dim,6,12 -v fl_nm,fl_nm_arr ~/nco/data/in.nc
    ncks -H -m -v char_var_nul,char_var_space,char_var_multinul ~/nco/data/in.nc
    ncks -H -C -v three_dmn_rec_var -d time,,,2 in.nc
-   ncks -H -C -v lon -d lon,3,1 in.nc
-*/
+   ncks -H -C -v lon -d lon,3,1 in.nc */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h> /* Autotools tokens */
@@ -65,21 +54,19 @@
 #include <sys/stat.h> /* stat() */
 #include <time.h> /* machine time */
 #include <unistd.h> /* all sorts of POSIX stuff */
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>  /* getopt_long() */
-#else 
+#ifndef HAVE_GETOPT_LONG
 #include "nco_getopt.h"
-#endif /* !HAVE_GETOPT_H */
+#else /* !NEED_GETOPT_LONG */ 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif /* !HAVE_GETOPT_H */ 
+#endif /* HAVE_GETOPT_LONG */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* Wrappers for netCDF 3.X C-library */
-
-/* #define MAIN_PROGRAM_FILE MUST precede #include nco.h */
+#include <netcdf.h> /* netCDF definitions and C library */
+/* #define MAIN_PROGRAM_FILE MUST precede #include libnco.h */
 #define MAIN_PROGRAM_FILE
-#include "nco.h" /* NCO definitions */
-#include "libnco.h" /* netCDF operator library */
-
+#include "libnco.h" /* netCDF Operator (NCO) library */
 
 int 
 main(int argc,char **argv)
@@ -87,77 +74,87 @@ main(int argc,char **argv)
   bool ALPHABETIZE_OUTPUT=True; /* Option a */
   bool EXCLUDE_INPUT_LIST=False; /* Option c */
   bool FILE_RETRIEVED_FROM_REMOTE_LOCATION;
+  bool FL_LST_IN_FROM_STDIN=False; /* [flg] fl_lst_in comes from stdin */
   bool FORCE_APPEND=False; /* Option A */
   bool FORCE_OVERWRITE=False; /* Option O */
-  bool FORTRAN_STYLE=False; /* Option F */
+  bool FORTRAN_IDX_CNV=False; /* Option F */
   bool HISTORY_APPEND=True; /* Option h */
   bool NCO_BNR_WRT=False; /* [flg] Write binary file */
-  bool OUTPUT_DATA=False; /* Option H */
-  bool OUTPUT_VARIABLE_METADATA=False; /* Option m */
-  bool OUTPUT_GLOBAL_METADATA=False; /* Option M */
-  bool PRINT_DIMENSIONAL_UNITS=False; /* Option u */
-  bool PRN_DMN_IDX_CRD_VAL=True; /* Option q Print leading dimension/coordinate indices/values */
+  bool PRN_VAR_DATA=False; /* [flg] Print variable data */
+  bool PRN_VAR_METADATA=False; /* [flg] Print variable metadata */
+  bool PRN_GLB_METADATA=False; /* [flg] Print global metadata */
+  bool PRN_VAR_DATA_TGL=False; /* [flg] Toggle print variable data Option H */
+  bool PRN_VAR_METADATA_TGL=False; /* [flg] Toggle print variable metadata Option m */
+  bool PRN_GLB_METADATA_TGL=False; /* [flg] Toggle print global metadata Option M */
+  bool PRN_DMN_UNITS=False; /* [flg] Print dimensional units Option u */
+  bool PRN_DMN_IDX_CRD_VAL=True; /* [flg] Print leading dimension/coordinate indices/values Option q */
   bool PROCESS_ALL_COORDINATES=False; /* Option c */
   bool PROCESS_ASSOCIATED_COORDINATES=True; /* Option C */
   bool REMOVE_REMOTE_FILES_AFTER_PROCESSING=True; /* Option R */
 
-  char **var_lst_in=NULL_CEWI;
+  char **var_lst_in=NULL;
   char **fl_lst_abb=NULL; /* Option a */
   char **fl_lst_in;
   char *dlm_sng=NULL;
   char *fl_in=NULL;
   char *fl_pth_lcl=NULL; /* Option l */
   char *lmt_arg[NC_MAX_DIMS];
-  char *opt_sng;
   char *fl_bnr=NULL; /* [sng] Unformatted binary output file */
-  char *fl_out;
+  char *fl_out=NULL; /* Option o */
   char *fl_out_tmp;
   char *fl_pth=NULL; /* Option p */
   char *time_bfr_srt;
   char *cmd_ln;
-  char CVS_Id[]="$Id: ncks.c,v 1.79 2003/01/09 00:27:37 rorik Exp $"; 
-  char CVS_Revision[]="$Revision: 1.79 $";
-  
+
+  const char * const CVS_Id="$Id: ncks.c,v 1.109 2004/09/24 00:54:14 zender Exp $"; 
+  const char * const CVS_Revision="$Revision: 1.109 $";
+  const char * const opt_sng="aABb:CcD:d:FHhl:MmOo:p:qrRs:uv:x-:";
+
   extern char *optarg;
-  
   extern int optind;
   
   FILE *fp_bnr=NULL_CEWI; /* [fl] Unformatted binary output file handle */
 
+  int abb_arg_nbr=0;
+  int fl_nbr=0;
   int fll_md_old; /* [enm] Old fill mode */
+  int glb_att_nbr;
   int idx;
-  int jdx;
   int in_id;  
-  int nbr_abb_arg=0;
-  int nbr_dmn_fl;
+  int jdx;
   int lmt_nbr=0; /* Option d. NB: lmt_nbr gets incremented */
-  int nbr_glb_att;
+  int nbr_dmn_fl;
   int nbr_var_fl;
-  int nbr_xtr=0; /* nbr_xtr won't otherwise be set for -c with no -v */
-  int nbr_fl=0;
+  int nbr_xtr=0; /* nbr_xtr will not otherwise be set for -c with no -v */
   int opt;
   int rec_dmn_id=NCO_REC_DMN_UNDEFINED;
     
   lmt_sct *lmt;
 
-  lmt_all *lmt_lst;        /* Container for lmt */
-  lmt_all *lmt_tmp;         /* temporary pointer */
+  lmt_all_sct *lmt_lst; /* Container for lmt */
+  lmt_all_sct *lmt_tmp; /* Temporary pointer */
 
   char dmn_nm[NC_MAX_NAME];
   long dmn_sz;
 
-  nm_id_sct *xtr_lst=NULL; /* xtr_lst can get realloc()'d from NULL with -c option */
+  nm_id_sct *xtr_lst=NULL; /* xtr_lst may be alloc()'d from NULL with -c option */
 
-  time_t clock;
+  time_t time_crr_time_t;
 
   static struct option opt_lng[]=
-    {
+    { /* Structure ordered by short option key if possible */
+      {"abc",no_argument,0,'a'},
       {"alphabetize",no_argument,0,'a'},
       {"append",no_argument,0,'A'},
-      {"binary-file",required_argument,0,'b'},
+      {"apn",no_argument,0,'A'},
+      {"bnr",no_argument,0,'B'},
       {"binary",no_argument,0,'B'},
+      {"binary-file",required_argument,0,'b'},
+      {"fl_bnr",required_argument,0,'b'},
       {"coords",no_argument,0,'c'},
-      {"nocoords",no_argument,0,'C'},
+      {"crd",no_argument,0,'c'},
+      {"no-coords",no_argument,0,'C'},
+      {"no-crd",no_argument,0,'C'},
       {"debug",required_argument,0,'D'},
       {"dbg_lvl",required_argument,0,'D'},
       {"dimension",required_argument,0,'d'},
@@ -169,20 +166,29 @@ main(int argc,char **argv)
       {"print",no_argument,0,'H'},
       {"prn",no_argument,0,'H'},
       {"local",required_argument,0,'l'},
-      {"meta",no_argument,0,'m'},
-      {"Meta",no_argument,0,'M'},
+      {"lcl",required_argument,0,'l'},
+      {"lcl",required_argument,0,'l'},
+      {"metadata",no_argument,0,'m'},
+      {"mtd",no_argument,0,'m'},
+      {"Metadata",no_argument,0,'M'},
+      {"Mtd",no_argument,0,'M'},
       {"overwrite",no_argument,0,'O'},
+      {"ovr",no_argument,0,'O'},
+      {"output",required_argument,0,'o'},
+      {"fl_out",required_argument,0,'o'},
       {"path",required_argument,0,'p'},
       {"quiet",no_argument,0,'q'},
       {"retain",no_argument,0,'R'},
+      {"rtn",no_argument,0,'R'},
       {"revision",no_argument,0,'r'},
+      {"version",no_argument,0,'r'},
+      {"vrs",no_argument,0,'r'},
+      {"sng",required_argument,0,'s'},
       {"string",required_argument,0,'s'},
       {"format",required_argument,0,'s'},
       {"fmt",required_argument,0,'s'},
       {"units",no_argument,0,'u'},
       {"variable",required_argument,0,'v'},
-      {"version",no_argument,0,'r'},
-      {"vrs",no_argument,0,'r'},
       {"exclude",no_argument,0,'x'},
       {"xcl",no_argument,0,'x'},
       {"help",no_argument,0,'?'},
@@ -192,15 +198,14 @@ main(int argc,char **argv)
 
   /* Start clock and save command line */ 
   cmd_ln=nco_cmd_ln_sng(argc,argv);
-  clock=time((time_t *)NULL);
-  time_bfr_srt=ctime(&clock); time_bfr_srt=time_bfr_srt; /* Avoid compiler warning until variable is used for something */
+  time_crr_time_t=time((time_t *)NULL);
+  time_bfr_srt=ctime(&time_crr_time_t); time_bfr_srt=time_bfr_srt; /* Avoid compiler warning until variable is used for something */
   fl_bnr=(char *)strdup("ncks.bnr");
   
   /* Get program name and set program enum (e.g., prg=ncra) */
   prg_nm=prg_prs(argv[0],&prg);
 
   /* Parse command line arguments */
-  opt_sng="aABb:CcD:d:FHhl:MmOp:qrRs:uv:x-:";
   while((opt = getopt_long(argc,argv,opt_sng,opt_lng,&opt_idx)) != EOF){
     switch(opt){
     case 'a': /* Toggle ALPHABETIZE_OUTPUT */
@@ -229,8 +234,8 @@ main(int argc,char **argv)
       lmt_arg[lmt_nbr]=(char *)strdup(optarg);
       lmt_nbr++;
       break;
-    case 'H': /* Print data to screen */
-      OUTPUT_DATA=True;
+    case 'H': /* Toggle printing data to screen */
+      PRN_VAR_DATA_TGL=True;
       break;
     case 'h': /* Toggle appending to history global attribute */
       HISTORY_APPEND=!HISTORY_APPEND;
@@ -238,17 +243,20 @@ main(int argc,char **argv)
     case 'l': /* Local path prefix for files retrieved from remote file system */
       fl_pth_lcl=optarg;
       break;
-    case 'm': /* Print variable metadata to screen */
-      OUTPUT_VARIABLE_METADATA=True;
+    case 'm': /* Toggle printing variable metadata to screen */
+      PRN_VAR_METADATA_TGL=True;
       break;
-    case 'M': /* Print global metadata to screen */
-      OUTPUT_GLOBAL_METADATA=True;
+    case 'M': /* Toggle printing global metadata to screen */
+      PRN_GLB_METADATA_TGL=True;
       break;
     case 'O': /* Toggle FORCE_OVERWRITE */
       FORCE_OVERWRITE=!FORCE_OVERWRITE;
       break;
+    case 'o': /* Name of output file */
+      fl_out=(char *)strdup(optarg);
+      break;
     case 'F': /* Toggle index convention. Default is 0-based arrays (C-style). */
-      FORTRAN_STYLE=!FORTRAN_STYLE;
+      FORTRAN_IDX_CNV=!FORTRAN_IDX_CNV;
       break;
     case 'p': /* Common file path */
       fl_pth=optarg;
@@ -261,18 +269,20 @@ main(int argc,char **argv)
       break;
     case 'r': /* Print CVS program information and copyright notice */
       (void)copyright_prn(CVS_Id,CVS_Revision);
-      (void)nco_lib_vrs_prn();
+      (void)nco_lbr_vrs_prn();
       nco_exit(EXIT_SUCCESS);
       break;
     case 's': /* User specified delimiter string for printed output */
       dlm_sng=optarg;
       break;
     case 'u': /* Turn on the printing of dimensional units. */
-      PRINT_DIMENSIONAL_UNITS=!PRINT_DIMENSIONAL_UNITS;
+      PRN_DMN_UNITS=!PRN_DMN_UNITS;
       break;
     case 'v': /* Variables to extract/exclude */
+      /* Replace commas with hashes when within braces (convert back later) */
+      (void)nco_lst_comma2hash(optarg);
       var_lst_in=lst_prs(optarg,",",&nbr_xtr);
-      break;
+       break;
     case 'x': /* Exclude rather than extract variables specified with -v */
       EXCLUDE_INPUT_LIST=True;
       break;
@@ -293,95 +303,115 @@ main(int argc,char **argv)
 
   
   /* Process positional arguments and fill in filenames */
-  fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&nbr_fl,&fl_out);
+  fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&fl_nbr,&fl_out,&FL_LST_IN_FROM_STDIN);
   
   /* Make uniform list of user-specified dimension limits */
   lmt=nco_lmt_prs(lmt_nbr,lmt_arg);
   
   /* Parse filename */
-  fl_in=nco_fl_nm_prs(fl_in,0,&nbr_fl,fl_lst_in,nbr_abb_arg,fl_lst_abb,fl_pth);
+  fl_in=nco_fl_nm_prs(fl_in,0,&fl_nbr,fl_lst_in,abb_arg_nbr,fl_lst_abb,fl_pth);
   /* Make sure file is on local system and is readable or die trying */
   fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
   /* Open file for reading */
   nco_open(fl_in,NC_NOWRITE,&in_id);
   
   /* Get number of variables, dimensions, and global attributes in file */
-  (void)nco_inq(in_id,&nbr_dmn_fl,&nbr_var_fl,&nbr_glb_att,&rec_dmn_id);
+  (void)nco_inq(in_id,&nbr_dmn_fl,&nbr_var_fl,&glb_att_nbr,&rec_dmn_id);
   
-  /* Form initial extraction list from user input */
+  /* Form initial extraction list which may include extended regular expressions */
   xtr_lst=nco_var_lst_mk(in_id,nbr_var_fl,var_lst_in,PROCESS_ALL_COORDINATES,&nbr_xtr);
 
   /* Change included variables to excluded variables */
   if(EXCLUDE_INPUT_LIST) xtr_lst=nco_var_lst_xcl(in_id,nbr_var_fl,xtr_lst,&nbr_xtr);
 
   /* Add all coordinate variables to extraction list */
-  if(PROCESS_ALL_COORDINATES) xtr_lst=nco_var_lst_add_crd(in_id,nbr_var_fl,nbr_dmn_fl,xtr_lst,&nbr_xtr);
+  if(PROCESS_ALL_COORDINATES) xtr_lst=nco_var_lst_add_crd(in_id,nbr_dmn_fl,xtr_lst,&nbr_xtr);
 
   /* Make sure coordinates associated extracted variables are also on extraction list */
   if(PROCESS_ASSOCIATED_COORDINATES) xtr_lst=nco_var_lst_ass_crd_add(in_id,xtr_lst,&nbr_xtr);
 
   /* Sort extraction list alphabetically or by variable ID */
-  if(nbr_xtr > 1) xtr_lst=nco_lst_srt(xtr_lst,nbr_xtr,ALPHABETIZE_OUTPUT);
+  if(nbr_xtr > 1) xtr_lst=nco_lst_srt_nm_id(xtr_lst,nbr_xtr,ALPHABETIZE_OUTPUT);
     
   /* We now have final list of variables to extract. Phew. */
   
   /* Place all dims in lmt_lst */
-  lmt_lst = (lmt_all * )nco_malloc(nbr_dmn_fl*sizeof(lmt_all));
+  lmt_lst=(lmt_all_sct *)nco_malloc(nbr_dmn_fl*sizeof(lmt_all_sct));
 
-   for(idx =0 ; idx <nbr_dmn_fl ; idx++){
+  for(idx=0;idx<nbr_dmn_fl;idx++){
     (void)nco_inq_dim(in_id,idx,dmn_nm,&dmn_sz);
-    lmt_tmp = &lmt_lst[idx];
-    lmt_tmp->lmt_dmn  = (lmt_sct **) nco_malloc(sizeof(lmt_sct *));
-    lmt_tmp->lmt_dmn[0] = (lmt_sct *) nco_malloc(sizeof(lmt_sct));
+    lmt_tmp=&lmt_lst[idx];
+    lmt_tmp->lmt_dmn=(lmt_sct **)nco_malloc(sizeof(lmt_sct *));
+    lmt_tmp->lmt_dmn[0]=(lmt_sct *)nco_malloc(sizeof(lmt_sct));
     lmt_tmp->dmn_nm=strdup(dmn_nm);
     lmt_tmp->lmt_dmn_nbr=1;
-    /* iniialize lmt struct */
+    lmt_tmp->dmn_sz_org=dmn_sz;
+    lmt_tmp->WRP=False;
+    lmt_tmp->BASIC_DMN=True;
+    /* Initialize lmt struct */
     lmt_tmp->lmt_dmn[0]->nm=lmt_tmp->dmn_nm;
-    lmt_tmp->lmt_dmn[0]->id = idx;
-    lmt_tmp->lmt_dmn[0]->is_rec_dmn = (idx == rec_dmn_id ? True : False);
-    lmt_tmp->lmt_dmn[0]->srt = 0L;
-    lmt_tmp->lmt_dmn[0]->end = dmn_sz -1L;
-    lmt_tmp->lmt_dmn[0]->cnt = dmn_sz;
-    lmt_tmp->lmt_dmn[0]->srd = 1L;
-    /* flag which shows that struct has been inialized here. A HACK */
-    lmt_tmp->lmt_dmn[0]->lmt_typ = -1;
-  }     
+    lmt_tmp->lmt_dmn[0]->id=idx;
+    lmt_tmp->lmt_dmn[0]->is_rec_dmn=(idx == rec_dmn_id ? True : False);
+    lmt_tmp->lmt_dmn[0]->srt=0L;
+    lmt_tmp->lmt_dmn[0]->end=dmn_sz-1L;
+    lmt_tmp->lmt_dmn[0]->cnt=dmn_sz;
+    lmt_tmp->lmt_dmn[0]->srd=1L;
+    /* Flag to show that struct has been inialized. fxm: A HACK */
+    lmt_tmp->lmt_dmn[0]->lmt_typ=-1;
+  } /* end loop over dimensions */
 
-  /* Now add user specified limits lmt_lst */
-  for(idx=0;idx<lmt_nbr;idx++) {
-  /* Find coordinate/dimension values associated with user-specified limits */
-     (void)nco_lmt_evl(in_id,lmt+idx,0L,FORTRAN_STYLE);
-     for(jdx=0; jdx < nbr_dmn_fl ; jdx++) {
-       if(!strcmp(lmt[idx].nm,lmt_lst[jdx].dmn_nm)){   
-         lmt_tmp = &lmt_lst[jdx];
-         if(lmt_tmp->lmt_dmn[0]->lmt_typ == -1) { 
-	   lmt_tmp->lmt_dmn[0]=lmt+idx; 
-           }else{ 
-           
-	   lmt_tmp->lmt_dmn  = (lmt_sct **) nco_realloc(lmt_tmp->lmt_dmn,((lmt_tmp->lmt_dmn_nbr) +1)*sizeof(lmt_sct *));
-           lmt_tmp->lmt_dmn[(lmt_tmp->lmt_dmn_nbr)++]=lmt+idx;
-	 }
-         break;
-       } /* end if */
-     }
-     /* dimension in limit not found */
-     if( jdx == nbr_dmn_fl ) {
-       (void)fprintf(stderr,"Unable to find limit dimension %s in list\n ",lmt[idx].nm);
-       nco_exit(EXIT_FAILURE);
-     }
+  /* Add user specified limits lmt_lst */
+  for(idx=0;idx<lmt_nbr;idx++){
+    /* Find coordinate/dimension values associated with user-specified limits */
+    (void)nco_lmt_evl(in_id,lmt+idx,0L,FORTRAN_IDX_CNV);
+    for(jdx=0;jdx<nbr_dmn_fl;jdx++) {
+      if(!strcmp(lmt[idx].nm,lmt_lst[jdx].dmn_nm)){   
+	lmt_tmp=&lmt_lst[jdx];
+	lmt_tmp->BASIC_DMN=False;
+	if(lmt_tmp->lmt_dmn[0]->lmt_typ == -1) { 
+	  lmt_tmp->lmt_dmn[0]=lmt+idx; 
+	}else{ 
+	  lmt_tmp->lmt_dmn=(lmt_sct **)nco_realloc(lmt_tmp->lmt_dmn,((lmt_tmp->lmt_dmn_nbr)+1)*sizeof(lmt_sct *));
+	  lmt_tmp->lmt_dmn[(lmt_tmp->lmt_dmn_nbr)++]=lmt+idx;
+	} /* endif */
+	break;
+      } /* end if */
+    } /* end loop over dimensions */
+    /* dimension in limit not found */
+    if(jdx == nbr_dmn_fl){
+      (void)fprintf(stderr,"Unable to find limit dimension %s in list\n ",lmt[idx].nm);
+      nco_exit(EXIT_FAILURE);
+    } /* end if err */
   } /* end loop over idx */       
-
-
-  /* Find and store the final size of each dimension */
-  for(idx = 0 ; idx < nbr_dmn_fl ; idx++){
-    (void)nco_msa_clc_cnt( lmt_lst+idx );
-    /* if( lmt_lst[idx].lmt_dmn_nbr >1 ) (void)nco_msa_prn_idx(&lmt_lst[idx]); */
-  }
   
-	 
+  /* Split up wrapped limits */
+  for(idx=0;idx<nbr_dmn_fl;idx++)
+    if(lmt_lst[idx].BASIC_DMN == False)
+      (void)nco_msa_wrp_splt(lmt_lst+idx);
+  
+  /* Find and store final size of each dimension */
+  for(idx=0;idx<nbr_dmn_fl;idx++){
+    (void)nco_msa_clc_cnt(lmt_lst+idx);
+    /* if(lmt_lst[idx].lmt_dmn_nbr > 1) (void)nco_msa_prn_idx(&lmt_lst[idx]); */
+  } /* end loop over dimensions */
+  
+  if(fl_out == NULL){
+    /* Default is to print data and metadata to screen if output file is not specified 
+       Default is not to print data and metadata to screen if output file is specified */
+    if(PRN_VAR_DATA_TGL) PRN_VAR_DATA=False; else PRN_VAR_DATA=True;
+    if(PRN_VAR_METADATA_TGL) PRN_VAR_METADATA=False; else PRN_VAR_METADATA=True;
+    if(var_lst_in == NULL) PRN_GLB_METADATA=True;
+    if(PRN_GLB_METADATA_TGL) PRN_GLB_METADATA=!PRN_VAR_METADATA;
+  }else{
+    if(PRN_VAR_DATA_TGL) PRN_VAR_DATA=True; else PRN_VAR_DATA=False;
+    if(PRN_VAR_METADATA_TGL) PRN_VAR_METADATA=True; else PRN_VAR_METADATA=False;
+    if(var_lst_in == NULL) PRN_GLB_METADATA=True;
+    if(PRN_GLB_METADATA_TGL) PRN_GLB_METADATA=!PRN_VAR_METADATA;
+  } /* endelse */
+
   if(fl_out != NULL){
     int out_id;  
-
+    
     /* Open output file */
     fl_out_tmp=nco_fl_out_open(fl_out,FORCE_APPEND,FORCE_OVERWRITE,&out_id);
     
@@ -390,10 +420,11 @@ main(int argc,char **argv)
     
     /* Catenate timestamped command line to "history" global attribute */
     if(HISTORY_APPEND) (void)nco_hst_att_cat(out_id,cmd_ln);
-
+    
     for(idx=0;idx<nbr_xtr;idx++){
       int var_out_id;
-       /* Define variable in output file */
+
+      /* Define variable in output file */
       if(lmt_nbr > 0) var_out_id=nco_cpy_var_dfn_lmt(in_id,out_id,rec_dmn_id,xtr_lst[idx].nm,lmt_lst,nbr_dmn_fl); else var_out_id=nco_cpy_var_dfn(in_id,out_id,rec_dmn_id,xtr_lst[idx].nm);
       /* Copy variable's attributes */
       (void)nco_att_cpy(in_id,out_id,xtr_lst[idx].id,var_out_id,True);
@@ -412,12 +443,11 @@ main(int argc,char **argv)
     for(idx=0;idx<nbr_xtr;idx++){
       if(dbg_lvl > 2 && !NCO_BNR_WRT) (void)fprintf(stderr,"%s, ",xtr_lst[idx].nm);
       if(dbg_lvl > 0) (void)fflush(stderr);
-
+      /* Old hyperslab routines */
       /* if(lmt_nbr > 0) (void)nco_cpy_var_val_lmt(in_id,out_id,fp_bnr,NCO_BNR_WRT,xtr_lst[idx].nm,lmt,lmt_nbr); else (void)nco_cpy_var_val(in_id,out_id,fp_bnr,NCO_BNR_WRT,xtr_lst[idx].nm); */
-
+      /* Multi-slab routines */
       if(lmt_nbr > 0) (void)nco_cpy_var_val_mlt_lmt(in_id,out_id,fp_bnr,NCO_BNR_WRT,xtr_lst[idx].nm,lmt_lst,nbr_dmn_fl); else (void)nco_cpy_var_val(in_id,out_id,fp_bnr,NCO_BNR_WRT,xtr_lst[idx].nm);
     } /* end loop over idx */
-    
 
     /* [fnc] Close unformatted binary data file */
     if(NCO_BNR_WRT) (void)nco_bnr_close(fp_bnr,fl_bnr);
@@ -427,8 +457,8 @@ main(int argc,char **argv)
 
   } /* end if fl_out != NULL */
   
-  if(OUTPUT_GLOBAL_METADATA){
-    (void)fprintf(stdout,"Opened file %s: dimensions = %i, variables = %i, global atts. = %i, id = %i\n",fl_in,nbr_dmn_fl,nbr_var_fl,nbr_glb_att,in_id);
+  if(PRN_GLB_METADATA){
+    (void)fprintf(stdout,"Opened file %s: dimensions = %i, variables = %i, global atts. = %i, id = %i\n",fl_in,nbr_dmn_fl,nbr_var_fl,glb_att_nbr,in_id);
     if(rec_dmn_id != NCO_REC_DMN_UNDEFINED){
       char rec_dmn_nm[NC_MAX_NAME];
       long rec_dmn_sz;
@@ -439,27 +469,30 @@ main(int argc,char **argv)
     
     /* Print all global attributes */
     (void)nco_prn_att(in_id,NC_GLOBAL);
-  } /* endif OUTPUT_GLOBAL_METADATA */
+  } /* endif PRN_GLB_METADATA */
   
-  if(OUTPUT_VARIABLE_METADATA){
+  if(PRN_VAR_METADATA){
     for(idx=0;idx<nbr_xtr;idx++){
       /* Print variable's definition */
       (void)nco_prn_var_dfn(in_id,xtr_lst[idx].nm);
       /* Print variable's attributes */
       (void)nco_prn_att(in_id,xtr_lst[idx].id);
     } /* end loop over idx */
-  } /* end if OUTPUT_VARIABLE_METADATA */
-  
-  if(OUTPUT_DATA){
-    for(idx=0;idx<nbr_xtr;idx++) (void)nco_prn_var_val_lmt(in_id,xtr_lst[idx].nm,lmt,lmt_nbr,dlm_sng,FORTRAN_STYLE,PRINT_DIMENSIONAL_UNITS,PRN_DMN_IDX_CRD_VAL);
-  } /* end if OUTPUT_DATA */
+  } /* end if PRN_VAR_METADATA */
+
+  /* if(PRN_VAR_DATA){
+    for(idx=0;idx<nbr_xtr;idx++) (void)nco_prn_var_val_lmt(in_id,xtr_lst[idx].nm,lmt,lmt_nbr,dlm_sng,FORTRAN_IDX_CNV,PRN_DMN_UNITS,PRN_DMN_IDX_CRD_VAL);
+    } */
+
+  if(PRN_VAR_DATA){
+    for(idx=0;idx<nbr_xtr;idx++) (void)nco_msa_prn_var_val(in_id,xtr_lst[idx].nm,lmt_lst,nbr_dmn_fl,dlm_sng,FORTRAN_IDX_CNV,PRN_DMN_UNITS,PRN_DMN_IDX_CRD_VAL);
+  } /* end if PRN_VAR_DATA */
   
   /* Close input netCDF file */
   nco_close(in_id);
   
   /* Remove local copy of file */
   if(FILE_RETRIEVED_FROM_REMOTE_LOCATION && REMOVE_REMOTE_FILES_AFTER_PROCESSING) (void)nco_fl_rm(fl_in);
-
   nco_exit_gracefully();
   return EXIT_SUCCESS;
 } /* end main() */
