@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.h,v 1.11 2002/12/30 02:56:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.h,v 1.18 2004/08/16 04:13:33 zender Exp $ */
 
 /* Purpose: Attribute utilities */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -22,18 +22,17 @@
 #include <time.h> /* machine time */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* netCDF3.0 wrapper functions */
+#include <netcdf.h> /* netCDF definitions and C library */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 
 /* Personal headers */
-#include "nco.h" /* NCO definitions */
+#include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_cnf_typ.h" /* Conform variable types */
 #include "nco_ctl.h" /* Program flow control functions */
+#include "nco_lst_utl.h" /* List utilities */
 #include "nco_mmr.h" /* Memory management */
 #include "nco_mss_val.h" /* Missing value utilities */
-#if (!defined HAVE_STRCASECMP) || (!defined HAVE_STRDUP)
 #include "nco_sng_utl.h" /* String utilities */
-#endif /* HAVE_STRCASECMP || HAVE_STRDUP */
 
 #ifdef __cplusplus
 extern "C" {
@@ -43,7 +42,7 @@ void
 nco_aed_prc /* [fnc] Process single attribute edit for single variable */
 (const int nc_id, /* I [id] Input netCDF file ID */
  const int var_id, /* I [id] ID of variable on which to perform attribute editing */
- const aed_sct aed); /* I [id] Structure containing information necessary to edit */
+ const aed_sct aed); /* I [sct] Structure containing information necessary to edit */
 
 void 
 nco_att_cpy  /* [fnc] Copy attributes from input netCDF file to output netCDF file */
@@ -53,11 +52,22 @@ nco_att_cpy  /* [fnc] Copy attributes from input netCDF file to output netCDF fi
  const int var_out_id, /* I [id] netCDF output-variable ID */
  const bool PCK_ATT_CPY); /* I [flg] Copy attributes "scale_factor", "add_offset" */
 
+void 
+nco_fl_lst_att_cat /* [fnc] Add input file list global attribute */
+(const int out_id, /* I [id] netCDF output-file ID */
+ CST_X_PTR_CST_PTR_CST_Y(char,fl_lst_in), /* I [sng] Input file list */
+ const int fl_nbr); /* I [nbr] Number of files in input file list */
+
 void 
 nco_hst_att_cat /* [fnc] Add command line, date stamp to history attribute */
 (const int out_id, /* I [id] netCDF output-file ID */
  const char * const hst_sng); /* I [sng] String to add to history attribute */
 
+aed_sct * /* O [sct] List of attribute edit structures */
+nco_prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure list */
+(const int nbr_aed, /* I [nbr] Number of attributes in list */
+ X_CST_PTR_CST_PTR_Y(char,aed_arg)); /* I/O [sng] List of user-specified attribute edits (delimiters are changed to NULL on output */
+
 int /* [flg] Variable and attribute names are conjoined */
 nco_prs_att /* [fnc] Parse conjoined variable and attribute names */
 (rnm_sct * const rnm_att, /* I/O [sct] Structure [Variable:]Attribute name on input, Attribute name on output */
@@ -68,6 +78,11 @@ nco_prs_rnm_lst /* [fnc] Set old_nm, new_nm elements of rename structure */
 (const int nbr_rnm, /* I [nbr] Number of elements in rename list */
  CST_X_PTR_CST_PTR_CST_Y(char,rnm_arg)); /* I [sng] Unstructured list of old, new names */
 
+void 
+nco_thr_att_cat /* [fnc] Add threading global attribute */
+(const int out_id, /* I [id] netCDF output-file ID */
+ const int thr_nbr); /* I [nbr] Thread number */
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif /* __cplusplus */
