@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_bnr.h,v 1.4 2002/12/30 02:56:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_bnr.h,v 1.8 2004/01/12 18:11:07 zender Exp $ */
 
 /* Purpose: Binary write utilities */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -16,11 +16,11 @@
 #include <stdio.h> /* stderr, FILE, NULL, printf */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* netCDF3.0 wrapper functions */
+#include <netcdf.h> /* netCDF definitions and C library */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 
 /* Personal headers */
-#include "nco.h" /* NCO definitions */
+#include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_ctl.h" /* Program flow control functions */
 
 #ifdef __cplusplus
@@ -39,7 +39,6 @@ nco_bnr_close /* [fnc] Close unformatted binary data file for writing */
 size_t /* O [nbr] Number of elements successfully written */
 nco_bnr_wrt /* [fnc] Write unformatted binary data */
 (FILE * const fp_bnr, /* I [fl] Unformatted binary output file handle */
- const char * const fl_bnr, /* I [sng] Unformatted binary output file */
  const char * const var_nm, /* I [sng] Variable name */
  const long var_sz, /* I [nbr] Variable size */
  const nc_type var_typ, /* I [enm] Variable type */
