@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.c,v 1.13 2002/12/30 02:56:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.c,v 1.22 2004/09/06 22:48:47 zender Exp $ */
 
 /* Purpose: Conform variable types */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include "nco_cnf_typ.h" /* Conform variable types */
@@ -74,11 +74,23 @@ nco_typ_cnv_rth  /* [fnc] Convert char, short, long, int types to doubles before
   /* Threads: Routine is thread safe and calls no unsafe routines */
   /* Purpose: Convert char, short, long, int types to doubles for arithmetic
      Conversions are performed unless arithmetic operation type is min or max
-     Floats (and doubles, of course) are not converted for performance reason 
-     This routine is usually called 
-     Remember to convert back after weighting and arithmetic are complete! */
+     Floats (and doubles, of course) are not converted for performance reasons
+     Convert back after weighting and arithmetic are complete! */
 
-  if(var->type != NC_FLOAT && var->type != NC_DOUBLE && nco_op_typ != nco_op_min && nco_op_typ != nco_op_max) var=nco_var_cnf_typ((nc_type)NC_DOUBLE,var);
+  /* Variables which are unpacked into NC_FLOAT should remain NC_FLOAT here
+     Unpacking happens 'transparently' when original data are read by nco_var_get() 
+     Output structures (i.e., var_prc_out) often correspond to original input type
+     Thus var may have typ_upk=NC_FLOAT and type=NC_SHORT
+     In that case, promote based on typ_upk rather than on type
+     Otherwise most var's that had been unpacked would be converted to NC_DOUBLE here
+     That would put them in conflict with the corresponding var_out, which is usually
+     based on typ_upk
+     Check this first, then proceed with normal non-float->double conversion */
+  if(var->typ_upk == NC_FLOAT){
+    var=nco_var_cnf_typ((nc_type)NC_FLOAT,var);
+  }else{ /* Conversion only for appropriate operation types */ 
+    if(var->type != NC_FLOAT && var->type != NC_DOUBLE && nco_op_typ != nco_op_min && nco_op_typ != nco_op_max) var=nco_var_cnf_typ((nc_type)NC_DOUBLE,var);
+  } /* end if */
   
   return var;
 } /* nco_typ_cnv_rth() */
@@ -95,14 +107,14 @@ nco_cnv_var_typ_dsk  /* [fnc] Revert variable to on-disk type */
 } /* nco_cnv_var_typ_dsk() */
 
 var_sct * /* O [sct] Variable with mss_val converted to typ_upk */
-nco_cnv_mss_val_typ  /* [fnc] Convert missing_value, if any, to typ_upk */
-(var_sct *var, /* I [sct] Variable with missing_value to be converted */
+nco_cnv_mss_val_typ  /* [fnc] Convert missing_value, if any, to mss_val_out_typ */
+(var_sct *var, /* I [sct] Variable with missing_value to convert */
  const nc_type mss_val_out_typ) /* I [enm] Type of mss_val on output */
 {
-  /* Purpose: Convert variable missing_value field, if any, to typ_upk
-     Routine is currently called only by ncra, for following reason:
+  /* Purpose: Convert variable missing_value field, if any, to mss_val_out_typ
+     Routine is currently called only by ncra and by nco_var_get(), for following reason:
      Most applications should call nco_var_cnf_typ() without calling nco_cnv_mss_val_typ()
-     since that routine converts misssing_value type along with variable type
+     since nco_var_cnf_typ() converts misssing_value type along with variable type
      The important exception to this is ncra
      ncra refreshes variable metadata (including missing_value, if any) 
      once per file (naturally), but refreshes variable values once per record.
@@ -139,9 +151,12 @@ nco_cnv_mss_val_typ  /* [fnc] Convert missing_value, if any, to typ_upk */
   /* Sequence of following commands is important (copy before overwriting!) */
   mss_val_in=var->mss_val;
   mss_val_out.vp=(void *)nco_malloc(nco_typ_lng(mss_val_out_typ));
-  (void)val_conform_type(var_in_typ,mss_val_in,mss_val_out_typ,mss_val_out);
+  (void)nco_val_cnf_typ(var_in_typ,mss_val_in,mss_val_out_typ,mss_val_out);
   var->mss_val=mss_val_out;
-  /* Free original */
+  /* Free original 
+     Of course this only changes var_in->mss_val
+     Calling routine must update var_out->mss_val if var_out->val points to var_in->val
+     This dangling pointer was a problem in ncpdq */
   mss_val_in.vp=nco_free(mss_val_in.vp);
   
   return var; /* O [sct] Variable with mss_val converted to typ_upk */
@@ -155,7 +170,7 @@ nco_var_cnf_typ /* [fnc] Return copy of input variable typecast to desired type
   /* Threads: Routine is thread safe and makes no unsafe routines */
   /* Purpose: Return copy of input variable typecast to desired type
      Routine assumes variable and missing_value are same type in memory
-     This is currently always true except for a brief time in ncra
+     This is currently always true except briefly in ncra (and possibly ncpdq)
      This condition is unsafe and is described more fully in nco_cnv_mss_val_typ() */
   long idx;
   long sz;
@@ -192,14 +207,14 @@ nco_var_cnf_typ /* [fnc] Return copy of input variable typecast to desired type
   val_out=var_out->val;
   
   /* Copy and typecast missing_value attribute, if any */
-  /* fxm: 20020814: ncra bug here since this tries to re-convert missing values that have already been promoted to double during arithmetic */
+  /* fxm: 20020814: ncra bug here since this tries to re-convert missing values that have already been promoted to double during arithmetic pck_dbg */
   if(var_out->has_mss_val){
     ptr_unn var_in_mss_val;
 
     /* Sequence of following commands is important (copy before overwriting!) */
     var_in_mss_val=var_out->mss_val;
     var_out->mss_val.vp=(void *)nco_malloc(nco_typ_lng(var_out->type));
-    (void)val_conform_type(var_in_typ,var_in_mss_val,var_out_typ,var_out->mss_val);
+    (void)nco_val_cnf_typ(var_in_typ,var_in_mss_val,var_out_typ,var_out->mss_val);
     /* Free original */
     var_in_mss_val.vp=nco_free(var_in_mss_val.vp);
   } /* end if */
@@ -285,7 +300,7 @@ nco_var_cnf_typ /* [fnc] Return copy of input variable typecast to desired type
 } /* end nco_var_cnf_typ() */
 
 void
-val_conform_type /* [fnc] Copy val_in and typecast from typ_in to typ_out */
+nco_val_cnf_typ /* [fnc] Copy val_in and typecast from typ_in to typ_out */
 (const nc_type typ_in, /* I [enm] Type of input value */
  ptr_unn val_in, /* I [ptr] Pointer to input value */
  const nc_type typ_out, /* I [enm] Type of output value */
@@ -293,7 +308,7 @@ val_conform_type /* [fnc] Copy val_in and typecast from typ_in to typ_out */
 {
   /* Threads: Routine is thread safe and makes no unsafe routines */
   /* Purpose: Fill val_out with copy of val_in that has been typecast from typ_in to typ_out
-     Last-referenced state of both value pointers is assumed to be .vp, and the val_out union is returned in that state */
+     Last-referenced state of both value pointers is assumed to be .vp, and val_out union is returned in that state */
 
   /* val_in and val_out should not be same pointer union since
      val_out must hold enough space (one element of type typ_out) to hold output
@@ -372,10 +387,10 @@ val_conform_type /* [fnc] Copy val_in and typecast from typ_in to typ_out */
      value and are thus purely local to this routine. The only thing changed by this
      routine is the contents of the location pointed to by the pointer to the output value. */
   
-} /* end val_conform_type */
+} /* end nco_val_cnf_typ */
 
 int /* O [enm] Dummy return */
-scv_conform_type /* [fnc] Convert scalar attribute to typ_new using C implicit coercion */
+nco_scv_cnf_typ /* [fnc] Convert scalar attribute to typ_new using C implicit coercion */
 (const nc_type typ_new, /* I [enm] Type to convert scv_old to */
  scv_sct * const scv_old) /* I/O [sct] Scalar value to convert */
 {
@@ -443,7 +458,7 @@ scv_conform_type /* [fnc] Convert scalar attribute to typ_new using C implicit c
   scv_new.type=typ_new;
   *scv_old=scv_new;
   return 1;      
-} /* end scv_conform_type */
+} /* end nco_scv_cnf_typ */
 
 nc_type /* O [enm] Highest precision of input variables */
 ncap_var_retype /* [fnc] Promote variable to higher common precision */
@@ -470,10 +485,10 @@ ncap_scv_scv_cnf_typ_hgh_prc /* [fnc] Promote arguments to higher precision if n
   if(scv_1->type == scv_2->type){
     return scv_1->type;
   }else if(scv_1->type > scv_2->type){
-    (void)scv_conform_type(scv_1->type,scv_2);
+    (void)nco_scv_cnf_typ(scv_1->type,scv_2);
     return scv_1->type;
   }else{
-    (void)scv_conform_type(scv_2->type,scv_1);
+    (void)nco_scv_cnf_typ(scv_2->type,scv_1);
     return scv_2->type;
   } /* endif */
 } /* end ncap_scv_scv_cnf_typ_hgh_prc */
@@ -492,7 +507,7 @@ ncap_var_scv_cnf_typ_hgh_prc /* [fnc] Promote arguments to higher precision if n
   if((*var)->type == scv->type){
     return (*var)->type;
   }else if((*var)->type > scv->type){
-    (void)scv_conform_type((*var)->type,scv); 
+    (void)nco_scv_cnf_typ((*var)->type,scv); 
     return (*var)->type;
   }else{
     *var=nco_var_cnf_typ(scv->type,*var);
