@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.h,v 1.12 2002/12/30 02:56:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.h,v 1.19 2004/09/06 22:48:48 zender Exp $ */
 
 /* Purpose: Conform variable types */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -17,11 +17,11 @@
 #include <string.h> /* strcmp. . . */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* netCDF3.0 wrapper functions */
+#include <netcdf.h> /* netCDF definitions and C library */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 
 /* Personal headers */
-#include "nco.h" /* NCO definitions */
+#include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_mmr.h" /* Memory management */
 
 
@@ -54,19 +54,19 @@ nco_var_cnf_typ /* [fnc] Return copy of input variable typecast to desired type
  var_sct *var_in); /* I/O [enm] Pointer to variable structure (may be destroyed) */
 
 var_sct * /* O [sct] Variable with mss_val converted to typ_upk */
-nco_cnv_mss_val_typ  /* [fnc] Convert missing_value, if any, to typ_upk */
-(var_sct *var, /* I [sct] Variable with missing_value to be converted */
+nco_cnv_mss_val_typ  /* [fnc] Convert missing_value, if any, to mss_val_out_typ */
+(var_sct *var, /* I [sct] Variable with missing_value to convert */
  const nc_type mss_val_out_typ); /* I [enm] Type of mss_val on output */
 
 void
-val_conform_type /* [fnc] Copy val_in and typecast from typ_in to typ_out */
+nco_val_cnf_typ /* [fnc] Copy val_in and typecast from typ_in to typ_out */
 (const nc_type typ_in, /* I [enm] Type of input value */
  ptr_unn val_in, /* I [ptr] Pointer to input value */
  const nc_type typ_out, /* I [enm] Type of output value */
  ptr_unn val_out); /* I [ptr] Pointer to output value */
 
 int /* O [enm] Dummy return */
-scv_conform_type /* [fnc] Convert scalar attribute to typ_new using C implicit coercion */
+nco_scv_cnf_typ /* [fnc] Convert scalar attribute to typ_new using C implicit coercion */
 (const nc_type typ_new, /* I [enm] Type to convert scv_old to */
  scv_sct * const scv_old); /* I/O [sct] Scalar value to convert */
 
