@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_ctl.h,v 1.8 2002/12/30 02:56:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_ctl.h,v 1.15 2004/06/18 01:21:05 zender Exp $ */
 
 /* Purpose: Program flow control functions */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -24,11 +24,11 @@
 #include <unistd.h> /* POSIX stuff */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* netCDF3.0 wrapper functions */
+#include <netcdf.h> /* netCDF definitions and C library */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 
 /* Personal headers */
-#include "nco.h" /* NCO definitions */
+#include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_mmr.h" /* Memory management */
 
 #ifdef __cplusplus
@@ -40,19 +40,26 @@ nco_exit /* [fnc] Wrapper for exit() */
 (int rcd); /* I [enm] Return code */
 
 void 
-nco_exit_gracefully(void); /* [fnc] Clean up timers, file descriptors, then exit */
+nco_exit_gracefully(void); /* [fnc] Clean up timers, file descriptors, memory, then exit */
 
 bool /* [flg] Program does arithmetic */
-is_rth_opr /* [fnc] Query whether program does arithmetic */
+nco_is_rth_opr /* [fnc] Query whether program does arithmetic */
+(const int prg_id); /* [enm] Program ID */
+
+bool /* [flg] Program is multi-file operator */
+nco_is_mlt_fl_opr /* [fnc] Query whether program is multi-file operator */
 (const int prg_id); /* [enm] Program ID */
 
 void
-nco_lib_vrs_prn(void); /* [fnc] Print netCDF library version */
+nco_lbr_vrs_prn(void); /* [fnc] Print netCDF library version */
 
 char * /* O [sng] nm_in stripped of any path (i.e., program name stub) */ 
 prg_prs /* [fnc] Strip program name to stub and return program ID */
 (const char * const nm_in, /* I [sng] Name of program, i.e., argv[0] (may include path prefix) */
- int * const prg); /* O [enm] Enumerated number corresponding to nm_in */
+ int * const prg_lcl); /* O [enm] Enumerated number corresponding to nm_in */
+
+const char * /* O [sng] Mnemonic that describes current NCO version */
+nco_nmn_get(void); /* [fnc] Return mnemonic that describes current NCO version */
 
 void 
 nco_usg_prn(void); /* [fnc] Print correct program usage */
