@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.c,v 1.8 2002/12/30 02:56:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.c,v 1.15 2004/08/14 21:00:00 zender Exp $ */
 
 /* Purpose: Dimension utilities */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include "nco_dmn_utl.h" /* Dimension utilities */
@@ -12,7 +12,7 @@ void
 nco_dmn_dfn /* [fnc] Define dimensions in output file */
 (const char * const fl_nm, /* I [sng] Name of output file */
  const int nc_id, /* I [id] netCDF output file ID */
- dmn_sct ** const dmn, /* I [sct] Dimension structures to be defined in output file */
+ dmn_sct ** const dmn, /* I [sct] Dimension structures to define in output file */
  const int nbr_dmn) /* I [nbr] Number of dimension structures in structure list */
 {
   /* Purpose: Define dimensions in output file */
@@ -39,18 +39,19 @@ nco_dmn_dfn /* [fnc] Define dimensions in output file */
   
 } /* end nco_dmn_dfn() */
 
-dmn_sct * /* O [sct] copy of input dimension structure */
+dmn_sct * /* O [sct] Copy of input dimension structure */
 nco_dmn_dpl /* [fnc] Duplicate input dimension structure */
 (const dmn_sct * const dmn) /* I [sct] Dimension structure to duplicate */
 {
   /* Purpose: nco_malloc() and return duplicate of input dmn_sct */
+  const void *dmn_vp=dmn;
 
   dmn_sct *dmn_cpy;
 
   dmn_cpy=(dmn_sct *)nco_malloc(sizeof(dmn_sct));
 
   /* Shallow copy structure */
-  (void)memcpy((void *)dmn_cpy,(void *)dmn,sizeof(dmn_sct));
+  (void)memcpy((void *)dmn_cpy,dmn_vp,sizeof(dmn_sct));
 
   /* fxm: Should copy name as well */
 
@@ -60,7 +61,7 @@ nco_dmn_dpl /* [fnc] Duplicate input dimension structure */
 
 dmn_sct * /* O [sct] Output dimension structure */
 nco_dmn_fll /* [fnc] Create and return completed dmn_sct */
-(const int nc_id, /* I [id] netCDF input file ID*/
+(const int nc_id, /* I [id] netCDF input file ID */
  const int dmn_id, /* I [id] Dimension ID */
  const char * const dmn_nm) /* I [sng] Dimension name */
 {
@@ -82,7 +83,7 @@ nco_dmn_fll /* [fnc] Create and return completed dmn_sct */
   dmn->is_crd_dmn=False;
   (void)nco_inq_dimlen(dmn->nc_id,dmn_id,&dmn->sz);
   
-  /* Get the record dimension ID */
+  /* Get record dimension ID */
   (void)nco_inq(dmn->nc_id,(int *)NULL,(int *)NULL,(int *)NULL,&rec_dmn_id);
   if(dmn->id == rec_dmn_id){
     dmn->is_rec_dmn=True;
@@ -199,10 +200,15 @@ nco_dmn_lst_ass_var /* [fnc] Create list of all dimensions associated with input
   /* We now have final list of dimensions to extract. Phew. */
   
   /* Free unused space in output dimension list */
+  /* valgrind fxm: 20040814 valgrind reports: Are obsolete names free()'d before realloc()?
+     ==31046== 112 bytes in 1 blocks are definitely lost in loss record 2 of 6
+     ==31046==    at 0x1B9079FF: realloc (vg_replace_malloc.c:197)
+     ==31046==    by 0x8056084: nco_realloc (nco_mmr.c:235)
+     ==31046==    by 0x8051490: nco_dmn_lst_ass_var (nco_dmn_utl.c:203)
+     ==31046==    by 0x804A057: main (ncpdq.c:327) */
   dmn=(nm_id_sct *)nco_realloc((void *)dmn,*nbr_dmn*sizeof(nm_id_sct));
   
   return dmn;
-
 } /* end nco_dmn_lst_ass_var() */
 
 nm_id_sct * /* O [sct] Dimension list */
@@ -211,7 +217,7 @@ nco_dmn_lst_mk /* [fnc] Attach dimension IDs to dimension list */
  CST_X_PTR_CST_PTR_CST_Y(char,dmn_lst_in), /* I [sng] User-specified list of dimension names */
  const int nbr_dmn) /* I [nbr] Total number of dimensions in list */
 {
-  /* Purpose:  */
+  /* Purpose: Create list of dimension name-id structures from list of dimension name strings */
   int idx;
 
   nm_id_sct *dmn_lst;
@@ -232,9 +238,7 @@ nco_dmn_xrf  /* [fnc] Crossreference xrf elements of dimension structures */
  dmn_sct * const dmn_2) /* I/O [sct] Dimension structure */
 {
   /* Purpose: Make xrf elements of dimension structures point to eachother */
-
   dmn_1->xrf=dmn_2;
   dmn_2->xrf=dmn_1;
-
 } /* end nco_dmn_xrf() */
 
