@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.h,v 1.8 2002/12/30 02:56:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.h,v 1.13 2004/07/26 17:45:49 zender Exp $ */
 
 /* Purpose: Dimension utilities */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -21,11 +21,11 @@
 #include <string.h> /* strcmp. . . */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* netCDF3.0 wrapper functions */
+#include <netcdf.h> /* netCDF definitions and C library */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 
 /* Personal headers */
-#include "nco.h" /* NCO definitions */
+#include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_mmr.h" /* Memory management */
 
 #ifdef __cplusplus
@@ -36,7 +36,7 @@ void
 nco_dmn_dfn /* [fnc] Define dimensions in output file */
 (const char * const fl_nm, /* I [sng] Name of output file */
  const int nc_id, /* I [id] netCDF output file ID */
- dmn_sct ** const dmn, /* I [sct] Dimension structures to be defined in output file */
+ dmn_sct ** const dmn, /* I [sct] Dimension structures to define in output file */
  const int nbr_dmn); /* I [nbr] Number of dimension structures in structure list */
 
 dmn_sct * /* O [sct] copy of input dimension structure */
