@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_fl_utl.h,v 1.11 2002/12/30 02:56:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_fl_utl.h,v 1.22 2004/07/09 16:36:11 zender Exp $ */
 
 /* Purpose: File manipulation */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -33,11 +33,11 @@
 #endif /* !WIN32 */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* netCDF3.0 wrapper functions */
+#include <netcdf.h> /* netCDF definitions and C library */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 
 /* Personal headers */
-#include "nco.h" /* NCO definitions */
+#include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_ctl.h" /* Program flow control functions */
 #include "nco_mmr.h" /* Memory management */
 
@@ -45,19 +45,26 @@
 extern "C" {
 #endif /* __cplusplus */
 
+void
+nco_fl_cmp_err_chk(void); /* [fnc] Perform error checking on file */
+
 void
 nco_fl_cp /* [fnc] Copy first file to second */
 (const char * const fl_src, /* I [sng] Name of source file to copy */
  const char * const fl_dst); /* I [sng] Name of destination file */
 
-void
-fl_mv /* [fnc] Move first file to second */
-(const char * const fl_src, /* I [sng] Name of source file to move */
- const char * const fl_dst); /* I [sng] Name of destination file */
+char * /* O [sng] Canonical file name*/
+nco_fl_info_get /* [fnc] Determine canonical filename and properties */
+(const char * const fl_nm_lcl); /* I [sng] Name of file */
 
-void 
-nco_fl_rm /* [fnc] Remove file */
-(char *fl_nm); /* I [sng] File to be removed */
+char ** /* O [sng] List of user-specified filenames */
+nco_fl_lst_mk /* [fnc] Create file list from command line positional arguments */
+(CST_X_PTR_CST_PTR_CST_Y(char,argv), /* I [sng] Argument list */
+ const int argc, /* I [nbr] Argument count */
+ int arg_crr, /* I [idx] Index of current argument */
+ int * const fl_nbr, /* O [nbr] Number of files in input file list */
+ char ** const fl_out, /* I/O [sng] Name of output file */
+ bool *FL_LST_IN_FROM_STDIN); /* O [flg] fl_lst_in comes from stdin */
 
 char * /* O [sng] Filename of locally available file */
 nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
@@ -65,21 +72,18 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
  const char * const fl_pth_lcl, /* I [sng] Local storage area for files retrieved from remote locations */
  int * const FILE_RETRIEVED_FROM_REMOTE_LOCATION); /* O [flg] File was retrieved from remote location */
 
-char ** /* O [sng] List of user-specified filenames */
-nco_fl_lst_mk /* [fnc] Create file list from command line positional arguments */
-(CST_X_PTR_CST_PTR_CST_Y(char,argv), /* I [sng] Argument list */
- const int argc, /* I [nbr] Argument count */
- int arg_crr, /* I [idx] Index of current argument */
- int * const nbr_fl, /* O [nbr] Number of files in input file list */
- char ** const fl_out); /* O [sng] Name of output file */
+void
+nco_fl_mv /* [fnc] Move first file to second */
+(const char * const fl_src, /* I [sng] Name of source file to move */
+ const char * const fl_dst); /* I [sng] Name of destination file */
 
 char * /* O [sng] Name of file to retrieve */
 nco_fl_nm_prs /* [fnc] Construct file name from input arguments */
 (char *fl_nm, /* I/O [sng] Current filename, if any */
- const int fl_nbr, /* I [nbr] Ordinal index of file in input file list */
- int * const nbr_fl, /* I/O [nbr] number of files to be processed */
+ const int fl_idx, /* I [nbr] Ordinal index of file in input file list */
+ int * const fl_nbr, /* I/O [nbr] Number of files to be processed */
  char * const * const fl_lst_in, /* I [sng] User-specified filenames */
- const int nbr_abb_arg, /* I [nbr] Number of abbreviation arguments */
+ const int abb_arg_nbr, /* I [nbr] Number of abbreviation arguments */
  CST_X_PTR_CST_PTR_CST_Y(char,fl_lst_abb), /* I [sng] NINTAP-style arguments, if any */
  const char * const fl_pth); /* I [sng] Path prefix for files in fl_lst_in */
 
@@ -94,10 +98,12 @@ void
 nco_fl_out_cls /* [fnc] Close temporary output file, move it to permanent output file */
 (const char * const fl_out, /* I [sng] Name of permanent output file */
  const char * const fl_out_tmp, /* I [sng] Name of temporary output file to close and move to permanent output file */
- const int nc_id); /* I [id] file ID of fl_out_tmp */
+ const int nc_id); /* I [id] File ID of fl_out_tmp */
+
+void 
+nco_fl_rm /* [fnc] Remove file */
+(char *fl_nm); /* I [sng] File to be removed */
 
-void
-nco_fl_cmp_err_chk(void); /* [fnc] Perform error checking on file */
 
 #ifdef __cplusplus
 } /* end extern "C" */
