@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_lmt.h,v 1.6 2002/12/30 02:56:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_lmt.h,v 1.18 2004/06/18 23:12:29 zender Exp $ */
 
 /* Purpose: Hyperslab limits */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -23,11 +23,14 @@
 #include <string.h> /* strcmp. . . */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* netCDF3.0 wrapper functions */
+#include <netcdf.h> /* netCDF definitions and C library */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
+#ifdef ENABLE_UDUNITS
+#include <udunits.h> /* Unidata units library */
+#endif /* !ENABLE_UDUNITS */
 
 /* Personal headers */
-#include "nco.h" /* NCO definitions */
+#include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_cnf_typ.h" /* Conform variable types */
 #include "nco_ctl.h" /* Program flow control functions */
 #include "nco_lst_utl.h" /* List utilities */
@@ -37,12 +40,18 @@
 extern "C" {
 #endif /* __cplusplus */
 
+enum lmt_typ{ /* [enm] Limit type */
+ lmt_crd_val, /* 0, Coordinate value limit */
+ lmt_dmn_idx, /* 1, Dimension index limit */
+ lmt_udu_sng /* 2, UDUnits string */
+}; /* end lmt_typ enum */
+  
 void
 nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications */
 (int nc_id, /* I [idx] netCDF file ID */
  lmt_sct *lmt_ptr, /* I/O [sct] Structure from nco_lmt_prs() or from nco_lmt_sct_mk() to hold dimension limit information */
  long cnt_crr, /* I [nbr] Number of valid records already processed (only used for record dimensions in multi-file operators) */
- bool FORTRAN_STYLE); /* I [flg] Hyperslab indices obey Fortran convention */
+ bool FORTRAN_IDX_CNV); /* I [flg] Hyperslab indices obey Fortran convention */
 
 lmt_sct * /* O [sct] Structure with user-specified strings for min and max limits */
 nco_lmt_prs /* [fnc] Create limit structures with name, min_sng, max_sng elements */
@@ -55,7 +64,18 @@ nco_lmt_sct_mk /* [fnc] Create stand-alone limit structure for given dimension *
  const int dmn_id, /* I [idx] ID of dimension for this limit structure */
  const lmt_sct * const lmt, /* I [sct] Array of limit structures from nco_lmt_evl() */
  int lmt_nbr, /* I [nbr] Number of limit structures */
- const bool FORTRAN_STYLE); /* I [flg] Hyperslab indices obey Fortran convention */
+ const bool FORTRAN_IDX_CNV); /* I [flg] Hyperslab indices obey Fortran convention */
+
+int /* [rcd] Successful conversion returns 0 */
+nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
+(const int nc_id, /* I [idx] netCDF file ID */
+ const int dmn_id, /* I [idx] netCDF dimension ID */
+ char *lmt_sng, /* I [ptr] Limit string */
+ double *lmt_val); /* O [val] Limit coordinate value */ 
+
+int /* 0 [enum] returns the limit type */
+nco_lmt_typ /* [fnc] determine limit type */
+(char* sng);/* I [ptr] pointer to the limit string */
 
 #ifdef __cplusplus
 } /* end extern "C" */
