@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_prn.c,v 1.11 2002/12/30 02:56:15 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_prn.c,v 1.18 2004/09/24 00:54:14 zender Exp $ */
 
 /* Purpose: Printing variables, attributes, metadata */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include "nco_prn.h" /* Printing variables, attributes, metadata */
@@ -65,7 +65,7 @@ nco_prn_att /* [fnc] Print all attributes of single variable */
     (void)cast_void_nctype(att[idx].type,&att[idx].val);
 
     (void)strcpy(dlm_sng,", ");
-    (void)sprintf(att_sng,"%s%%s\n",nco_typ_fmt_sng(att[idx].type));
+    (void)sprintf(att_sng,"%s%%s",nco_typ_fmt_sng(att[idx].type));
     switch(att[idx].type){
     case NC_FLOAT:
       for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.fp[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
@@ -108,7 +108,7 @@ nco_prn_att /* [fnc] Print all attributes of single variable */
 
 } /* end nco_prn_att() */
 
-char * /* O [sng] sprintf() format string for type typ */
+const char * /* O [sng] sprintf() format string for type typ */
 nco_typ_fmt_sng /* [fnc] Provide sprintf() format string for specified type */
 (const nc_type typ) /* I [enm] netCDF type to provide format string for */
 {
@@ -135,7 +135,7 @@ nco_typ_fmt_sng /* [fnc] Provide sprintf() format string for specified type */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
 
-  /* Some C compilers, e.g., SGI cc, need return statement to end non-void functions */
+  /* Some compilers, e.g., SGI cc, need return statement to end non-void functions */
   return (char *)NULL;
 } /* end nco_typ_fmt_sng() */
 
@@ -214,11 +214,11 @@ nco_prn_var_dfn /* [fnc] Print variable metadata */
     } /* end loop over dim */
     (void)sprintf(sng_foo,"%li*nco_typ_lng(%s)",dim[idx].sz,nco_typ_sng(var_typ));
     (void)strcat(sz_sng,sng_foo);
-    (void)fprintf(stdout,"%s memory size is %s = %li*%zu = %li bytes\n",var_nm,sz_sng,var_sz,nco_typ_lng(var_typ),var_sz*nco_typ_lng(var_typ));
+    (void)fprintf(stdout,"%s memory size is %s = %li*%lu = %lu bytes\n",var_nm,sz_sng,var_sz,(unsigned long)nco_typ_lng(var_typ),(unsigned long)(var_sz*nco_typ_lng(var_typ)));
   }else{
     long var_sz=1L;
 
-    (void)fprintf(stdout,"%s memory size is %ld*nco_typ_lng(%s) = %ld*%zu = %ld bytes\n",var_nm,var_sz,nco_typ_sng(var_typ),var_sz,nco_typ_lng(var_typ),var_sz*nco_typ_lng(var_typ));
+    (void)fprintf(stdout,"%s memory size is %ld*nco_typ_lng(%s) = %ld*%lu = %ld bytes\n",var_nm,var_sz,nco_typ_sng(var_typ),var_sz,(unsigned long)nco_typ_lng(var_typ),(unsigned long)(var_sz*nco_typ_lng(var_typ)));
   } /* end if variable is a scalar */
   (void)fflush(stdout);
   
@@ -238,19 +238,20 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
  const lmt_sct * const lmt, /* I [sct] Dimension limits */
  const int lmt_nbr, /* I [nbr] number of dimensions with user-specified limits */
  char * const dlm_sng, /* I [sng] User-specified delimiter string, if any */
- const bool FORTRAN_STYLE, /* I [flg] Hyperslab indices obey Fortran convention */
- const bool PRINT_DIMENSIONAL_UNITS, /* I [flg] Print units attribute, if any */
+ const bool FORTRAN_IDX_CNV, /* I [flg] Hyperslab indices obey Fortran convention */
+ const bool PRN_DMN_UNITS, /* I [flg] Print units attribute, if any */
  const bool PRN_DMN_IDX_CRD_VAL) /* I [flg] Print dimension/coordinate indices/values */
 {
   /* Purpose: Print variable data 
      Routine truncates dimensions of printed output variable in accord with user-specified limits
-     fxm: routine does not correctly print hyperslabs which are wrapped, or which use a non-unity stride */
+     fxm: routine does not correctly print hyperslabs which are wrapped, or which use non-unity stride */
 
   bool SRD=False; /* Stride is non-unity */
   bool WRP=False; /* Coordinate is wrapped */
 
-  char *unit_sng="";
+  char nul_chr='\0';
   char var_sng[MAX_LEN_FMT_SNG];
+  char *unit_sng=NULL;
   
   int rcd=NC_NOERR; /* [rcd] Return code */
   int *dmn_id=NULL_CEWI;
@@ -269,9 +270,12 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
   dmn_sct *dim=NULL_CEWI;
   var_sct var;
 
-  /* Copy name into var structure for aesthetics. Unfortunately,
-     Solaris machines can overwrite var.nm with next nco_malloc(), so
-     continue to use var_nm for output just to be safe. */
+  /* Initialize units string, overwrite later if necessary */
+  unit_sng=&nul_chr;
+
+  /* Copy name into variable structure for aesthetics
+     Unfortunately, Solaris may overwrite var.nm with next nco_malloc(), 
+     so continue to use var_nm for output just to be safe. */
   var.nm=(char *)strdup(var_nm);
 
   /* Is requested variable in input file? */
@@ -362,7 +366,7 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
   for(idx=0;idx<var.nbr_dim;idx++) var.sz*=dmn_cnt[idx];
 
   /* Allocate enough space to hold variable */
-  var.val.vp=(void *)malloc(var.sz*nco_typ_lng(var.type));
+  var.val.vp=(void *)nco_malloc(var.sz*nco_typ_lng(var.type));
   if(var.val.vp == NULL){
     (void)fprintf(stderr,"%s: ERROR unable to malloc() %ld bytes for %s\n",prg_nm_get(),var.sz*nco_typ_lng(var.type),var.nm);
     nco_exit(EXIT_FAILURE);
@@ -380,16 +384,16 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
   /* Typecast pointer to values before access */
   (void)cast_void_nctype(var.type,&var.val);
 
-  if(PRINT_DIMENSIONAL_UNITS){
-    char units_nm[]="units"; /* [sng] Name of units attribute */
-    int rcd; /* [rcd] Return code */
+  if(PRN_DMN_UNITS){
+    const char units_nm[]="units"; /* [sng] Name of units attribute */
+    int rcd_lcl; /* [rcd] Return code */
     int att_id; /* [id] Attribute ID */
     long att_sz;
     nc_type att_typ;
 
     /* Does variable have character attribute named units_nm? */
-    rcd=nco_inq_attid_flg(in_id,var.id,units_nm,&att_id);
-    if(rcd == NC_NOERR){
+    rcd_lcl=nco_inq_attid_flg(in_id,var.id,units_nm,&att_id);
+    if(rcd_lcl == NC_NOERR){
       (void)nco_inq_att(in_id,var.id,units_nm,&att_typ,&att_sz);
       if(att_typ == NC_CHAR){
 	unit_sng=(char *)nco_malloc((att_sz+1)*nco_typ_lng(att_typ));
@@ -397,7 +401,7 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
 	unit_sng[(att_sz+1)*nco_typ_lng(att_typ)-1]='\0';
       } /* end if */
     } /* end if */
-  } /* end if PRINT_DIMENSIONAL_UNITS */
+  } /* end if PRN_DMN_UNITS */
 
   if(dlm_sng != NULL){
     /* Print each element with user-supplied formatting code */
@@ -480,7 +484,7 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
       for(idx=0;idx<var.nbr_dim;idx++) hyp_srt+=dmn_srt[idx]*hyp_mod[idx];
     } /* end if */
 
-    if(FORTRAN_STYLE){
+    if(FORTRAN_IDX_CNV){
       ftn_idx_off=1;
       arr_lft_dlm='(';
       arr_rgt_dlm=')';
@@ -502,7 +506,7 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
       /* Treat character arrays as strings if possible */
       if(var.type == NC_CHAR){
 	/* Do not print final dimension (C-convention) of character arrays */
-	if(FORTRAN_STYLE) dmn_idx_prn_srt=1; else dmn_nbr_prn=var.nbr_dim-1;
+	if(FORTRAN_IDX_CNV) dmn_idx_prn_srt=1; else dmn_nbr_prn=var.nbr_dim-1;
       } /* endif */
       
       dmn_sbs_ram[var.nbr_dim-1]=lmn%dmn_cnt[var.nbr_dim-1];
@@ -524,7 +528,7 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
 	for(idx=dmn_idx_prn_srt;idx<dmn_nbr_prn;idx++){
 	  
 	  /* Reverse dimension ordering for Fortran convention */
-	  if(FORTRAN_STYLE) dmn_idx=var.nbr_dim-1-idx; else dmn_idx=idx;
+	  if(FORTRAN_IDX_CNV) dmn_idx=var.nbr_dim-1-idx; else dmn_idx=idx;
 	  
 	  /* Printed dimension subscript includes indexing convention (C or Fortran) */
 	  dmn_sbs_prn=dmn_sbs_dsk[dmn_idx]+ftn_idx_off;
@@ -560,7 +564,7 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
 	/* Print all characters in last dimension each time penultimate dimension subscript changes to its start value
 	   Ironic that printing characters is much more tedious than numbers */
 	/* Search for NUL-termination within size of last dimension */
-	if(memchr((void *)(var.val.cp+lmn),'\0',dmn_cnt[var.nbr_dim-1])){
+	if(memchr((void *)(var.val.cp+lmn),'\0',(size_t)dmn_cnt[var.nbr_dim-1])){
 	  /* Memory region is NUL-terminated, i.e., a valid string */
 	  /* Print strings inside double quotes */
 	  (void)sprintf(var_sng,"%%s%c%%ld--%%ld%c=\"%%s\" %%s",arr_lft_dlm,arr_rgt_dlm);
