@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_flt.c,v 1.6 2002/12/30 02:56:15 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_flt.c,v 1.12 2004/07/06 19:37:35 zender Exp $ */
 
 /* Purpose: Float-precision arithmetic */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include "nco_rth_flt.h" /* Float-precision arithmetic */
@@ -14,6 +14,7 @@
    Names of these optional float functions end in "f", as specified by ANSI
    Create any needed float functions simply by coercing I/O of double versions
    MacOS X does not provide float versions of _any_ standard function! */
+/* fxm: TODO #37 inline these definitions? */
 #ifdef NEED_FMODF
 float fmodf(float x,float y){return (float)(fmod((double)x,(double)y));}
 #endif /* !NEED_FMODF */ 
@@ -56,6 +57,9 @@ float erff(float x){return (float)(erf((double)x));}
 #ifdef NEED_EXPF
 float expf(float x){return (float)(exp((double)x));}
 #endif /* !NEED_EXPF */ 
+#ifdef NEED_FABSF
+float fabsf(float x){return (float)(fabs((double)x));}
+#endif /* !NEED_FABSF */ 
 #ifdef NEED_FLOORF
 float floorf(float x){return (float)(floor((double)x));}
 #endif /* !NEED_FLOORF */ 
@@ -66,7 +70,10 @@ float gammaf(float x){return (float)(gamma((double)x));}
 float log10f(float x){return (float)(log10((double)x));}
 #endif /* !NEED_LOG10F */ 
 #ifdef NEED_LOGF
-float logf(float x){return (float)(log((double)x));}
+float logf(float x){
+  /* fxm TODO ncap57: Eventually remove this debugging statement and the include stdio that it requires */
+  /* (void)fprintf(stderr,"%s: DEBUG Using NCO-supplied function logf() from nco_rth_flt.c\n",prg_nm_get()); */
+  return (float)(log((double)x));}
 #endif /* !NEED_LOGF */ 
 #ifdef NEED_SINF
 float sinf(float x){return (float)(sin((double)x));}
@@ -83,4 +90,3 @@ float tanf(float x){return (float)(tan((double)x));}
 #ifdef NEED_TANHF
 float tanhf(float x){return (float)(tanh((double)x));}
 #endif /* !NEED_TANHF */ 
-
