@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_flt.h,v 1.9 2002/12/30 02:56:15 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_flt.h,v 1.18 2004/07/09 16:36:11 zender Exp $ */
 
 /* Purpose: Float-precision arithmetic */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -19,9 +19,12 @@
 /* Standard header files */
 #include <math.h> /* sin cos cos sin 3.14159 */
 
+/* fxm stdio only needed for TODO ncap57 */
+#include <stdio.h> /* stderr, FILE, NULL, etc. */
+
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* netCDF3.0 wrapper functions */
+#include <netcdf.h> /* netCDF definitions and C library */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 
 /* Personal headers */
 
@@ -29,16 +32,16 @@
 extern "C" {
 #endif /* __cplusplus */
 
-  /* Math float prototypes required by AIX, Solaris, but not by Linux, IRIX */
-  /* Basic math: acos, asin, atan, cos, exp, log, log10, sin, sqrt, tan */
+#ifndef HPUX
+  /* Math float prototypes required by AIX, Solaris, but not by Linux, IRIX
+     20040708: HP-UX does not like these */
+  /* Basic math: acos, asin, atan, cos, exp, fabs, log, log10, pow, sin, sqrt, tan */
   float acosf(float);
   float asinf(float);
   float atanf(float);
   float cosf(float);
-  float erff(float);
-  float erfcf(float);
   float expf(float);
-  float gammaf(float);
+  float fabsf(float);
   float logf(float);
   float log10f(float);
   float powf(float,float);
@@ -46,6 +49,11 @@ extern "C" {
   float sqrtf(float);
   float tanf(float);
 
+  /* Advanced math: erf, erfc, gamma */
+  float erff(float);
+  float erfcf(float);
+  float gammaf(float);
+
   /* Hyperbolic trigonometric: acosh, asinh, atanh, cosh, sinh, tanh */
   float acoshf(float);
   float asinhf(float);
@@ -64,6 +72,8 @@ extern "C" {
   float roundf(float);
   float truncf(float);
 
+#endif /* HPUX */
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif /* __cplusplus */
