@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_scl_utl.c,v 1.8 2002/12/30 02:56:15 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_scl_utl.c,v 1.13 2004/09/03 23:06:47 zender Exp $ */
 
 /* Purpose: Scalar utilities */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include "nco_scl_utl.h" /* Scalar utilities */
@@ -15,9 +15,6 @@ scl_dbl_mk_var /* [fnc] Convert scalar double into netCDF variable */
   /* Purpose: Convert scalar double into netCDF variable
      Routine duplicates most functions of nco_var_fll() 
      Both functions should share as much initialization code as possible */
-
-  static char *var_nm="Internally generated variable";
-
   var_sct *var;
 
   var=(var_sct *)nco_malloc(sizeof(var_sct));
@@ -26,11 +23,11 @@ scl_dbl_mk_var /* [fnc] Convert scalar double into netCDF variable */
   (void)var_dfl_set(var); /* [fnc] Set defaults for each member of variable structure */
 
   /* Overwrite defaults with values appropriate for artificial variable */
-  var->nm=(char *)strdup(var_nm);
+  var->nm=(char *)strdup("Internally generated variable");
   var->nbr_dim=0;
   var->type=NC_DOUBLE;
   var->val.vp=(void *)nco_malloc(nco_typ_lng(var->type));
-  (void)memcpy((void *)var->val.vp,(void *)(&val),nco_typ_lng(var->type));
+  (void)memcpy((void *)var->val.vp,(const void *)(&val),nco_typ_lng(var->type));
 
   return var;
 } /* end scl_dbl_mk_var() */
@@ -74,9 +71,6 @@ scl_ptr_mk_var /* [fnc] Convert void pointer to scalar of any type into NCO vari
   /* Purpose: Convert void pointer to scalar of any type into NCO variable
      Routine duplicates many functions of nco_var_fll() 
      Both functions should share as much initialization code as possible */
-
-  static char *var_nm="Internally generated variable";
-
   var_sct *var;
   
   var=(var_sct *)nco_malloc(sizeof(var_sct));
@@ -85,7 +79,7 @@ scl_ptr_mk_var /* [fnc] Convert void pointer to scalar of any type into NCO vari
   (void)var_dfl_set(var); /* [fnc] Set defaults for each member of variable structure */
   
   /* Overwrite defaults with values appropriate for artificial variable */
-  var->nm=(char *)strdup(var_nm);
+  var->nm=(char *)strdup("Internally generated variable");
   var->nbr_dim=0;
   var->type=val_typ;
   /* Allocate new space here so that variable can eventually be deleted 
@@ -119,7 +113,7 @@ ptr_unn_2_scl_dbl /* [fnc] Convert first element of NCO variable to a scalar dou
   
   /* Valid memory address exists */
   ptr_unn_scl_dbl.vp=(void *)nco_malloc(nco_typ_lng(NC_DOUBLE)); /* [unn] Pointer union to double precision value of first element */
-  (void)val_conform_type(type,val,NC_DOUBLE,ptr_unn_scl_dbl);
+  (void)nco_val_cnf_typ(type,val,NC_DOUBLE,ptr_unn_scl_dbl);
   scl_dbl=ptr_unn_scl_dbl.dp[0];
   ptr_unn_scl_dbl.vp=nco_free(ptr_unn_scl_dbl.vp);
 
