@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_scm.c,v 1.8 2003/01/01 02:42:48 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_scm.c,v 1.20 2004/05/13 18:18:59 zender Exp $ */
 
 /* Purpose: Software configuration management */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include "nco_scm.h" /* Software configuration management */
@@ -25,8 +25,9 @@ cvs_vrs_prs(void) /* [fnc] Return CVS version string */
   char *nco_sng_ptr=NULL;
   char *usc_1_ptr=NULL;
   char *usc_2_ptr=NULL;
-  char cvs_Name[]="$Name: nco-2_7_1 $";
-  char nco_sng[]="nco"; 
+
+  const char cvs_Name[]="$Name:  $";
+  const char nco_sng[]="nco"; 
 
   int cvs_nm_sng_len;
   int cvs_vrs_sng_len;
@@ -53,26 +54,26 @@ cvs_vrs_prs(void) /* [fnc] Return CVS version string */
     int day;
     int yr;
     struct tm *gmt_tm;
-    time_t clock;
+    time_t time_crr_time_t;
 
-    clock=time((time_t *)NULL);
-    gmt_tm=gmtime(&clock); 
+    time_crr_time_t=time((time_t *)NULL);
+    gmt_tm=gmtime(&time_crr_time_t); 
     /* localtime() gives YYYYMMDD in MDT, but this conflicts with CVS, which uses GMT */
-    /*    gmt_tm=localtime(&clock); */
+    /*    gmt_tm=localtime(&time_crr_time_t); */
 
     mth=gmt_tm->tm_mon+1;
     day=gmt_tm->tm_mday;
     yr=gmt_tm->tm_year+1900;
 
     cvs_vrs_sng_len=4+2+2;
-    cvs_vrs_sng=(char *)nco_malloc(cvs_vrs_sng_len+1);
+    cvs_vrs_sng=(char *)nco_malloc((size_t)cvs_vrs_sng_len+1);
     (void)sprintf(cvs_vrs_sng,"%04i%02i%02i",yr,mth,day);
     return cvs_vrs_sng;
   } /* endif dly_snp */
 
   /* cvs_nm_sng is, e.g., "nco1_1" */
-  cvs_nm_sng=(char *)nco_malloc(cvs_nm_sng_len+1);
-  strncpy(cvs_nm_sng,cvs_Name+7,cvs_nm_sng_len); /* 7 is strlen("$Name: ") */
+  cvs_nm_sng=(char *)nco_malloc((size_t)cvs_nm_sng_len+1);
+  strncpy(cvs_nm_sng,cvs_Name+7,(size_t)cvs_nm_sng_len); /* 7 is strlen("$Name: ") */
   cvs_nm_sng[cvs_nm_sng_len]='\0';
 
   /* cvs_vrs_sng is, e.g., "1.1" */
@@ -85,8 +86,8 @@ cvs_vrs_prs(void) /* [fnc] Return CVS version string */
   if(usc_1_ptr == NULL)(void)fprintf(stderr,"%s: WARNING cvs_vrs_prs() reports usc_1_ptr == NULL\n",prg_nm_get());
   cvs_mjr_vrs_len=(int)(usc_1_ptr-dsh_ptr)-1; /* NB: cast pointer to int before subtracting */
   usc_2_ptr=strstr(usc_1_ptr+1,"_");
-  cvs_mjr_vrs_sng=(char *)nco_malloc(cvs_mjr_vrs_len+1);
-  cvs_mjr_vrs_sng=strncpy(cvs_mjr_vrs_sng,cvs_nm_sng+nco_sng_len+1,cvs_mjr_vrs_len);
+  cvs_mjr_vrs_sng=(char *)nco_malloc((size_t)cvs_mjr_vrs_len+1);
+  cvs_mjr_vrs_sng=strncpy(cvs_mjr_vrs_sng,cvs_nm_sng+nco_sng_len+1,(size_t)cvs_mjr_vrs_len);
   cvs_mjr_vrs_sng[cvs_mjr_vrs_len]='\0';
   cvs_mjr_vrs=strtol(cvs_mjr_vrs_sng,(char **)NULL,10);
   if(usc_2_ptr == NULL){
@@ -98,16 +99,16 @@ cvs_vrs_prs(void) /* [fnc] Return CVS version string */
     cvs_pch_vrs_len=cvs_nm_sng_len-cvs_mjr_vrs_len-1-cvs_mnr_vrs_len-1;
     cvs_vrs_sng_len=cvs_mjr_vrs_len+1+cvs_mnr_vrs_len+1+cvs_pch_vrs_len;
   } /* end else */
-  cvs_mnr_vrs_sng=(char *)nco_malloc(cvs_mnr_vrs_len+1);
-  cvs_mnr_vrs_sng=strncpy(cvs_mnr_vrs_sng,usc_1_ptr+1,cvs_mnr_vrs_len);
+  cvs_mnr_vrs_sng=(char *)nco_malloc((size_t)cvs_mnr_vrs_len+1);
+  cvs_mnr_vrs_sng=strncpy(cvs_mnr_vrs_sng,usc_1_ptr+1,(size_t)cvs_mnr_vrs_len);
   cvs_mnr_vrs_sng[cvs_mnr_vrs_len]='\0';
   cvs_mnr_vrs=strtol(cvs_mnr_vrs_sng,(char **)NULL,10);
 
-  cvs_pch_vrs_sng=(char *)nco_malloc(cvs_pch_vrs_len+1);
+  cvs_pch_vrs_sng=(char *)nco_malloc((size_t)cvs_pch_vrs_len+1);
   cvs_pch_vrs_sng[cvs_pch_vrs_len]='\0';
-  cvs_vrs_sng=(char *)nco_malloc(cvs_vrs_sng_len+1);
+  cvs_vrs_sng=(char *)nco_malloc((size_t)cvs_vrs_sng_len+1);
   if(usc_2_ptr != NULL){
-    cvs_pch_vrs_sng=strncpy(cvs_pch_vrs_sng,usc_2_ptr+1,cvs_pch_vrs_len);
+    cvs_pch_vrs_sng=strncpy(cvs_pch_vrs_sng,usc_2_ptr+1,(size_t)cvs_pch_vrs_len);
     cvs_pch_vrs=strtol(cvs_pch_vrs_sng,(char **)NULL,10);
     (void)sprintf(cvs_vrs_sng,"%li.%li.%li",cvs_mjr_vrs,cvs_mnr_vrs,cvs_pch_vrs);
   }else{
@@ -159,7 +160,7 @@ copyright_prn /* [fnc] Print copyright notice */
     /* CVS_Id is defined */
     date_cvs_lng=10;
     date_cvs=(char *)nco_malloc((date_cvs_lng+1)*sizeof(char));
-    (void)strncpy(date_cvs,strchr(CVS_Id,'/')-4,date_cvs_lng);
+    (void)strncpy(date_cvs,strchr(CVS_Id,'/')-4,(size_t)date_cvs_lng);
     date_cvs[date_cvs_lng]='\0';
   }else{
     /* CVS_Id is undefined */
@@ -170,7 +171,7 @@ copyright_prn /* [fnc] Print copyright notice */
     /* CVS_Revision is defined */
     vrs_cvs_lng=strrchr(CVS_Revision,'$')-strchr(CVS_Revision,':')-3;
     vrs_rcs=(char *)nco_malloc((vrs_cvs_lng+1)*sizeof(char));
-    (void)strncpy(vrs_rcs,strchr(CVS_Revision,':')+2,vrs_cvs_lng);
+    (void)strncpy(vrs_rcs,strchr(CVS_Revision,':')+2,(size_t)vrs_cvs_lng);
     vrs_rcs[vrs_cvs_lng]='\0';
   }else{
     /* CVS_Revision is undefined */
@@ -180,29 +181,20 @@ copyright_prn /* [fnc] Print copyright notice */
   vrs_cvs=cvs_vrs_prs();
 
   if(strlen(CVS_Id) > strlen("*Id*")){
-    (void)fprintf(stderr,"%s version %s last modified %s built %s on %s by %s\n",prg_nm_get(),vrs_cvs,date_cvs,date_cpp,hst_cpp,usr_cpp);
+    (void)fprintf(stderr,"NCO netCDF Operators version %s last modified %s built %s on %s by %s\n",vrs_cpp,date_cvs,date_cpp,hst_cpp,usr_cpp);
   }else{
-    (void)fprintf(stderr,"%s version %s built %s on %s by %s\n",prg_nm_get(),vrs_cpp,date_cpp,hst_cpp,usr_cpp);
+    (void)fprintf(stderr,"NCO netCDF Operators version %s built %s on %s by %s\n",vrs_cpp,date_cpp,hst_cpp,usr_cpp);
+  } /* endif */
+  (void)fprintf(stderr,"Copyright (C) 1995--2004 Charlie Zender\n");
+  if(strlen(CVS_Id) > strlen("*Id*")){
+    (void)fprintf(stderr,"%s version %s\n",prg_nm_get(),vrs_cvs);
+  }else{
+    (void)fprintf(stderr,"%s version %s\n",prg_nm_get(),vrs_cpp);
   } /* endif */
-
-  /*  (void)fprintf(stderr,"NCO netCDF Operators version %s by Charlie Zender\n",vrs_cvs);
-  (void)fprintf(stderr,"%s version %s (%s) \"%s\"\n",prg_nm_get(),vrs_rcs,date_cvs,nmn_get());
-  (void)fprintf(stderr,"Copyright 1995--1999 University Corporation for Atmospheric Research\n");
-  (void)fprintf(stderr,"Portions copyright 1999--2000 Regents of the University of California\n"); */
-
-  (void)fprintf(stderr,"NCO netCDF Operators version %s\n",vrs_cvs);
-  (void)fprintf(stderr,"Copyright (C) 1995--2003 Charlie Zender\n");
-  (void)fprintf(stderr,"%s version %s (%s) \"%s\"\n",prg_nm_get(),vrs_rcs,date_cvs,nmn_get());
   (void)fprintf(stdout,"NCO is free software and comes with ABSOLUTELY NO WARRANTY\nNCO is distributed under the terms of the GNU General Public License\n");
 
+  date_cvs=(char *)nco_free(date_cvs);
   vrs_rcs=(char *)nco_free(vrs_rcs);
   vrs_cvs=(char *)nco_free(vrs_cvs);
 } /* end copyright_prn() */
 
-char * /* O [sng] Mnemonic that describes current NCO version */
-nmn_get(void) /* [fnc] Return mnemonic that describes current NCO version */
-{ 
-  /* Purpose: Return mnemonic that describes current NCO version */
-  return "The ocean view from Diedrich's Coffee House in Laguna Beach is rippling with pre-storm activity";
-} /* end nmn_get() */
-
