@@ -1,14 +1,14 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_sng_utl.c,v 1.10 2002/12/30 02:56:15 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_sng_utl.c,v 1.15 2004/06/20 20:00:11 zender Exp $ */
 
 /* Purpose: String utilities */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include "nco_sng_utl.h" /* String utilities */
 
-#ifndef HAVE_STRCASECMP
+#ifdef NEED_STRCASECMP
 int /* O [enm] [-1,0,1] sng_1 [<,=,>] sng_2 */
 strcasecmp /* [fnc] Lexicographical case-insensitive string comparison */
 (const char *sng_1, /* I [sng] First string */
@@ -24,9 +24,9 @@ strcasecmp /* [fnc] Lexicographical case-insensitive string comparison */
     if(chr_1 == 0) return 0;
   } /* end while */
 } /* end strcasecmp() */
-#endif /* HAVE_STRCASECMP */
+#endif /* !NEED_STRCASECMP */
 
-#ifndef HAVE_STRDUP
+#ifdef NEED_STRDUP
 char * /* [sng] Copy of input string */
 strdup /* [fnc] Duplicate string */
 (char *sng_in) /* [sng] String to duplicate */
@@ -34,12 +34,13 @@ strdup /* [fnc] Duplicate string */
   /* Purpose: Provide strdup() for broken systems 
      Input string must be NUL-terminated */
   int sng_lng=strlen(sng_in)+1;
-  /* Use normal malloc() not nco_malloc() since strdup() is system function */
-  char *sng_out=(char *)malloc(sng_lng*sizeof(char));
+  /* Use nco_malloc() even though strdup() is system function 
+     This ensures all NCO code goes through nco_malloc()  */
+  char *sng_out=(char *)nco_malloc(sng_lng*sizeof(char));
   if(sng_out) strcpy(sng_out,sng_in);
   return sng_out;
 } /* end strdup() */
-#endif /* HAVE_STRDUP */
+#endif /* !NEED_STRDUP */
 
 char * /* O [sng] Parsed command line */
 nco_cmd_ln_sng /* [fnc] Re-construct command line from arguments */
@@ -77,13 +78,15 @@ sng_ascii_trn /* [fnc] Replace C language '\X' escape codes in string with ASCII
   /* Purpose: Replace C language '\X' escape codes in string with ASCII bytes 
      Return number of escape sequences found and actually translated
      This should be same as number of bytes by which string length has shrunk
-     For example, consecutive characters "\n" are translated into ASCII '\n' = 10 which diminishes string length by 1
+     For example, consecutive characters "\n" are translated into ASCII '\n' = 10
+     Each such translation diminishes string length by one
      Function works for arbitrary number of escape codes in input string
      The escape sequence for NUL itself, \0, causes a warning and is not translated
      Input string must be NUL-terminated or NULL
-     Translation is done in place, so if original string is required, it should be copied prior to calling sng_ascii_trn()
-     This procedure can only diminish, not lengthen, size of input string
-     Therefore it may be performed in place safely without the need to operate on a copy of the string
+     This procedure can only diminish, not lengthen, input string size
+     Therefore it may be performed in place safely, i.e., not string copy is necessary
+     Translation is done in place so copy original prior to calling sng_ascii_trn()!!!
+
      Address pointed to by sng does not change, but memory at that address is altered
      when characters are "moved to the left" if C language escape sequences are embedded.
      Thus string length may shrink */
@@ -97,7 +100,7 @@ sng_ascii_trn /* [fnc] Replace C language '\X' escape codes in string with ASCII
   int trn_nbr=0; /* Number of escape sequences translated */
   
   /* ncatted allows character attributes of 0 length
-     Such "strings" do not even have a NUL-terminator and so may not safely be tested by strchr() */
+     Such "strings" do not have NUL-terminator and so may not safely be tested by strchr() */
   if(sng == NULL) return trn_nbr;
   
   /* C language '\X' escape codes are always preceded by a backslash */
