@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_avg.c,v 1.14 2002/12/30 02:56:15 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_avg.c,v 1.20 2004/09/05 22:59:03 zender Exp $ */
 
 /* Purpose: Average variables */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include "nco_var_avg.h" /* Average variables */
@@ -16,33 +16,30 @@ nco_var_avg /* [fnc] reduce given variable over specified dimensions */
  const int nco_op_typ) /* I [enm] Operation type, default is average */
 {
   /* Threads: Routine is thread safe and calls no unsafe routines */
-  /* Routine to reduce given variable over specified dimensions. 
-     "Reduce" means to reduce the rank of variable by performing an arithmetic operation
-     The default operation is to average, but nco_op_typ can also be min, max, etc.
-     The input variable structure is destroyed and the routine returns the resized, partially reduced variable
-     For some operations, such as min, max, ttl, the variable returned by nco_var_avg() is complete and need not be further processed
-     But to complete the averaging operation, the output variable must be normalized by its tally array
+  /* Purpose: Reduce given variable over specified dimensions 
+     "Reduce" means to reduce rank of variable by performing an arithmetic operation
+     Output variable is duplicate of input variable, except for averaging dimensions
+     Default operation is averaging, but nco_op_typ can also be min, max, etc.
+     nco_var_avg() overwrites contents, if any, of tally array with number of valid reduction operations
+
+     Input variable structure is destroyed and routine returns resized, partially reduced variable
+     For some operations, such as min, max, ttl, variable returned by nco_var_avg() is complete and need not be further processed
+     For averaging operation, output variable must be normalized by its tally array
      In other words, nco_var_nrm() should be called subsequently if normalization is desired
-     Normalization is not done internally to nco_var_avg() in order to allow the user more flexibility
-  */ 
+     Normalization is not done internally to nco_var_avg() to allow user more flexibility */ 
 
-  /* Create output variable as a duplicate of the input variable, except for dimensions which are to be averaged over */
-
-  /* nco_var_avg() overwrites contents, if any, of tally array with number of valid reduction operations */
-
-  /* There are three variables to keep track of in this routine, their abbreviations are:
+  /* Routine keeps track of three variables whose abbreviations are:
      var: Input variable (already hyperslabbed)
-     avg: A contiguous arrangement of all elements of var that contribute to a single element of fix (a quasi-hyperslab)
-     fix: Output (averaged) variable
-   */
+     avg: Contiguous arrangement of all elements of var contributing to a single element of fix (a quasi-hyperslab)
+     fix: Output (averaged) variable */
 
   dmn_sct **dmn_avg;
   dmn_sct **dmn_fix;
 
   int idx_avg_var[NC_MAX_DIMS];
-  /*  int idx_var_avg[NC_MAX_DIMS];*/
+  /*  int idx_var_avg[NC_MAX_DIMS];*/ /* Variable is unused but instructive anyway */
   int idx_fix_var[NC_MAX_DIMS];
-  /*  int idx_var_fix[NC_MAX_DIMS];*/
+  /*  int idx_var_fix[NC_MAX_DIMS];*/ /* Variable is unused but instructive anyway */
   int idx;
   int idx_dmn;
   int nbr_dmn_avg;
@@ -157,7 +154,7 @@ nco_var_avg /* [fnc] reduce given variable over specified dimensions */
       val_sz_byte=nco_typ_lng(fix->type);
       val=(char *)fix->val.vp;
       for(idx=0;idx<fix_sz;idx++,val+=val_sz_byte)
-	if(!memcmp(val,mss_val,val_sz_byte)) fix_tally[idx]=0L;
+	if(!memcmp(val,mss_val,(size_t)val_sz_byte)) fix_tally[idx]=0L;
     } /* fix->has_mss_val */
   } /* end if avg_sz == 1L */
 
@@ -187,19 +184,19 @@ nco_var_avg /* [fnc] reduce given variable over specified dimensions */
     var_cp=(char *)var->val.vp;
     var_sz=var->sz;
     
-    /* Reuse the existing value buffer (it is of size var_sz, created by nco_var_dpl())*/
+    /* Reuse existing value buffer (it is of size var_sz, created by nco_var_dpl()) */
     avg_val=fix->val;
     avg_cp=(char *)avg_val.vp;
-    /* Create a new value buffer for output (averaged) size */
+    /* Create new value buffer for output (averaged) size */
     fix->val.vp=(void *)nco_malloc(fix_sz*nco_typ_lng(fix->type));
-    /* Resize (or just plain allocate) the tally array */
+    /* Resize (or just plain allocate) tally array */
     fix->tally=(long *)nco_realloc(fix->tally,fix_sz*sizeof(long));
 
-    /* Re-initialize value and tally arrays */
+    /* Initialize value and tally arrays */
     (void)nco_zero_long(fix_sz,fix->tally);
     (void)nco_var_zero(fix->type,fix_sz,fix->val);
   
-    /* Compute map for each dimension of variable */
+    /* Compute map for each dimension of input variable */
     for(idx=0;idx<nbr_dmn_var;idx++) dmn_var_map[idx]=1L;
     for(idx=0;idx<nbr_dmn_var-1;idx++)
       for(idx_dmn=idx+1;idx_dmn<nbr_dmn_var;idx_dmn++)
@@ -217,7 +214,7 @@ nco_var_avg /* [fnc] reduce given variable over specified dimensions */
       for(idx_dmn=idx+1;idx_dmn<nbr_dmn_avg;idx_dmn++)
 	dmn_avg_map[idx]*=dmn_avg[idx_dmn]->cnt;
     
-    /* var_lmn is the offset into 1-D array */
+    /* var_lmn is offset into 1-D array */
     for(var_lmn=0;var_lmn<var_sz;var_lmn++){
 
       /* dmn_ss are corresponding indices (subscripts) into N-D array */
@@ -231,13 +228,12 @@ nco_var_avg /* [fnc] reduce given variable over specified dimensions */
       fix_lmn=0L;
       for(idx=0;idx<nbr_dmn_fix;idx++) fix_lmn+=dmn_ss[idx_fix_var[idx]]*dmn_fix_map[idx];
       
-      /* Map N-D array indices into a 1-D offset from offset of its group */
+      /* Map N-D array indices into 1-D offset from group offset */
       avg_lmn=0L;
       for(idx=0;idx<nbr_dmn_avg;idx++) avg_lmn+=dmn_ss[idx_avg_var[idx]]*dmn_avg_map[idx];
       
       /* Copy current element in input array into its slot in sorted avg_val */
-      (void)memcpy(avg_cp+(fix_lmn*avg_sz+avg_lmn)*typ_sz,var_cp+var_lmn*typ_sz,typ_sz);
-      
+      (void)memcpy(avg_cp+(fix_lmn*avg_sz+avg_lmn)*typ_sz,var_cp+var_lmn*typ_sz,(size_t)typ_sz);
     } /* end loop over var_lmn */
     
     /* Input data are now sorted and stored (in avg_val) in blocks (of length avg_sz)
@@ -302,10 +298,14 @@ nco_var_avg_reduce_ttl /* [fnc] Sum blocks of op1 into each element of op2 */
   /* Each operation has GNUC and non-GNUC blocks:
      GNUC: Utilize (non-ANSI-compliant) compiler support for local automatic arrays
      This results in more elegent loop structure and, theoretically, in faster performance
+     20040225: In reality, the GNUC non-ANSI blocks fail on some large files
+     This may be because they allocate significant local storage on the stack
+     
      non-GNUC: Fully ANSI-compliant structure
      Fortran: Support deprecated */
 
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
   long idx_op1;
 #endif /* __GNUC__ */
   const long sz_blk=sz_op1/sz_op2;
@@ -337,7 +337,8 @@ nco_var_avg_reduce_ttl /* [fnc] Sum blocks of op1 into each element of op2 */
 
   switch(type){
   case NC_FLOAT:
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){ 
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
@@ -385,7 +386,8 @@ nco_var_avg_reduce_ttl /* [fnc] Sum blocks of op1 into each element of op2 */
 #endif /* __GNUC__ */
     break;
   case NC_DOUBLE:
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
@@ -433,7 +435,8 @@ nco_var_avg_reduce_ttl /* [fnc] Sum blocks of op1 into each element of op2 */
 #endif /* __GNUC__ */
     break;
   case NC_INT:
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
@@ -481,7 +484,8 @@ nco_var_avg_reduce_ttl /* [fnc] Sum blocks of op1 into each element of op2 */
 #endif /* __GNUC__ */
     break;
   case NC_SHORT:
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
@@ -563,7 +567,8 @@ nco_var_avg_reduce_min /* [fnc] Place minimum of op1 blocks into each element of
      See nco_var_avg_reduce_ttl() for more algorithmic documentation
      nco_var_avg_reduce_max() is derived from nco_var_avg_reduce_min() */
 
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
   long idx_op1;
 #endif /* __GNUC__ */
   const long sz_blk=sz_op1/sz_op2;
@@ -577,7 +582,7 @@ nco_var_avg_reduce_min /* [fnc] Place minimum of op1 blocks into each element of
   unsigned char mss_val_chr;
   signed char mss_val_byt;
   
-  bool flg_mss=False;
+  bool flg_mss=False; /* [flg] Block has valid (non-missing) values */
   
   /* Typecast pointer to values before access */
   (void)cast_void_nctype(type,&op1);
@@ -599,7 +604,8 @@ nco_var_avg_reduce_min /* [fnc] Place minimum of op1 blocks into each element of
   switch(type){
   case NC_FLOAT:
     
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){ 
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
@@ -653,7 +659,8 @@ nco_var_avg_reduce_min /* [fnc] Place minimum of op1 blocks into each element of
     break;
   case NC_DOUBLE:
     
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
@@ -705,7 +712,8 @@ nco_var_avg_reduce_min /* [fnc] Place minimum of op1 blocks into each element of
 #endif /* __GNUC__ */
     break;
   case NC_INT:
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
@@ -757,7 +765,8 @@ nco_var_avg_reduce_min /* [fnc] Place minimum of op1 blocks into each element of
 #endif /* __GNUC__ */
     break;
   case NC_SHORT:
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
@@ -843,7 +852,8 @@ nco_var_avg_reduce_max /* [fnc] Place maximum of op1 blocks into each element of
      See nco_var_avg_reduce_ttl() for more algorithmic documentation
      nco_var_avg_reduce_max() is derived from nco_var_avg_reduce_min() */
 
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
   long idx_op1;
 #endif /* __GNUC__ */
   const long sz_blk=sz_op1/sz_op2;
@@ -879,7 +889,8 @@ nco_var_avg_reduce_max /* [fnc] Place maximum of op1 blocks into each element of
   switch(type){
   case NC_FLOAT:
     
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){ 
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
@@ -933,7 +944,8 @@ nco_var_avg_reduce_max /* [fnc] Place maximum of op1 blocks into each element of
     break;
   case NC_DOUBLE:
     
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
@@ -985,7 +997,8 @@ nco_var_avg_reduce_max /* [fnc] Place maximum of op1 blocks into each element of
 #endif /* __GNUC__ */
     break;
   case NC_INT:
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
@@ -1037,7 +1050,8 @@ nco_var_avg_reduce_max /* [fnc] Place maximum of op1 blocks into each element of
 #endif /* __GNUC__ */
     break;
   case NC_SHORT:
-#ifndef __GNUC__
+#define FXM_NCO315 1
+#ifdef FXM_NCO315
     /* ANSI-compliant branch */
     if(!has_mss_val){
       for(idx_op2=0;idx_op2<sz_op2;idx_op2++){
