@@ -1,15 +1,15 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_lst.c,v 1.12 2002/12/30 02:56:15 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_lst.c,v 1.40 2004/09/08 17:56:23 zender Exp $ */
 
 /* Purpose: Variable list utilities */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include "nco_var_lst.h" /* Variable list utilities */
 
 nm_id_sct * /* O [sct] Variable extraction list */
-nco_var_lst_mk /* [fnc] Create variable extraction list */
+nco_var_lst_mk_old /* [fnc] Create variable extraction list */
 (const int nc_id, /* I [enm] netCDF file ID */
  const int nbr_var, /* I [nbr] Number of variables in input file */
  CST_X_PTR_CST_PTR_CST_Y(char,var_lst_in), /* I [sng] User-specified list of variable names */
@@ -21,7 +21,7 @@ nco_var_lst_mk /* [fnc] Create variable extraction list */
   int rcd=NC_NOERR; /* [rcd] Return code */
   int idx;
 
-  nm_id_sct *xtr_lst=NULL; /* xtr_lst can get realloc()'d from NULL with -c option */
+  nm_id_sct *xtr_lst=NULL; /* xtr_lst may bealloc()'d from NULL with -c option */
 
   if(*nbr_xtr > 0){
     /* If user named variables with -v option then check validity of user's list and find IDs */
@@ -55,8 +55,182 @@ nco_var_lst_mk /* [fnc] Create variable extraction list */
   } /* end else */
 
   return xtr_lst;
+} /* end nco_var_lst_mk_old() */
 
-} /* end nco_var_lst_mk() */
+nm_id_sct * /* O [sct] Variable extraction list */
+nco_var_lst_mk /* [fnc] Create variable extraction list using regular expressions */
+(const int nc_id, /* I [enm] netCDF file ID */
+ const int nbr_var, /* I [nbr] Number of variables in input file */
+ char **var_lst_in, /* I [sng] User-specified list of variable names */
+ const bool PROCESS_ALL_COORDINATES, /* I [flg] Process all coordinates */
+ int * const nbr_xtr) /* I/O [nbr] Number of variables in current extraction list */
+{
+  /* Purpose: Create variable extraction list with or without regular expressions */
+
+  int idx;
+  int jdx;
+  int nbr_tmp=0;
+  int nbr_match;
+
+  char *var_sng;
+
+  nm_id_sct *xtr_lst=NULL; /* xtr_lst may be alloc()'d from NULL with -c option */
+  nm_id_sct *in_lst=NULL; 
+  bool *in_bool=NULL;
+
+  /* Create list of all vars in input file */
+  char var_nm[NC_MAX_NAME];
+    
+  in_lst=(nm_id_sct *)nco_malloc(nbr_var*sizeof(nm_id_sct));
+  for(idx=0;idx<nbr_var;idx++){
+    /* Get name of each variable. */
+    (void)nco_inq_varname(nc_id,idx,var_nm);
+    in_lst[idx].nm=(char *)strdup(var_nm);
+    in_lst[idx].id=idx;
+  } /* end loop over idx */
+  
+    /* Return all variables if .. */
+  if(*nbr_xtr==0 && !PROCESS_ALL_COORDINATES){
+    *nbr_xtr=nbr_var;
+    return in_lst;
+  } /* end if */     
+      
+  /* intialize and alloc  bool array to all False */
+  in_bool=(bool *)nco_calloc((size_t)nbr_var,sizeof(bool));
+  
+  /* Loop through var_lst_in */
+  for(idx=0;idx<*nbr_xtr;idx++){
+     var_sng=var_lst_in[idx];
+
+     /* Convert pounds (#s) back to commas */
+     while(*var_sng){
+       if (*var_sng=='#') *var_sng=',';
+       var_sng++;
+     } /* end while */
+     var_sng=var_lst_in[idx];
+
+     /* If var_sng is regular expression... */
+     if(strpbrk(var_sng,".*^$\\[]()<>+?|{}")){
+       /* ...and regular expression library is present */
+#ifdef NCO_HAVE_REGEX_FUNCTIONALITY
+       nbr_match=nco_var_meta_search(nbr_var,in_lst,var_sng,in_bool);
+       if(nbr_match==0) (void)fprintf(stdout,"%s: WARNING: Regular expression \"%s\" does not match any variables\nHINT: http://nco.sf.net/nco.html#rx",prg_nm_get(),var_sng); 
+       continue;
+#else
+      (void)fprintf(stdout,"%s: ERROR: Sorry, wildcarding (extended regular expression matches to variables) is not available, so unable to compile regular expression \"%s\".\nHINT: Make sure libregex.a is on path and re-build NCO.\n",prg_nm_get(),var_sng);
+       nco_exit(EXIT_FAILURE);
+#endif /* NCO_HAVE_REGEX_FUNCTIONALITY */
+     } /* end if regular expression */
+    
+    /* Normal variable so search through var array */
+     for(jdx=0;jdx<nbr_var;jdx++)
+       if(!strcmp(var_sng,in_lst[jdx].nm)) break;
+       /* No matches found so die gently */
+     if(jdx==nbr_var){
+       (void)fprintf(stdout,"%s: ERROR nco_var_lst_mk() reports user-specified variable \"%s\" is not in input file\n",prg_nm_get(),var_sng); 
+       nco_exit(EXIT_FAILURE);
+     }else{
+       in_bool[jdx]=True;
+     } /* end else */
+  }
+  /* Create final list of vars using var list and bool array */
+
+  /* malloc() xtr_lst to maximium size(nbr_var) */
+  xtr_lst=(nm_id_sct *)nco_malloc(nbr_var*sizeof(nm_id_sct));
+  nbr_tmp=0;
+
+  for(idx=0;idx<nbr_var;idx++){
+    /* Copy var to output array */
+    if(in_bool[idx]){
+      xtr_lst[nbr_tmp].nm=(char *)strdup(in_lst[idx].nm);
+      xtr_lst[nbr_tmp].id=in_lst[idx].id;
+      nbr_tmp++;
+    } /* end if */
+    (void)nco_free(in_lst[idx].nm);
+  } /* end loop over var */
+
+  /* realloc() list to actual size */  
+  xtr_lst=(nm_id_sct *)nco_realloc(xtr_lst,nbr_tmp*sizeof(nm_id_sct));
+
+  (void)nco_free(in_lst);
+  (void)nco_free(in_bool);
+
+  *nbr_xtr=nbr_tmp;    
+  return xtr_lst;
+} /* end nco_var_lst_mk_meta() */
+
+/* Compile only if regular expression library is present */
+#ifdef NCO_HAVE_REGEX_FUNCTIONALITY
+
+int /* O [nbr] Number of matches found */
+nco_var_meta_search /* [fnc] Search for pattern matches in var string list */
+(int nbr_var, /* I [nbr] number of vars in srch_sng and size of in_bool */
+ nm_id_sct *in_lst, /* I [sct] List of all variables in input file (with IDs) */
+ char *rexp, /* I [sng] Regular expression pattern */
+ bool *in_bool) /* O [flg] Matched vars holder */
+{
+  /* If regular expression does not compile then program dies */
+  int idx;
+  int err_no;
+  int cflags;
+  int eflags;
+  int nbr_mtch=0;
+  size_t no_sub;
+  
+  regmatch_t *result;
+  regex_t *r;
+
+  r=(regex_t *)nco_malloc(sizeof(regex_t));
+
+  /* Choose RE_SYNTAX_POSIX_EXTENDED regular expression type */
+  cflags=(REG_EXTENDED | REG_NEWLINE);
+  /* Set execution flags */
+  eflags=0;
+
+  /* Compile regular expression */
+  if((err_no=regcomp(r,rexp,cflags)) != 0){ /* Compile regular expression */
+    char const * rx_err_sng;  
+    /* POSIX regcomp return error codes */
+    switch(err_no){
+    case REG_BADPAT: rx_err_sng="Invalid pattern."; break;  
+    case REG_ECOLLATE: rx_err_sng="Not implemented."; break;
+    case REG_ECTYPE: rx_err_sng="Invalid character class name."; break;
+    case REG_EESCAPE: rx_err_sng="Trailing backslash."; break;
+    case REG_ESUBREG: rx_err_sng="Invalid back reference."; break;
+    case REG_EBRACK: rx_err_sng="Unmatched left bracket."; break;
+    case REG_EPAREN: rx_err_sng="Parenthesis imbalance."; break;
+    case REG_EBRACE: rx_err_sng="Unmatched {."; break;
+    case REG_BADBR: rx_err_sng="Invalid contents of { }."; break;
+    case REG_ERANGE: rx_err_sng="Invalid range end."; break;
+    case REG_ESPACE: rx_err_sng="Ran out of memory."; break;
+    case REG_BADRPT: rx_err_sng="No preceding re for repetition op"; break;
+    default: rx_err_sng="Invalid pattern."; break;  
+    } /* end switch */
+    (void)fprintf(stdout,"%s: ERROR nco_var_meta_search() error in regular expression \"%s\" %s \n",prg_nm_get(),rexp,rx_err_sng); 
+    nco_exit(EXIT_FAILURE);
+  } /* end if err */
+
+  no_sub=r->re_nsub+1; /* How many matches are there in a line? */
+
+  /* Search string */
+  result=(regmatch_t *)nco_malloc(sizeof(regmatch_t)*no_sub);
+
+  /* Search each var string for matches */
+  for(idx=0;idx<nbr_var;idx++){  
+    if(!regexec(r, in_lst[idx].nm,no_sub,result,eflags)){
+      in_bool[idx]=True;
+      nbr_mtch++;
+    } /* end if */
+  } /* end loop over variables */
+
+  regfree(r); /* Free regular expression data structure */
+  (void *)nco_free(r);
+  (void *)nco_free(result);
+
+   return nbr_mtch;
+} /* end nco_var_meta_search() */
+
+#endif /* NCO_HAVE_REGEX_FUNCTIONALITY */
 
 nm_id_sct * /* O [sct] Extraction list */
 nco_var_lst_xcl /* [fnc] Convert exclusion list to extraction list */
@@ -111,7 +285,6 @@ nco_var_lst_xcl /* [fnc] Convert exclusion list to extraction list */
 nm_id_sct * /* O [sct] Extraction list */
 nco_var_lst_add_crd /* [fnc] Add all coordinates to extraction list */
 (const int nc_id, /* I [id] netCDF file ID */
- const int nbr_var, /* I [nbr] Number of variables in input file */
  const int nbr_dim, /* I [nbr] Number of dimensions in input file */
  nm_id_sct *xtr_lst, /* I/O [sct] Current extraction list (destroyed) */
  int * const nbr_xtr) /* I/O [nbr] Number of variables in current extraction list */
@@ -139,9 +312,9 @@ nco_var_lst_add_crd /* [fnc] Add all coordinates to extraction list */
 	if(crd_id == xtr_lst[lst_idx].id) break;
       } /* end loop over lst_idx */
       if(lst_idx == *nbr_xtr){
-	/* Coordinate is not already on the list, put it there. */
+	/* Coordinate is not already on the list, put it there */
 	if(*nbr_xtr == 0) xtr_lst=(nm_id_sct *)nco_malloc((*nbr_xtr+1)*sizeof(nm_id_sct)); else xtr_lst=(nm_id_sct *)nco_realloc((void *)xtr_lst,(*nbr_xtr+1)*sizeof(nm_id_sct));
-	/* According to the man page for realloc(), this should work even when xtr_lst == NULL */
+	/* According to man page for realloc(), this should work even when xtr_lst == NULL */
 /*	xtr_lst=(nm_id_sct *)nco_realloc((void *)xtr_lst,(*nbr_xtr+1)*sizeof(nm_id_sct));*/
 	xtr_lst[*nbr_xtr].nm=(char *)strdup(crd_nm);
 	xtr_lst[*nbr_xtr].id=crd_id;
@@ -151,7 +324,6 @@ nco_var_lst_add_crd /* [fnc] Add all coordinates to extraction list */
   } /* end loop over idx */
   
   return xtr_lst;
-  
 } /* end nco_var_lst_add_crd() */
 
 void
@@ -193,9 +365,11 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
 (var_sct * const * const var, /* I [sct] Variable list (input file) */
  var_sct * const * const var_out, /* I [sct] Variable list (output file) */
  const int nbr_var, /* I [nbr] Number of variables */
- const bool NCAR_CSM_FORMAT, /* I [flg] File adheres to NCAR CSM conventions */
+ const bool NCAR_CCSM_FORMAT, /* I [flg] File adheres to NCAR CCSM conventions */
+ const int nco_pck_map, /* I [enm] Packing map */
+ const int nco_pck_plc, /* I [enm] Packing policy */
  CST_X_PTR_CST_PTR_CST_Y(dmn_sct,dmn_xcl), /* I [sct] Dimensions not allowed in fixed variables */
- const int nbr_dmn_xcl, /* I [nbr] Number of excluded dimensions */
+ const int nbr_dmn_xcl, /* I [nbr] Number of altered dimensions */
  var_sct *** const var_fix_ptr, /* O [sct] Fixed-variables (input file) */
  var_sct *** const var_fix_out_ptr, /* O [sct] Fixed-variables (output file) */
  int * const nbr_var_fix, /* O [nbr] Number of fixed variables */
@@ -211,8 +385,8 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
   int prg; /* Program key */
 
   enum op_typ{
-    fix, /* 0 */
-    prc /* 1 */
+    fix, /* 0 [enm] Fix variable (operator alters neither data nor metadata) */
+    prc /* 1 [enm] Process variable (operator may alter data or metadata) */
   };
 
   int idx_dmn;
@@ -237,7 +411,7 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
   /* Find operation type for each variable: for now this is either fix or prc */
   for(idx=0;idx<nbr_var;idx++){
     
-    /* Initialize operation type */
+    /* Initialize operation type to processed. Change to fixed where warranted later. */
     var_op_typ[idx]=prc;
     var_nm=var[idx]->nm;
     var_type=var[idx]->type;
@@ -247,44 +421,75 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
     case ncap:
       var_op_typ[idx]=fix;
       break;
-    case ncra:
-      if(!var[idx]->is_rec_var) var_op_typ[idx]=fix;
+    case ncatted:
+      /* Do nothing */
+      break;
+    case ncbo:
+      if((var[idx]->is_crd_var) || (var_type == NC_CHAR) || (var_type == NC_BYTE)) var_op_typ[idx]=fix;
       break;
     case ncea:
       if((var[idx]->is_crd_var) || (var_type == NC_CHAR) || (var_type == NC_BYTE)) var_op_typ[idx]=fix;
       break;
-    case ncdiff:
-      if((var[idx]->is_crd_var) || (var_type == NC_CHAR) || (var_type == NC_BYTE)) var_op_typ[idx]=fix;
+    case ncecat:
+      if(var[idx]->is_crd_var) var_op_typ[idx]=fix;
       break;
     case ncflint:
       if((var_type == NC_CHAR) || (var_type == NC_BYTE) || (var[idx]->is_crd_var && !var[idx]->is_rec_var)) var_op_typ[idx]=fix;
       break;
-    case ncwa:
-      /* Process every variable containing an excluded (averaged) dimension */
-      for(idx_dmn=0;idx_dmn<var[idx]->nbr_dim;idx_dmn++){
-	for(idx_xcl=0;idx_xcl<nbr_dmn_xcl;idx_xcl++){
-	  if(var[idx]->dim[idx_dmn]->id == dmn_xcl[idx_xcl]->id) break;
-	} /* end loop over idx_xcl */
-	if(idx_xcl != nbr_dmn_xcl){
-	  var_op_typ[idx]=prc;
-	  break;
-	} /* end if */
-      } /* end loop over idx_dmn */
-      /* Variables which do not contain an excluded (averaged) dimension must be fixed */
-      if(idx_dmn == var[idx]->nbr_dim) var_op_typ[idx]=fix;
+    case ncks:
+      /* Do nothing */
+      break;
+    case ncra:
+      if(!var[idx]->is_rec_var) var_op_typ[idx]=fix;
       break;
     case ncrcat:
       if(!var[idx]->is_rec_var) var_op_typ[idx]=fix;
       break;
-    case ncecat:
-      if(var[idx]->is_crd_var) var_op_typ[idx]=fix;
+    case ncpdq:
+    case ncwa:
+      if(nco_pck_plc != nco_pck_plc_nil){
+	/* Packing operation requested
+	   Variables are processed for packing/unpacking operator unless ... */
+	if(
+	   /* ...unpacking requested for unpacked variable... */
+	   (nco_pck_plc == nco_pck_plc_upk && !var[idx]->pck_ram) ||
+	   /* ...or packing unpacked requested and variable is already packed... */
+	   (nco_pck_plc == nco_pck_plc_all_xst_att && var[idx]->pck_ram) ||
+	   /* ...or re-packing packed requested and variable is unpacked... */
+	   (nco_pck_plc == nco_pck_plc_xst_new_att && !var[idx]->pck_ram) ||
+	   /* ...or... */
+	   (
+	    /* ...any type of packing requested... */
+	    (nco_pck_plc == nco_pck_plc_all_new_att || 
+	    nco_pck_plc == nco_pck_plc_all_xst_att || 
+	    nco_pck_plc == nco_pck_plc_xst_new_att) &&
+	    /* ...yet map does not allow (re-)packing... */
+	    !nco_pck_plc_typ_get(nco_pck_map,var[idx]->typ_upk,(nc_type *)NULL)
+	    )
+	   )
+	  var_op_typ[idx]=fix;
+      }else{ /* endif packing operation requested */
+	/* Process every variable containing an altered (averaged, re-ordered, reversed) dimension */
+	for(idx_dmn=0;idx_dmn<var[idx]->nbr_dim;idx_dmn++){
+	  for(idx_xcl=0;idx_xcl<nbr_dmn_xcl;idx_xcl++){
+	    if(var[idx]->dim[idx_dmn]->id == dmn_xcl[idx_xcl]->id) break;
+	  } /* end loop over idx_xcl */
+	  if(idx_xcl != nbr_dmn_xcl){
+	    var_op_typ[idx]=prc;
+	    break;
+	  } /* end if */
+	} /* end loop over idx_dmn */
+	/* Fix variables with no altered (averaged, re-ordered, reversed) dimensions */
+	if(idx_dmn == var[idx]->nbr_dim) var_op_typ[idx]=fix;
+      } /* endif averaging or re-ordering */
       break;
+    default: nco_dfl_case_prg_id_err(); break;
     } /* end switch */
     
-    if(NCAR_CSM_FORMAT){
+    if(NCAR_CCSM_FORMAT){
       if(!strcmp(var_nm,"ntrm") || !strcmp(var_nm,"ntrn") || !strcmp(var_nm,"ntrk") || !strcmp(var_nm,"ndbase") || !strcmp(var_nm,"nsbase") || !strcmp(var_nm,"nbdate") || !strcmp(var_nm,"nbsec") || !strcmp(var_nm,"mdt") || !strcmp(var_nm,"mhisf")) var_op_typ[idx]=fix;
-      if(prg == ncdiff && (!strcmp(var_nm,"hyam") || !strcmp(var_nm,"hybm") || !strcmp(var_nm,"hyai") || !strcmp(var_nm,"hybi") || !strcmp(var_nm,"gw") || !strcmp(var_nm,"ORO") || !strcmp(var_nm,"date") || !strcmp(var_nm,"datesec"))) var_op_typ[idx]=fix;
-    } /* end if NCAR_CSM_FORMAT */
+      if(prg == ncbo && (!strcmp(var_nm,"hyam") || !strcmp(var_nm,"hybm") || !strcmp(var_nm,"hyai") || !strcmp(var_nm,"hybi") || !strcmp(var_nm,"gw") || !strcmp(var_nm,"ORO") || !strcmp(var_nm,"date") || !strcmp(var_nm,"datesec"))) var_op_typ[idx]=fix;
+    } /* end if NCAR_CCSM_FORMAT */
 
   } /* end loop over var */
 
@@ -299,11 +504,7 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
       var_prc[*nbr_var_prc]=var[idx];
       var_prc_out[*nbr_var_prc]=var_out[idx];
       ++*nbr_var_prc;
-/*      if(var[idx]->type != NC_FLOAT && var[idx]->type != NC_DOUBLE && prg != ncdiff && prg != ncrcat){
-	(void)fprintf(stderr,"%s: WARNING variable \"%s\" will be coerced from %s to NC_FLOAT\n",prg_nm_get(),var[idx]->nm,nco_typ_sng(var[idx]->type));
-	var_out[idx]->type=NC_FLOAT;
-      } *//* end if */
-      if(((var[idx]->type == NC_CHAR) || (var[idx]->type == NC_BYTE)) && ((prg != ncrcat) && (prg != ncecat))){
+      if(((var[idx]->type == NC_CHAR) || (var[idx]->type == NC_BYTE)) && ((prg != ncecat) && (prg != ncpdq) && (prg != ncrcat))){
 	(void)fprintf(stderr,"%s: WARNING Variable %s is of type %s, for which processing (i.e., averaging, differencing) is ill-defined\n",prg_nm_get(),var[idx]->nm,nco_typ_sng(var[idx]->type));
       } /* end if */
     } /* end else */
@@ -316,41 +517,52 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
   } /* end if */
 
   /* fxm: Remove ncap exception when finished with ncap list processing */
-  if(*nbr_var_prc==0 && prg != ncap){
+  /* fxm: ncpdq processes all variables when pakcing requested */
+  if(*nbr_var_prc == 0 && prg != ncap && prg != ncpdq){
     (void)fprintf(stdout,"%s: ERROR no variables fit criteria for processing\n",prg_nm_get());
     switch(prg_get()){
     case ncap:
-      (void)fprintf(stdout,"%s: HINT Extraction list must contain some derived fields\n",prg_nm_get());
-    case ncra:
-      (void)fprintf(stdout,"%s: HINT Extraction list must contain record variables that are not NC_CHAR or NC_BYTE in order to perform a running average\n",prg_nm_get());
+      (void)fprintf(stdout,"%s: HINT Extraction list must contain at least one derived field\n",prg_nm_get());
+    case ncatted:
+      /* Do nothing */
+      break;
+    case ncbo:
+      (void)fprintf(stdout,"%s: HINT Extraction list must contain a non-coordinate variable that is not NC_CHAR or NC_BYTE in order to perform a binary operation (e.g., subtraction)\n",prg_nm_get());
       break;
     case ncea:
-      (void)fprintf(stdout,"%s: HINT Extraction list must contain non-coordinate variables that are not NC_CHAR or NC_BYTE in order to perform an ensemble average\n",prg_nm_get());
-      break;
-    case ncdiff:
-      (void)fprintf(stdout,"%s: HINT Extraction list must contain non-coordinate variables that are not NC_CHAR or NC_BYTE in order to subtract\n",prg_nm_get());
-      break;
-    case ncflint:
-      (void)fprintf(stdout,"%s: HINT Extraction list must contain variables that are not NC_CHAR or NC_BYTE in order to interpolate\n",prg_nm_get());
-      break;
-    case ncwa:
-      (void)fprintf(stdout,"%s: HINT Extraction list must contain variables that contain an averaging dimension in order to perform an average\n",prg_nm_get());
-      break;
-    case ncrcat:
-      (void)fprintf(stdout,"%s: HINT Extraction list must contain record variables in order to perform a record concatenation\n",prg_nm_get());
+      (void)fprintf(stdout,"%s: HINT Extraction list must contain a non-coordinate variable that is not NC_CHAR or NC_BYTE\n",prg_nm_get());
       break;
     case ncecat:
-      (void)fprintf(stdout,"%s: HINT Extraction list must contain non-coordinate variables in order to perform an ensemble concatenation\n",prg_nm_get());
+      (void)fprintf(stdout,"%s: HINT Extraction list must contain a non-coordinate variable\n",prg_nm_get());
       break;
+    case ncflint:
+      (void)fprintf(stdout,"%s: HINT Extraction list must contain a variable that is not NC_CHAR or NC_BYTE\n",prg_nm_get());
+      break;
+    case ncks:
+      /* Do nothing */
+      break;
+    case ncpdq:
+      (void)fprintf(stdout,"%s: HINT Extraction list must contain a variable that shares at least one dimension with the re-order list\n",prg_nm_get());
+      break;
+    case ncra:
+      (void)fprintf(stdout,"%s: HINT Extraction list must contain a record variable that is not NC_CHAR or NC_BYTE\n",prg_nm_get());
+      break;
+    case ncrcat:
+      (void)fprintf(stdout,"%s: HINT Extraction list must contain a record variable which to concatenate\n",prg_nm_get());
+      break;
+    case ncwa:
+      (void)fprintf(stdout,"%s: HINT Extraction list must contain a variable that contains an averaging dimension\n",prg_nm_get());
+      break;
+    default: nco_dfl_case_prg_id_err(); break;
     } /* end switch */
     nco_exit(EXIT_FAILURE);
   } /* end if */
 
   /* Free unused space and save pointers in output variables */
-  if(*nbr_var_fix > 0) *var_fix_ptr=(var_sct **)nco_realloc(var_fix,*nbr_var_fix*sizeof(var_sct *)); else *var_fix_ptr=NULL;
-  if(*nbr_var_fix > 0) *var_fix_out_ptr=(var_sct **)nco_realloc(var_fix_out,*nbr_var_fix*sizeof(var_sct *)); else *var_fix_out_ptr=NULL;
-  if(*nbr_var_prc > 0) *var_prc_ptr=(var_sct **)nco_realloc(var_prc,*nbr_var_prc*sizeof(var_sct *)); else *var_prc_ptr=NULL;
-  if(*nbr_var_prc > 0) *var_prc_out_ptr=(var_sct **)nco_realloc(var_prc_out,*nbr_var_prc*sizeof(var_sct *)); else *var_prc_out_ptr=NULL;
+  *var_fix_ptr=(var_sct **)nco_realloc(var_fix,*nbr_var_fix*sizeof(var_sct *));
+  *var_fix_out_ptr=(var_sct **)nco_realloc(var_fix_out,*nbr_var_fix*sizeof(var_sct *));
+  *var_prc_ptr=(var_sct **)nco_realloc(var_prc,*nbr_var_prc*sizeof(var_sct *));
+  *var_prc_out_ptr=(var_sct **)nco_realloc(var_prc_out,*nbr_var_prc*sizeof(var_sct *));
 
 } /* end nco_var_lst_dvd */
 
