@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_lst.h,v 1.9 2002/12/30 02:56:15 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_lst.h,v 1.24 2004/09/07 04:31:19 zender Exp $ */
 
 /* Purpose: Variable list utilities */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -21,12 +21,19 @@
 #include <stdlib.h> /* strtod, strtol, malloc, getopt, exit */
 #include <string.h> /* strcmp. . . */
 
+#ifdef HAVE_REGEX_H
+#ifdef MACOSX
+#include <sys/types.h> /* 20040822: Provide off_t required by Mac OS X regex.h */
+#endif /* !MACOSX */
+#include <regex.h> /* POSIX regular expressions library */
+#endif /* HAVE_REGEX_H */
+
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* netCDF3.0 wrapper functions */
+#include <netcdf.h> /* netCDF definitions and C library */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 
 /* Personal headers */
-#include "nco.h" /* NCO definitions */
+#include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_ctl.h" /* Program flow control functions */
 #include "nco_mmr.h" /* Memory management */
 #include "nco_var_utl.h" /* Variable utilities */
@@ -37,12 +44,27 @@ extern "C" {
 
 nm_id_sct * /* O [sct] Variable extraction list */
 nco_var_lst_mk /* [fnc] Create variable extraction list */
+(const int nc_id, /* I [enm] netCDF file ID */
+ const int nbr_var, /* I [nbr] Number of variables in input file */
+ char **var_lst_in, /* I [sng] User-specified list of variable names and re */
+ const bool PROCESS_ALL_COORDINATES, /* I [flg] Process all coordinates */
+ int * const nbr_xtr); /* I/O [nbr] Number of variables in current extraction list */
+
+nm_id_sct * /* O [sct] Variable extraction list */
+nco_var_lst_mk_old /* [fnc] Create variable extraction list */
 (const int nc_id, /* I [enm] netCDF file ID */
  const int nbr_var, /* I [nbr] Number of variables in input file */
  CST_X_PTR_CST_PTR_CST_Y(char,var_lst_in), /* I [sng] User-specified list of variable names */
  const bool PROCESS_ALL_COORDINATES, /* I [flg] Process all coordinates */
  int * const nbr_xtr); /* I/O [nbr] Number of variables in current extraction list */
 
+int /* O [nbr] Number of matches found */
+nco_var_meta_search /* [fnc] Search for pattern matches in var string list */
+(int nbr_var, /* I [nbr] number of vars in srch_sng and size of in_bool */
+ nm_id_sct *in_lst, /* I [sct] List of all variables in input file (with IDs) */
+ char *rexp, /* I [sng] Regular expression pattern */
+ bool *in_bool); /* O [flg] Matched vars holder */
+
 nm_id_sct * /* O [sct] Extraction list */
 nco_var_lst_xcl /* [fnc] Convert exclusion list to extraction list */
 (const int nc_id, /* I netCDF file ID */
@@ -53,7 +75,6 @@ nco_var_lst_xcl /* [fnc] Convert exclusion list to extraction list */
 nm_id_sct * /* O [sct] Extraction list */
 nco_var_lst_add_crd /* [fnc] Add all coordinates to extraction list */
 (const int nc_id, /* I [id] netCDF file ID */
- const int nbr_var, /* I [nbr] Number of variables in input file */
  const int nbr_dim, /* I [nbr] Number of dimensions in input file */
  nm_id_sct *xtr_lst, /* I/O [sct] Current extraction list (destroyed) */
  int * const nbr_xtr); /* I/O [nbr] Number of variables in current extraction list */
@@ -73,9 +94,11 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
 (var_sct * const * const var, /* I [sct] Variable list (input file) */
  var_sct * const * const var_out, /* I [sct] Variable list (output file) */
  const int nbr_var, /* I [nbr] Number of variables */
- const bool NCAR_CSM_FORMAT, /* I [flg] File adheres to NCAR CSM conventions */
+ const bool NCAR_CCSM_FORMAT, /* I [flg] File adheres to NCAR CCSM conventions */
+ const int nco_pck_map, /* I [enm] Packing map */
+ const int nco_pck_plc, /* I [enm] Packing policy */
  CST_X_PTR_CST_PTR_CST_Y(dmn_sct,dmn_xcl), /* I [sct] Dimensions not allowed in fixed variables */
- const int nbr_dmn_xcl, /* I [nbr] Number of excluded dimensions */
+ const int nbr_dmn_xcl, /* I [nbr] Number of altered dimensions */
  var_sct *** const var_fix_ptr, /* O [sct] Fixed-variables (input file) */
  var_sct *** const var_fix_out_ptr, /* O [sct] Fixed-variables (output file) */
  int * const nbr_var_fix, /* O [nbr] Number of fixed variables */
