@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_utl.h,v 1.17 2002/12/30 02:56:15 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_utl.h,v 1.26 2004/09/07 04:31:19 zender Exp $ */
 
 /* Purpose: Variable utilities */
 
-/* Copyright (C) 1995--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 1995--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -23,14 +23,14 @@
 #include <limits.h> /* need LONG_MAX */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* netCDF3.0 wrapper functions */
+#include <netcdf.h> /* netCDF definitions and C library */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 #ifdef _OPENMP
 #include <omp.h> /* OpenMP pragmas */
 #endif /* not _OPENMP */
 
 /* Personal headers */
-#include "nco.h" /* NCO definitions */
+#include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_att_utl.h" /* Attribute utilities */
 #include "nco_bnr.h" /* Binary write utilities */
 #include "nco_ctl.h" /* Program flow control functions */
@@ -41,7 +41,7 @@
 extern "C" {
 #endif /* __cplusplus */
 
-  int  /* O [id] Outputfile variable ID */
+  int /* O [id] Output file variable ID */
   nco_cpy_var_dfn /* [fnc] Copy variable metadata from input to output file */
   (const int in_id, /* I [id] netCDF input file ID */
    const int out_id, /* I [id] netCDF output file ID */
@@ -54,7 +54,7 @@ extern "C" {
    const int out_id, /* I [id] netCDF output file ID */
    const int rec_dmn_id, /* I [id] Input file record dimension ID  */
    const char * const var_nm, /* I [sng] Input variable name */
-   const lmt_all * const lmt_lst, /* I [sct] Hyperslab limits */
+   const lmt_all_sct * const lmt_lst, /* I [sct] Hyperslab limits */
    const int lmt_lst_nbr); /* I [nbr] Number of hyperslab limits */
 
   void
@@ -94,7 +94,9 @@ extern "C" {
    var_sct * const * const var, /* I/O [sct] Variables to be defined in output file */
    const int nbr_var, /* I [nbr] Number of variables to be defined */
    CST_X_PTR_CST_PTR_CST_Y(dmn_sct,dmn_ncl), /* I [sct] Dimensions included in output file */
-   const int nbr_dmn_ncl); /* I [nbr] Number of dimensions in list */
+   const int nbr_dmn_ncl, /* I [nbr] Number of dimensions in list */
+   const int nco_pck_map, /* I [enm] Packing map */
+   const int nco_pck_plc); /* I [enm] Packing policy */
   
   var_sct * /* O [sct] Copy of input variable */
   nco_var_dpl /* [fnc] Duplicate input variable */
