@@ -1,43 +1,33 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncrename.c,v 1.52 2003/01/10 18:05:14 rorik Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncrename.c,v 1.68 2004/08/05 08:18:25 zender Exp $ */
 
 /* ncrename -- netCDF renaming operator */
 
 /* Purpose: Rename dimensions, variables, and attributes of a netCDF file */
 
-/* Copyright (C) 1995--2003 Charlie Zender
+/* Copyright (C) 1995--2004 Charlie Zender
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
-   of the License, or (at your option) any later version.
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
+   The full license text is at http://www.gnu.ai.mit.edu/copyleft/gpl.html 
+   and in the file nco/doc/LICENSE in the NCO source distribution.
    
    As a special exception to the terms of the GPL, you are permitted 
-   to link the NCO source code with the NetCDF and HDF libraries 
-   and distribute the resulting executables under the terms of the GPL, 
-   but in addition obeying the extra stipulations of the netCDF and 
-   HDF library licenses.
+   to link the NCO source code with the DODS, HDF, netCDF, and UDUnits
+   libraries and to distribute the resulting executables under the terms 
+   of the GPL, but in addition obeying the extra stipulations of the 
+   DODS, HDF, netCDF, and UDUnits licenses.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+   See the GNU General Public License for more details.
    
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-
-   The file LICENSE contains the GNU General Public License, version 2
-   It may be viewed interactively by typing, e.g., ncks -L
-
-   The author of this software, Charlie Zender, would like to receive
-   your suggestions, improvements, bug-reports, and patches for NCO.
-   Please contact the project at http://sourceforge.net/projects/nco or by writing
-
+   The original author of this software, Charlie Zender, wants to improve it
+   with the help of your suggestions, improvements, bug-reports, and patches.
+   Please contact the NCO project at http://nco.sf.net or by writing
    Charlie Zender
    Department of Earth System Science
    University of California at Irvine
-   Irvine, CA 92697-3100
- */
+   Irvine, CA 92697-3100 */
 
 /* Usage:
    ncrename -d old_dim1,new_dim1 -v old_var1,new_var1 -v old_var2,new_var2 -a old_att1,new_att1 in.nc foo.nc
@@ -56,27 +46,30 @@
 #include <sys/stat.h> /* stat() */
 #include <time.h> /* machine time */
 #include <unistd.h> /* all sorts of POSIX stuff */
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>  /* getopt_long() */
-#else
+#ifndef HAVE_GETOPT_LONG
 #include "nco_getopt.h"
-#endif /* HAVE_GETOPT_H */
+#else /* !NEED_GETOPT_LONG */ 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif /* !HAVE_GETOPT_H */ 
+#endif /* HAVE_GETOPT_LONG */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
-#include "nco_netcdf.h" /* netCDF 3.0 wrapper functions */
+#include <netcdf.h> /* netCDF definitions and C library */
+#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 
-/* #define MAIN_PROGRAM_FILE MUST precede #include nco.h */
+/* #define MAIN_PROGRAM_FILE MUST precede #include libnco.h */
 #define MAIN_PROGRAM_FILE
-#include "libnco.h" /* netCDF operator library */
+#include "libnco.h" /* netCDF Operator (NCO) library */
 
 int 
 main(int argc,char **argv)
 {
   bool OUTPUT_TO_NEW_NETCDF_FILE=False;
+  bool FILE_RETRIEVED_FROM_REMOTE_LOCATION;
+  bool FL_LST_IN_FROM_STDIN=False; /* [flg] fl_lst_in comes from stdin */
   bool FORCE_APPEND=False; /* Option A */
   bool FORCE_OVERWRITE=False; /* Option O */
-  bool FILE_RETRIEVED_FROM_REMOTE_LOCATION;
   bool HISTORY_APPEND=True; /* Option h */
   bool REMOVE_REMOTE_FILES_AFTER_PROCESSING=True; /* Option R */
   
@@ -86,37 +79,38 @@ main(int argc,char **argv)
   char *dmn_rnm_arg[NC_MAX_DIMS];
   char *var_rnm_arg[NC_MAX_VARS];
   char *att_rnm_arg[NC_MAX_ATTRS];
-  char *opt_sng;
-  char *fl_out;
+  char *fl_out=NULL; /* Option o */
   char *fl_pth_lcl=NULL; /* Option l */
   char *fl_pth=NULL; /* Option p */
   char *time_bfr_srt;
   char *cmd_ln;
-  char CVS_Id[]="$Id: ncrename.c,v 1.52 2003/01/10 18:05:14 rorik Exp $"; 
-  char CVS_Revision[]="$Revision: 1.52 $";
-  
+
+  const char * const CVS_Id="$Id: ncrename.c,v 1.68 2004/08/05 08:18:25 zender Exp $"; 
+  const char * const CVS_Revision="$Revision: 1.68 $";
+  const char * const opt_sng="a:D:d:hl:Oo:p:rv:-:";
+
   extern char *optarg;
-  
   extern int optind;
   
   int idx;
   int nc_id;  
-  int nbr_abb_arg=0;
+  int abb_arg_nbr=0;
   int nbr_var_rnm=0; /* Option v. NB: nbr_var_rnm gets incremented */
   int nbr_att_rnm=0; /* Option a. NB: nbr_att_rnm gets incremented */
   int nbr_dmn_rnm=0; /* Option d. NB: nbr_dmn_rnm gets incremented */
-  int nbr_fl=0;
+  int fl_nbr=0;
   int opt;
   int rcd=NC_NOERR; /* [rcd] Return code */
+  int thr_nbr=0; /* [nbr] Thread number Option t */
   
   rnm_sct *var_rnm_lst=NULL_CEWI;
   rnm_sct *dmn_rnm_lst=NULL_CEWI;
   rnm_sct *att_rnm_lst=NULL_CEWI;
 
-  time_t clock;
+  time_t time_crr_time_t;
 
   static struct option opt_lng[] =
-    {
+    { /* Structure ordered by short option key if possible */
       {"attribute",required_argument,0,'a'},
       {"debug",required_argument,0,'D'},
       {"dbg_lvl",required_argument,0,'D'},
@@ -125,7 +119,11 @@ main(int argc,char **argv)
       {"history",no_argument,0,'h'},
       {"hst",no_argument,0,'h'},
       {"local",required_argument,0,'l'},
+      {"lcl",required_argument,0,'l'},
       {"overwrite",no_argument,0,'O'},
+      {"ovr",no_argument,0,'O'},
+      {"output",required_argument,0,'o'},
+      {"fl_out",required_argument,0,'o'},
       {"path",required_argument,0,'p'},
       {"revision",no_argument,0,'r'},
       {"variable",required_argument,0,'v'},
@@ -138,14 +136,13 @@ main(int argc,char **argv)
 
   /* Start clock and save command line */ 
   cmd_ln=nco_cmd_ln_sng(argc,argv);
-  clock=time((time_t *)NULL);
-  time_bfr_srt=ctime(&clock); time_bfr_srt=time_bfr_srt; /* Avoid compiler warning until variable is used for something */
+  time_crr_time_t=time((time_t *)NULL);
+  time_bfr_srt=ctime(&time_crr_time_t); time_bfr_srt=time_bfr_srt; /* Avoid compiler warning until variable is used for something */
   
   /* Get program name and set program enum (e.g., prg=ncra) */
   prg_nm=prg_prs(argv[0],&prg);
 
   /* Parse command line arguments */
-  opt_sng="a:D:d:hl:Op:rv:-:";
   while((opt = getopt_long(argc,argv,opt_sng,opt_lng,&opt_idx)) != EOF){
     switch(opt){
     case 'A': /* Toggle FORCE_APPEND */
@@ -171,6 +168,9 @@ main(int argc,char **argv)
     case 'O': /* Toggle FORCE_OVERWRITE */
       FORCE_OVERWRITE=!FORCE_OVERWRITE;
       break;
+    case 'o': /* Name of output file */
+      fl_out=(char *)strdup(optarg);
+      break;
     case 'p': /* Common file path */
       fl_pth=optarg;
       break;
@@ -179,7 +179,7 @@ main(int argc,char **argv)
       break;
     case 'r': /* Print CVS program information and copyright notice */
       (void)copyright_prn(CVS_Id,CVS_Revision);
-      (void)nco_lib_vrs_prn();
+      (void)nco_lbr_vrs_prn();
       nco_exit(EXIT_SUCCESS);
       break;
     case 'v': /* Copy argument for later processing */
@@ -188,7 +188,7 @@ main(int argc,char **argv)
       break;
     case '?': /* Print proper usage */
       (void)nco_usg_prn();
-      nco_exit(EXIT_FAILURE);
+      nco_exit(EXIT_SUCCESS);
       break;
     case '-': /* Long options are not allowed */
       (void)fprintf(stderr,"%s: ERROR Long options are not available in this build. Use single letter options instead.\n",prg_nm_get());
@@ -201,7 +201,7 @@ main(int argc,char **argv)
   } /* end while loop */
   
   /* Process positional arguments and fill in filenames */
-  fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&nbr_fl,&fl_out);
+  fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&fl_nbr,&fl_out,&FL_LST_IN_FROM_STDIN);
   if(fl_out != NULL) OUTPUT_TO_NEW_NETCDF_FILE=True; else fl_out=fl_lst_in[0];
 
   if(nbr_var_rnm == 0 && nbr_att_rnm == 0 && nbr_dmn_rnm == 0){
@@ -218,21 +218,21 @@ main(int argc,char **argv)
   /* We have final list of variables, dimensions, and attributes to rename */
   
   /* Parse filename */
-  fl_in=nco_fl_nm_prs(fl_in,0,&nbr_fl,fl_lst_in,nbr_abb_arg,fl_lst_abb,fl_pth);
+  fl_in=nco_fl_nm_prs(fl_in,0,&fl_nbr,fl_lst_in,abb_arg_nbr,fl_lst_abb,fl_pth);
   /* Make sure file is on local system and is readable or die trying */
   fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
 
   if(OUTPUT_TO_NEW_NETCDF_FILE){
 
     if(!FORCE_OVERWRITE){
-      int rcd;
+      int rcd_lcl;
 
       struct stat stat_sct;
       
-      rcd=stat(fl_out,&stat_sct);
+      rcd_lcl=stat(fl_out,&stat_sct);
 
       /* If file already exists, then query user whether to overwrite */
-      if(rcd != -1){
+      if(rcd_lcl != -1){
         char usr_reply='z';
 	short nbr_itr=0;
 
@@ -253,17 +253,17 @@ main(int argc,char **argv)
         if(usr_reply == 'n'){
           nco_exit(EXIT_SUCCESS);
         } /* end if */
-      } /* end if */
-    } /* end if */
+      } /* end if rcd_lcl != -1 */
+    } /* end if FORCE_OVERWRITE */
     
     /* Copy input file to output file and then search through output, 
        changing names on the fly. This avoids possible XDR translation
        performance penalty of copying each variable with netCDF. */
     (void)nco_fl_cp(fl_in,fl_out);
 
-  } /* end if */
+  } /* end if OUTPUT_TO_NEW_NETCDF_FILE */
   
-  /* Open file. Writing must be enabled and file should be in define mode for renaming */
+  /* Open file enabled for writing. Place file in define mode for renaming. */
   rcd=nco_open(fl_out,NC_WRITE,&nc_id);
   (void)nco_redef(nc_id);
 
@@ -407,6 +407,15 @@ main(int argc,char **argv)
   /* Catenate timestamped command line to "history" global attribute */
   if(HISTORY_APPEND) (void)nco_hst_att_cat(nc_id,cmd_ln);
   
+  /* Initialize thread information */
+  thr_nbr=nco_openmp_ini(thr_nbr);
+  if(thr_nbr > 0 && HISTORY_APPEND) (void)nco_thr_att_cat(nc_id,thr_nbr);
+  
+#ifdef _OPENMP
+  /* fxm: hack to get libxlsmp library linked in */
+  (void)omp_in_parallel();
+#endif /* not _OPENMP */
+
   /* Take file out of define mode */
   (void)nco_enddef(nc_id);
     
