@@ -1,52 +1,43 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncwa.c,v 1.99 2003/01/09 00:27:37 rorik Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncwa.c,v 1.148 2004/09/07 04:31:19 zender Exp $ */
 
 /* ncwa -- netCDF weighted averager */
 
 /* Purpose: Compute averages of specified hyperslabs of specfied variables
    in a single input netCDF file and output them to a single file. */
 
-/* Copyright (C) 1995--2003 Charlie Zender
+/* Copyright (C) 1995--2004 Charlie Zender
+
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
+   The full license text is at http://www.gnu.ai.mit.edu/copyleft/gpl.html 
+   and in the file nco/doc/LICENSE in the NCO source distribution.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
-   of the License, or (at your option) any later version.
-   
    As a special exception to the terms of the GPL, you are permitted 
-   to link the NCO source code with the NetCDF and HDF libraries 
-   and distribute the resulting executables under the terms of the GPL, 
-   but in addition obeying the extra stipulations of the netCDF and 
-   HDF library licenses.
+   to link the NCO source code with the DODS, HDF, netCDF, and UDUnits
+   libraries and to distribute the resulting executables under the terms 
+   of the GPL, but in addition obeying the extra stipulations of the 
+   DODS, HDF, netCDF, and UDUnits licenses.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+   See the GNU General Public License for more details.
    
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-
-   The file LICENSE contains the GNU General Public License, version 2
-   It may be viewed interactively by typing, e.g., ncks -L
-
-   The author of this software, Charlie Zender, would like to receive
-   your suggestions, improvements, bug-reports, and patches for NCO.
-   Please contact the project at http://sourceforge.net/projects/nco or by writing
-
+   The original author of this software, Charlie Zender, wants to improve it
+   with the help of your suggestions, improvements, bug-reports, and patches.
+   Please contact the NCO project at http://nco.sf.net or by writing
    Charlie Zender
    Department of Earth System Science
    University of California at Irvine
-   Irvine, CA 92697-3100
- */
+   Irvine, CA 92697-3100 */
 
 /* fxm: 19981202 -n and -W switches were deactivated but code left in place
    while I rethink the normalization switches */
 
 /* Usage:
-   ncwa -O -a lon /home/zender/nco/data/in.nc foo.nc
-   ncwa -O -R -p /ZENDER/tmp -l /home/zender/nco/data in.nc foo.nc
+   ncwa -O -a lon ~/nco/data/in.nc foo.nc
+   ncwa -O -R -p /ZENDER/tmp -l ~/nco/data in.nc foo.nc
    ncwa -O -a lat -w gw -d lev,17 -v T -p /fs/cgd/csm/input/atm SEP1.T42.0596.nc foo.nc
+
    ncwa -O -C -a lat,lon,time -w gw -v PS -p /fs/cgd/csm/input/atm SEP1.T42.0596.nc foo.nc;ncks -H foo.nc
  */
 
@@ -62,35 +53,41 @@
 #include <sys/stat.h> /* stat() */
 #include <time.h> /* machine time */
 #include <unistd.h> /* all sorts of POSIX stuff */
-/* #include <assert.h> */ /* assert() debugging macro */
-/* #include <errno.h> */ /* errno */
-/* #include <malloc.h> */ /* malloc() stuff */
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>  /* getopt_long() */
-#else
+#ifndef HAVE_GETOPT_LONG
 #include "nco_getopt.h"
-#endif /* !HAVE_GETOPT_H */
+#else /* !NEED_GETOPT_LONG */ 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif /* !HAVE_GETOPT_H */ 
+#endif /* HAVE_GETOPT_LONG */
 
 /* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions */
+#include <netcdf.h> /* netCDF definitions and C library */
 
-/* #define MAIN_PROGRAM_FILE MUST precede #include nco.h */
+/* #define MAIN_PROGRAM_FILE MUST precede #include libnco.h */
 #define MAIN_PROGRAM_FILE
-#include "libnco.h" /* netCDF operator library */
+#include "ncap.h" /* netCDF arithmetic processor-specific definitions (symbol table, ...) */
+#include "libnco.h" /* netCDF Operator (NCO) library */
+
+/* Global variables (keep consistent with global variables declared in ncap.c) */ 
+size_t ncap_ncl_dpt_crr=0UL; /* [nbr] Depth of current #include file (incremented in ncap.l) */
+size_t *ncap_ln_nbr_crr; /* [cnt] Line number (incremented in ncap.l) */
+char **ncap_fl_spt_glb; /* [fl] Script file */
 
 int 
 main(int argc,char **argv)
 {
   bool EXCLUDE_INPUT_LIST=False; /* Option c */
   bool FILE_RETRIEVED_FROM_REMOTE_LOCATION;
+  bool FL_LST_IN_FROM_STDIN=False; /* [flg] fl_lst_in comes from stdin */
   bool FORCE_APPEND=False; /* Option A */
   bool FORCE_OVERWRITE=False; /* Option O */
-  bool FORTRAN_STYLE=False; /* Option F */
+  bool FORTRAN_IDX_CNV=False; /* Option F */
   bool HISTORY_APPEND=True; /* Option h */
   bool MUST_CONFORM=False; /* [flg] Must nco_var_cnf_dmn() find truly conforming variables? */
   bool DO_CONFORM_MSK; /* Did nco_var_cnf_dmn() find truly conforming mask? */
   bool DO_CONFORM_WGT=False; /* Did nco_var_cnf_dmn() find truly conforming weight? */
-  bool NCAR_CSM_FORMAT;
+  bool NCAR_CCSM_FORMAT;
   bool PROCESS_ALL_COORDINATES=False; /* Option c */
   bool PROCESS_ASSOCIATED_COORDINATES=True; /* Option C */
   bool REMOVE_REMOTE_FILES_AFTER_PROCESSING=True; /* Option R */
@@ -102,26 +99,28 @@ main(int argc,char **argv)
   bool opt_a_flg=False; /* Option a */
 
   char **dmn_avg_lst_in=NULL_CEWI; /* Option a */
-  char **var_lst_in=NULL_CEWI;
   char **fl_lst_abb=NULL; /* Option n */
   char **fl_lst_in=NULL_CEWI;
+  char **var_lst_in=NULL_CEWI;
+  char *cmd_ln;
   char *fl_in=NULL;
-  char *fl_pth_lcl=NULL; /* Option l */
-  char *lmt_arg[NC_MAX_DIMS];
-  char *opt_sng;
-  char *fl_out;
+  char *fl_out=NULL; /* Option o */
   char *fl_out_tmp=NULL_CEWI;
   char *fl_pth=NULL; /* Option p */
-  char *time_bfr_srt;
+  char *fl_pth_lcl=NULL; /* Option l */
+  char *lmt_arg[NC_MAX_DIMS];
   char *msk_nm=NULL;
   char *nco_op_typ_sng; /* Operation type */
+  char *time_bfr_srt;
   char *wgt_nm=NULL;
-  char *cmd_ln;
-  char CVS_Id[]="$Id: ncwa.c,v 1.99 2003/01/09 00:27:37 rorik Exp $"; 
-  char CVS_Revision[]="$Revision: 1.99 $";
+  char *msk_sng=NULL; /* Mask string to be "parsed" and values given to msk_nm, msk_val, op_typ_rlt */
+  
+  const char * const CVS_Id="$Id: ncwa.c,v 1.148 2004/09/07 04:31:19 zender Exp $"; 
+  const char * const CVS_Revision="$Revision: 1.148 $";
+  const char * const opt_sng="Aa:CcD:d:FhIl:M:m:nNOo:p:rRT:t:v:Ww:xy:z:-:";
   
   dmn_sct **dim=NULL_CEWI;
-  dmn_sct **dmn_out;
+  dmn_sct **dmn_out=NULL_CEWI;
   dmn_sct **dmn_avg=NULL_CEWI;
   
   double msk_val=1.0; /* Option M */
@@ -129,36 +128,42 @@ main(int argc,char **argv)
   extern char *optarg;
   extern int optind;
   
+  /* Using naked stdin/stdout/stderr in parallel region generates warning
+     Copy appropriate filehandle to variable scoped shared in parallel clause */
+  FILE * const fp_stderr=stderr; /* [fl] stderr filehandle CEWI */
+  FILE * const fp_stdout=stdout; /* [fl] stdout filehandle CEWI */
+
+  int abb_arg_nbr=0;
+  int fl_idx=int_CEWI;
+  int fl_nbr=0;
   int fll_md_old; /* [enm] Old fill mode */
   int idx=int_CEWI;
   int idx_avg;
-  int idx_fl=int_CEWI;
   int in_id=int_CEWI;  
-  int out_id;  
-  int nbr_abb_arg=0;
-  int nbr_dmn_fl;
-  int nbr_dmn_avg=0;
   int lmt_nbr=0; /* Option d. NB: lmt_nbr gets incremented */
-  int nbr_var_fl;
+  int nbr_dmn_avg=0;
+  int nbr_dmn_fl;
+  int nbr_dmn_out=0;
+  int nbr_dmn_xtr;
   int nbr_var_fix; /* nbr_var_fix gets incremented */
+  int nbr_var_fl;
   int nbr_var_prc; /* nbr_var_prc gets incremented */
   int nbr_xtr=0; /* nbr_xtr won't otherwise be set for -c with no -v */
-  int nbr_dmn_out;
-  int nbr_dmn_xtr;
-  int nbr_fl=0;
   int nco_op_typ=nco_op_avg; /* Operation type */
-  int opt;
   int op_typ_rlt=0; /* Option o */
+  int opt;
+  int out_id;  
   int rcd=NC_NOERR; /* [rcd] Return code */
-  int rec_dmn_id=-1;
+  int rec_dmn_id=NCO_REC_DMN_UNDEFINED; /* [id] Record dimension ID in input file */
+  int thr_nbr=0; /* [nbr] Thread number Option t */
   
   lmt_sct *lmt;
   
   nm_id_sct *dmn_lst;
-  nm_id_sct *xtr_lst=NULL; /* xtr_lst can get realloc()'d from NULL with -c option */
+  nm_id_sct *xtr_lst=NULL; /* xtr_lst may bealloc()'d from NULL with -c option */
   nm_id_sct *dmn_avg_lst;
   
-  time_t clock;
+  time_t time_crr_time_t;
   
   var_sct **var;
   var_sct **var_fix;
@@ -172,13 +177,17 @@ main(int argc,char **argv)
   var_sct *wgt_avg=NULL;
   var_sct *wgt_out=NULL;
   
+  prs_sct prs_arg;  /* I/O [sct] Global information required in ncwa parser */
+  
   static struct option opt_lng[]=
-    {
+    { /* Structure ordered by short option key if possible */
       {"average",required_argument,0,'a'},
       {"avg",required_argument,0,'a'},
       {"append",no_argument,0,'A'},
       {"coords",no_argument,0,'c'},
-      {"nocoords",no_argument,0,'C'},
+      {"crd",no_argument,0,'c'},
+      {"no-coords",no_argument,0,'C'},
+      {"no-crd",no_argument,0,'C'},
       {"debug",required_argument,0,'D'},
       {"dbg_lvl",required_argument,0,'D'},
       {"dimension",required_argument,0,'d'},
@@ -189,6 +198,7 @@ main(int argc,char **argv)
       {"hst",no_argument,0,'h'},
       {"wgt_msk_crd_var",no_argument,0,'I'},
       {"local",required_argument,0,'l'},
+      {"lcl",required_argument,0,'l'},
       {"mask-variable",required_argument,0,'m'},
       {"mask_variable",required_argument,0,'m'},
       {"mask",required_argument,0,'m'},
@@ -198,11 +208,22 @@ main(int argc,char **argv)
       {"mask_value",required_argument,0,'M'},
       {"msk_val",required_argument,0,'M'},
       {"nintap",required_argument,0,'n'},
+      {"nmr",no_argument,0,'N'},
       {"numerator",no_argument,0,'N'},
       {"overwrite",no_argument,0,'O'},
+      {"ovr",no_argument,0,'O'},
+      {"output",required_argument,0,'o'},
+      {"fl_out",required_argument,0,'o'},
       {"path",required_argument,0,'p'},
       {"retain",no_argument,0,'R'},
+      {"rtn",no_argument,0,'R'},
       {"revision",no_argument,0,'r'},
+      {"truth_condition",required_argument,0,'T'},
+      {"msk_cmp_typ",required_argument,0,'T'},
+      {"op_rlt",required_argument,0,'T'},
+      {"thr_nbr",required_argument,0,'t'},
+      {"threads",required_argument,0,'t'},
+      {"omp_num_threads",required_argument,0,'t'},
       {"variable",required_argument,0,'v'},
       {"version",no_argument,0,'r'},
       {"vrs",no_argument,0,'r'},
@@ -214,6 +235,8 @@ main(int argc,char **argv)
       {"wgt_var",no_argument,0,'w'},
       {"operation",required_argument,0,'y'},
       {"op_typ",required_argument,0,'y'},
+      {"mask_string",required_argument,0,'z'},
+      {"msk_sng",required_argument,0,'z'},
       {"help",no_argument,0,'?'},
       {0,0,0,0}
     }; /* end opt_lng */
@@ -221,8 +244,8 @@ main(int argc,char **argv)
 
   /* Start clock and save command line */ 
   cmd_ln=nco_cmd_ln_sng(argc,argv);
-  clock=time((time_t *)NULL);
-  time_bfr_srt=ctime(&clock); time_bfr_srt=time_bfr_srt; /* Avoid compiler warning until variable is used for something */
+  time_crr_time_t=time((time_t *)NULL);
+  time_bfr_srt=ctime(&time_crr_time_t); time_bfr_srt=time_bfr_srt; /* Avoid compiler warning until variable is used for something */
 
   time_bfr_srt=time_bfr_srt; /* CEWI: Avert compiler warning that variable is set but never used */
   NORMALIZE_BY_TALLY=NORMALIZE_BY_TALLY; /* CEWI: Avert compiler warning that variable is set but never used */
@@ -232,7 +255,6 @@ main(int argc,char **argv)
   prg_nm=prg_prs(argv[0],&prg);
 
   /* Parse command line arguments */
-  opt_sng="Aa:CcD:d:FhIl:M:m:nNo:Op:rRv:xWw:y:-:";
   while((opt = getopt_long(argc,argv,opt_sng,opt_lng,&opt_idx)) != EOF){
     switch(opt){
     case 'A': /* Toggle FORCE_APPEND */
@@ -240,8 +262,8 @@ main(int argc,char **argv)
       break;
     case 'a': /* Dimensions over which to average hyperslab */
       if(opt_a_flg){
-	(void)fprintf(stdout,"%s: ERROR Option -a appears more than once\n",prg_nm);
-	(void)fprintf(stdout,"%s: HINT Use -a dim1,dim2,... not -a dim1 -a dim2 ...\n",prg_nm);
+	(void)fprintf(fp_stdout,"%s: ERROR Option -a appears more than once\n",prg_nm);
+	(void)fprintf(fp_stdout,"%s: HINT Use -a dim1,dim2,... not -a dim1 -a dim2 ...\n",prg_nm);
 	(void)nco_usg_prn();
 	nco_exit(EXIT_FAILURE);
       } /* endif */
@@ -262,7 +284,7 @@ main(int argc,char **argv)
       lmt_nbr++;
       break;
     case 'F': /* Toggle index convention. Default is 0-based arrays (C-style). */
-      FORTRAN_STYLE=!FORTRAN_STYLE;
+      FORTRAN_IDX_CNV=!FORTRAN_IDX_CNV;
       break;
     case 'h': /* Toggle appending to history global attribute */
       HISTORY_APPEND=!HISTORY_APPEND;
@@ -273,7 +295,7 @@ main(int argc,char **argv)
     case 'l': /* Local path prefix for files retrieved from remote file system */
       fl_pth_lcl=optarg;
       break;
-    case 'm': /* Name of variable to use as mask in reducing.  Default is none */
+    case 'm': /* Name of variable to use as mask in reducing. Default is none */
       msk_nm=optarg;
       break;
     case 'M': /* Good data defined by relation to mask value. Default is 1. */
@@ -286,14 +308,14 @@ main(int argc,char **argv)
       break;
     case 'n':
       NORMALIZE_BY_WEIGHT=False;
-      (void)fprintf(stdout,"%s: ERROR This option has been disabled while I rethink its implementation\n",prg_nm);
+      (void)fprintf(fp_stdout,"%s: ERROR This option has been disabled while I rethink its implementation\n",prg_nm);
       nco_exit(EXIT_FAILURE);
       break;
     case 'O': /* Toggle FORCE_OVERWRITE */
       FORCE_OVERWRITE=!FORCE_OVERWRITE;
       break;
-    case 'o': /* Relational operator type.  Default is 0, eq, equality */
-      op_typ_rlt=nco_op_prs_rlt(optarg);
+    case 'o': /* Name of output file */
+      fl_out=(char *)strdup(optarg);
       break;
     case 'p': /* Common file path */
       fl_pth=optarg;
@@ -303,15 +325,23 @@ main(int argc,char **argv)
       break;
     case 'r': /* Print CVS program information and copyright notice */
       (void)copyright_prn(CVS_Id,CVS_Revision);
-      (void)nco_lib_vrs_prn();
+      (void)nco_lbr_vrs_prn();
        nco_exit(EXIT_SUCCESS);
       break;
+    case 'T': /* Relational operator type. Default is 0, eq, equality */
+      op_typ_rlt=nco_op_prs_rlt(optarg);
+      break;
+    case 't': /* Thread number */
+      thr_nbr=(int)strtol(optarg,(char **)NULL,10);
+      break;
     case 'v': /* Variables to extract/exclude */
+      /* Replace commas with hashes when within braces (convert back later) */
+      (void)nco_lst_comma2hash(optarg);
       var_lst_in=lst_prs(optarg,",",&nbr_xtr);
       break;
     case 'W':
       NORMALIZE_BY_TALLY=False;
-      (void)fprintf(stdout,"%s: ERROR This option has been disabled while I rethink its implementation\n",prg_nm);
+      (void)fprintf(fp_stdout,"%s: ERROR This option has been disabled while I rethink its implementation\n",prg_nm);
       nco_exit(EXIT_FAILURE);
       break;
     case 'w': /* Variable to use as weight in reducing.  Default is none */
@@ -324,9 +354,12 @@ main(int argc,char **argv)
       nco_op_typ_sng=(char *)strdup(optarg);
       nco_op_typ=nco_op_typ_get(nco_op_typ_sng);
       break;
+    case 'z': /* Mask string to be parsed */
+      msk_sng=optarg;
+      break;
     case '?': /* Print proper usage */
       (void)nco_usg_prn();
-      nco_exit(EXIT_FAILURE);
+      nco_exit(EXIT_SUCCESS);
       break;
     case '-': /* Long options are not allowed */
       (void)fprintf(stderr,"%s: ERROR Long options are not available in this build. Use single letter options instead.\n",prg_nm_get());
@@ -339,17 +372,43 @@ main(int argc,char **argv)
     } /* end switch */
   } /* end while loop */
 
+  /* Parse mask string */
+  if(msk_sng){
+    int zero=0;
+    /* Set arguments for scan */
+    prs_arg.fl_in=NULL; /* [sng] Input data file */
+    prs_arg.in_id=0; /* [id] Input data file ID */
+    prs_arg.fl_out=NULL; /* [sng] Output data file */
+    prs_arg.out_id=0; /* [id] Output data file ID */
+    prs_arg.att_lst=NULL; /* [sct] Attributes in script */
+    prs_arg.nbr_att=&zero; /* [nbr] Number of attributes in script */
+    prs_arg.dmn_in=NULL; /* [dmn_in] List of all dimensions in input */
+    prs_arg.nbr_dmn_in=0; /* [nbr] Number of  dimensions in input */
+    prs_arg.dmn_out=NULL; /* pointer to list of dims in output */
+    prs_arg.nbr_dmn_out=&zero; /* number of dims in above list */
+    prs_arg.sym_tbl=NULL; /* [fnc] Symbol table for functions */
+    prs_arg.sym_tbl_nbr=0; /* [nbr] Number of functions in table */
+    prs_arg.ntl_scn=False; /* [flg] Initial scan of script */
+    prs_arg.var_LHS=NULL; /* [var] LHS cast variable */
+    prs_arg.nco_op_typ=nco_op_nil; /* [enm] Operation type */
+  
+    /* Initialize line counter */
+    ncap_ln_nbr_crr=(size_t *)nco_realloc(ncap_ln_nbr_crr,ncap_ncl_dpt_crr+1UL); 
+    ncap_ln_nbr_crr[ncap_ncl_dpt_crr]=1UL; /* [cnt] Line number incremented in ncap.l */
+    if(ncap_ncwa_scn(&prs_arg,msk_sng,&msk_nm,&msk_val,&op_typ_rlt) == 0) nco_exit(EXIT_FAILURE); 
+  } /* endif msk_sng */
+
   /* Ensure we do not attempt to normalize by non-existent weight */
   if(wgt_nm == NULL) NORMALIZE_BY_WEIGHT=False;
 
   /* Process positional arguments and fill in filenames */
-  fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&nbr_fl,&fl_out);
+  fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&fl_nbr,&fl_out,&FL_LST_IN_FROM_STDIN);
 
   /* Make uniform list of user-specified dimension limits */
   lmt=nco_lmt_prs(lmt_nbr,lmt_arg);
     
   /* Parse filename */
-  fl_in=nco_fl_nm_prs(fl_in,0,&nbr_fl,fl_lst_in,nbr_abb_arg,fl_lst_abb,fl_pth);
+  fl_in=nco_fl_nm_prs(fl_in,0,&fl_nbr,fl_lst_in,abb_arg_nbr,fl_lst_abb,fl_pth);
   /* Make sure file is on local system and is readable or die trying */
   fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
   /* Open file for reading */
@@ -358,26 +417,23 @@ main(int argc,char **argv)
   /* Get number of variables, dimensions, and record dimension ID of input file */
   (void)nco_inq(in_id,&nbr_dmn_fl,&nbr_var_fl,(int *)NULL,&rec_dmn_id);
   
-  /* Form initial extraction list from user input */
+  /* Form initial extraction list which may include extended regular expressions */
   xtr_lst=nco_var_lst_mk(in_id,nbr_var_fl,var_lst_in,PROCESS_ALL_COORDINATES,&nbr_xtr);
 
   /* Change included variables to excluded variables */
   if(EXCLUDE_INPUT_LIST) xtr_lst=nco_var_lst_xcl(in_id,nbr_var_fl,xtr_lst,&nbr_xtr);
 
   /* Add all coordinate variables to extraction list */
-  if(PROCESS_ALL_COORDINATES) xtr_lst=nco_var_lst_add_crd(in_id,nbr_var_fl,nbr_dmn_fl,xtr_lst,&nbr_xtr);
+  if(PROCESS_ALL_COORDINATES) xtr_lst=nco_var_lst_add_crd(in_id,nbr_dmn_fl,xtr_lst,&nbr_xtr);
 
   /* Make sure coordinates associated extracted variables are also on extraction list */
   if(PROCESS_ASSOCIATED_COORDINATES) xtr_lst=nco_var_lst_ass_crd_add(in_id,xtr_lst,&nbr_xtr);
 
-  /* Remove record coordinate, if any, from extraction list */
-  if(False) xtr_lst=nco_var_lst_crd_xcl(in_id,rec_dmn_id,xtr_lst,&nbr_xtr);
-
   /* Sort extraction list by variable ID for fastest I/O */
-  if(nbr_xtr > 1) xtr_lst=nco_lst_srt(xtr_lst,nbr_xtr,False);
+  if(nbr_xtr > 1) xtr_lst=nco_lst_srt_nm_id(xtr_lst,nbr_xtr,False);
     
   /* Find coordinate/dimension values associated with user-specified limits */
-  for(idx=0;idx<lmt_nbr;idx++) (void)nco_lmt_evl(in_id,lmt+idx,0L,FORTRAN_STYLE);
+  for(idx=0;idx<lmt_nbr;idx++) (void)nco_lmt_evl(in_id,lmt+idx,0L,FORTRAN_IDX_CNV);
   
   /* Find dimensions associated with variables to be extracted */
   dmn_lst=nco_dmn_lst_ass_var(in_id,xtr_lst,nbr_xtr,&nbr_dmn_xtr);
@@ -392,7 +448,7 @@ main(int argc,char **argv)
   if(lmt_nbr > 0) (void)nco_dmn_lmt_mrg(dim,nbr_dmn_xtr,lmt,lmt_nbr);
 
   /* Not specifying any dimensions is interpreted as specifying all dimensions */
-  if (nbr_dmn_avg == 0){
+  if(nbr_dmn_avg == 0){
     nbr_dmn_avg=nbr_dmn_xtr;
     dmn_avg_lst_in=(char **)nco_malloc(nbr_dmn_avg*sizeof(char *));
     for(idx=0;idx<nbr_dmn_avg;idx++){
@@ -401,15 +457,15 @@ main(int argc,char **argv)
     (void)fprintf(stderr,"%s: INFO No dimensions specified with -a, therefore reducing (averaging, taking minimum, etc.) over all dimensions\n",prg_nm);
   } /* end if nbr_dmn_avg == 0 */
 
-  if (nbr_dmn_avg > 0){
-    /* Form list of reducing dimensions */
-    dmn_avg_lst=nco_dmn_lst_mk(in_id,dmn_avg_lst_in,nbr_dmn_avg);
-
+  if(nbr_dmn_avg > 0){
     if(nbr_dmn_avg > nbr_dmn_xtr){
-      (void)fprintf(stdout,"%s: ERROR More reducing dimensions than extracted dimensions\n",prg_nm);
+      (void)fprintf(fp_stdout,"%s: ERROR More reducing dimensions than extracted dimensions\n",prg_nm);
       nco_exit(EXIT_FAILURE);
     } /* end if */
 
+    /* Create structured list of reducing dimension names and IDs */
+    dmn_avg_lst=nco_dmn_lst_mk(in_id,dmn_avg_lst_in,nbr_dmn_avg);
+
     /* Form list of reducing dimensions from extracted input dimensions */
     dmn_avg=(dmn_sct **)nco_malloc(nbr_dmn_avg*sizeof(dmn_sct *));
     for(idx_avg=0;idx_avg<nbr_dmn_avg;idx_avg++){
@@ -421,10 +477,12 @@ main(int argc,char **argv)
       }else{
 	(void)fprintf(stderr,"%s: WARNING reducing dimension \"%s\" is not contained in any variable in extraction list\n",prg_nm,dmn_avg_lst[idx_avg].nm);
 	/* Collapse dimension average list by omitting irrelevent dimension */
-	(void)memmove(dmn_avg_lst,dmn_avg_lst,idx_avg*sizeof(nm_id_sct));
 	(void)memmove(dmn_avg_lst+idx_avg*sizeof(nm_id_sct),dmn_avg_lst+(idx_avg+1)*sizeof(nm_id_sct),(nbr_dmn_avg-idx_avg+1)*sizeof(nm_id_sct));
+	/* fxm: Possible bug since nbr_dmn_avg decremented before dmn_avg filled TODO nco367 */
 	--nbr_dmn_avg;
 	dmn_avg_lst=(nm_id_sct *)nco_realloc(dmn_avg_lst,nbr_dmn_avg*sizeof(nm_id_sct));
+	/* fxm: This seem reasonable */
+	/* dmn_avg=(dmn_sct **)nco_realloc(dmn_avg,nbr_dmn_avg*sizeof(dmn_sct *)); */
       } /* end else */
     } /* end loop over idx_avg */
 
@@ -433,7 +491,7 @@ main(int argc,char **argv)
       for(idx_avg=0;idx_avg<nbr_dmn_avg;idx_avg++){
 	if(idx_avg != idx){
 	  if(dmn_avg[idx]->id == dmn_avg[idx_avg]->id){
-	    (void)fprintf(stdout,"%s: ERROR %s specified more than once in reducing list\n",prg_nm,dmn_avg[idx]->nm);
+	    (void)fprintf(fp_stdout,"%s: ERROR %s specified more than once in reducing list\n",prg_nm,dmn_avg[idx]->nm);
 	    nco_exit(EXIT_FAILURE);
 	  } /* end if */
 	} /* end if */
@@ -455,24 +513,14 @@ main(int argc,char **argv)
     } /* end loop over idx_avg */
 
     if(nbr_dmn_out != nbr_dmn_xtr-nbr_dmn_avg){
-      (void)fprintf(stdout,"%s: ERROR nbr_dmn_out != nbr_dmn_xtr-nbr_dmn_avg\n",prg_nm);
+      (void)fprintf(fp_stdout,"%s: ERROR nbr_dmn_out != nbr_dmn_xtr-nbr_dmn_avg\n",prg_nm);
       nco_exit(EXIT_FAILURE);
     } /* end if */
     
-  }else{
+  } /* nbr_dmn_avg <= 0 */
 
-    /* Duplicate input dimension structures for output dimension structures */
-    nbr_dmn_out=nbr_dmn_xtr;
-    dmn_out=(dmn_sct **)nco_malloc(nbr_dmn_out*sizeof(dmn_sct *));
-    for(idx=0;idx<nbr_dmn_out;idx++){
-      dmn_out[idx]=nco_dmn_dpl(dim[idx]);
-      (void)nco_dmn_xrf(dim[idx],dmn_out[idx]);
-    } /* end loop over idx */
-
-  } /* end if */
-
-  /* Is this an NCAR CSM-format history tape? */
-  NCAR_CSM_FORMAT=nco_ncar_csm_inq(in_id);
+  /* Is this an NCAR CCSM-format history tape? */
+  NCAR_CCSM_FORMAT=nco_ncar_csm_inq(in_id);
 
   /* Fill in variable structure list for all extracted variables */
   var=(var_sct **)nco_malloc(nbr_xtr*sizeof(var_sct *));
@@ -485,7 +533,7 @@ main(int argc,char **argv)
   } /* end loop over idx */
 
   /* Divide variable lists into lists of fixed variables and variables to be processed */
-  (void)nco_var_lst_dvd(var,var_out,nbr_xtr,NCAR_CSM_FORMAT,dmn_avg,nbr_dmn_avg,&var_fix,&var_fix_out,&nbr_var_fix,&var_prc,&var_prc_out,&nbr_var_prc);
+  (void)nco_var_lst_dvd(var,var_out,nbr_xtr,NCAR_CCSM_FORMAT,nco_pck_plc_nil,nco_pck_map_nil,dmn_avg,nbr_dmn_avg,&var_fix,&var_fix_out,&nbr_var_fix,&var_prc,&var_prc_out,&nbr_var_prc);
 
   /* We now have final list of variables to extract. Phew. */
   if(dbg_lvl > 0){
@@ -504,13 +552,17 @@ main(int argc,char **argv)
   /* Catenate time-stamped command line to "history" global attribute */
   if(HISTORY_APPEND) (void)nco_hst_att_cat(out_id,cmd_ln);
 
+  /* Initialize thread information */
+  thr_nbr=nco_openmp_ini(thr_nbr);
+  if(thr_nbr > 0 && HISTORY_APPEND) (void)nco_thr_att_cat(out_id,thr_nbr);
+  
   /* Define dimensions in output file */
   (void)nco_dmn_dfn(fl_out,out_id,dmn_out,nbr_dmn_out);
 
   /* Define variables in output file, copy their attributes */
-  (void)nco_var_dfn(in_id,fl_out,out_id,var_out,nbr_xtr,dmn_out,nbr_dmn_out);
+  (void)nco_var_dfn(in_id,fl_out,out_id,var_out,nbr_xtr,dmn_out,nbr_dmn_out,nco_pck_plc_nil,nco_pck_map_nil);
 
-  /* New missing values must be added to output file while in define mode */
+  /* Add new missing values to output file while in define mode */
   if(msk_nm != NULL){
     for(idx=0;idx<nbr_var_prc;idx++){
       /* Define for var_prc_out because mss_val for var_prc will be overwritten in nco_var_refresh */
@@ -537,13 +589,13 @@ main(int argc,char **argv)
   /* Close first input netCDF file */
   nco_close(in_id);
   
-  /* Loop over input files (not currently used, nbr_fl == 1) */
-  for(idx_fl=0;idx_fl<nbr_fl;idx_fl++){
+  /* Loop over input files (not currently used, fl_nbr == 1) */
+  for(fl_idx=0;fl_idx<fl_nbr;fl_idx++){
     /* Parse filename */
-    if(idx_fl != 0) fl_in=nco_fl_nm_prs(fl_in,idx_fl,&nbr_fl,fl_lst_in,nbr_abb_arg,fl_lst_abb,fl_pth);
-    if(dbg_lvl > 0) (void)fprintf(stderr,"\nInput file %d is %s; ",idx_fl,fl_in);
+    if(fl_idx != 0) fl_in=nco_fl_nm_prs(fl_in,fl_idx,&fl_nbr,fl_lst_in,abb_arg_nbr,fl_lst_abb,fl_pth);
+    if(dbg_lvl > 0) (void)fprintf(stderr,"\nInput file %d is %s; ",fl_idx,fl_in);
     /* Make sure file is on local system and is readable or die trying */
-    if(idx_fl != 0) fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
+    if(fl_idx != 0) fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
     if(dbg_lvl > 0) (void)fprintf(stderr,"local file %s:\n",fl_in);
     rcd=nco_open(fl_in,NC_NOWRITE,&in_id);
     
@@ -579,10 +631,7 @@ main(int argc,char **argv)
       (void)nco_var_get(in_id,msk); /* Routine contains OpenMP critical regions */
     } /* end if */
 
-    /* Set up OpenMP multi-threading environment */
-    rcd+=nco_openmp_ini();
-
-  /* 
+  /* Test OpenMP parallelization:
      cd ~/nco/bld;make OPTS=D;cd -
      cd ~/nco/data;ncwa -D 1 -O in.nc foo.nc 2>&1 | m;cd -
 
@@ -610,33 +659,34 @@ main(int argc,char **argv)
      ncks -H -C -v one,two,three,four foo.nc | m
   */
 #ifdef _OPENMP
-/* Adding a default(none) clause causes a weird error: "Error: Variable __iob used without scope declaration in a parallel region with DEFAULT(NONE) scope". This appears to be a compiler bug. */
   /* OpenMP notes:
      firstprivate(): msk_out and wgt_out must be NULL on first call to nco_var_cnf_dmn()
      shared(): msk and wgt are not altered within loop
      private(): wgt_avg does not need initialization */
-#pragma omp parallel for firstprivate(msk_out,wgt_out) private(idx,DO_CONFORM_MSK,DO_CONFORM_WGT,wgt_avg) shared(nbr_var_prc,dbg_lvl,var_prc,var_prc_out,in_id,nco_op_typ,msk_nm,WGT_MSK_CRD_VAR,MUST_CONFORM,msk_val,op_typ_rlt,wgt_nm,dmn_avg,nbr_dmn_avg,NRM_BY_DNM,out_id,wgt,msk,MULTIPLY_BY_TALLY,prg_nm,rcd)
+#pragma omp parallel for default(none) firstprivate(msk_out,wgt_out) private(DO_CONFORM_MSK,DO_CONFORM_WGT,idx,wgt_avg) shared(MULTIPLY_BY_TALLY,MUST_CONFORM,NRM_BY_DNM,WGT_MSK_CRD_VAR,dbg_lvl,dmn_avg,fp_stderr,fp_stdout,in_id,msk,msk_nm,msk_val,nbr_dmn_avg,nbr_var_prc,nco_op_typ,op_typ_rlt,out_id,prg_nm,rcd,var_prc,var_prc_out,wgt,wgt_nm)
 #endif /* not _OPENMP */
     for(idx=0;idx<nbr_var_prc;idx++){ /* Process all variables in current file */
       if(dbg_lvl > 0) rcd+=nco_var_prc_crr_prn(idx,var_prc[idx]->nm);
-      if(dbg_lvl > 0) (void)fflush(stderr);
+      if(dbg_lvl > 0) (void)fflush(fp_stderr);
       
-      /* Allocate and, if necesssary, initialize accumulation space for all processed variables */
+      /* Allocate and, if necessary, initialize accumulation space for all processed variables */
       var_prc_out[idx]->sz=var_prc[idx]->sz;
-      if((var_prc_out[idx]->tally=var_prc[idx]->tally=(long *)malloc(var_prc_out[idx]->sz*sizeof(long))) == NULL){
-	(void)fprintf(stdout,"%s: ERROR Unable to malloc() %ld*%ld bytes for tally buffer for variable %s in main()\n",prg_nm_get(),var_prc_out[idx]->sz,(long)sizeof(long),var_prc_out[idx]->nm);
+      if((var_prc_out[idx]->tally=var_prc[idx]->tally=(long *)nco_malloc_flg(var_prc_out[idx]->sz*sizeof(long))) == NULL){
+	(void)fprintf(fp_stdout,"%s: ERROR Unable to malloc() %ld*%ld bytes for tally buffer for variable %s in main()\n",prg_nm_get(),var_prc_out[idx]->sz,(long)sizeof(long),var_prc_out[idx]->nm);
 	nco_exit(EXIT_FAILURE); 
-      } /* end if */
+      } /* end if err */
       (void)nco_zero_long(var_prc_out[idx]->sz,var_prc_out[idx]->tally);
-      if((var_prc_out[idx]->val.vp=(void *)malloc(var_prc_out[idx]->sz*nco_typ_lng(var_prc_out[idx]->type))) == NULL){
-	(void)fprintf(stdout,"%s: ERROR Unable to malloc() %ld*%zu bytes for value buffer for variable %s in main()\n",prg_nm_get(),var_prc_out[idx]->sz,nco_typ_lng(var_prc_out[idx]->type),var_prc_out[idx]->nm);
+      if((var_prc_out[idx]->val.vp=(void *)nco_malloc_flg(var_prc_out[idx]->sz*nco_typ_lng(var_prc_out[idx]->type))) == NULL){
+	(void)fprintf(fp_stdout,"%s: ERROR Unable to malloc() %ld*%lu bytes for value buffer for variable %s in main()\n",prg_nm_get(),var_prc_out[idx]->sz,(unsigned long)nco_typ_lng(var_prc_out[idx]->type),var_prc_out[idx]->nm);
 	nco_exit(EXIT_FAILURE); 
-      } /* end if */
+      } /* end if err */
       (void)nco_var_zero(var_prc_out[idx]->type,var_prc_out[idx]->sz,var_prc_out[idx]->val);
       
       (void)nco_var_refresh(in_id,var_prc[idx]); /* Routine contains OpenMP critical regions */
       /* Retrieve variable from disk into memory */
+      if(dbg_lvl > 2) (void)fprintf(fp_stdout,"%s: DEBUG: fxm TODO nco354 About to nco_var_get() %s\n",prg_nm,var_prc[idx]->nm);
       (void)nco_var_get(in_id,var_prc[idx]); /* Routine contains OpenMP critical regions */
+      if(dbg_lvl > 2) (void)fprintf(fp_stdout,"%s: DEBUG: fxm TODO nco354 Finished nco_var_get() %s\n",prg_nm,var_prc[idx]->nm);
       
       /* Convert char, short, long, int types to doubles before arithmetic */
       var_prc[idx]=nco_typ_cnv_rth(var_prc[idx],nco_op_typ);
@@ -655,7 +705,7 @@ main(int argc,char **argv)
 	  } /* end if */
 	  
 	  /* Mask by changing variable to missing value where condition is false */
-	  (void)nco_var_mask(var_prc[idx]->type,var_prc[idx]->sz,var_prc[idx]->has_mss_val,var_prc[idx]->mss_val,msk_val,op_typ_rlt,msk_out->val,var_prc[idx]->val);
+	  (void)nco_var_msk(var_prc[idx]->type,var_prc[idx]->sz,var_prc[idx]->has_mss_val,var_prc[idx]->mss_val,msk_val,op_typ_rlt,msk_out->val,var_prc[idx]->val);
 	} /* end if */
       } /* end if */
       /* Perform non-linear transformations before weighting */
@@ -689,6 +739,7 @@ main(int argc,char **argv)
       if(NRM_BY_DNM && DO_CONFORM_WGT && (!var_prc[idx]->is_crd_var || WGT_MSK_CRD_VAR)){
 	/* Duplicate wgt_out as wgt_avg so that wgt_out is not contaminated by any
 	   averaging operation and may be reused on next variable.
+	   fxm: Using wgt_out on next variable would be thread-unsafe! verify! OpenMP
 	   Free wgt_avg after each use but continue to reuse wgt_out */
 	wgt_avg=nco_var_dpl(wgt_out);
 	
@@ -696,7 +747,7 @@ main(int argc,char **argv)
 	  double mss_val_dbl=double_CEWI;
 	  /* Set denominator to missing value at all locations where variable is missing value
 	     If this is accomplished by setting weight to missing value wherever variable is missing value
-	     then weight must not be reused by next variable (which might conform but have missing values in different locations)
+	     then weight must not be re-used by next variable (which might conform but have missing values in different locations)
 	     This is one good reason to copy wgt_out into disposable wgt_avg for each new variable */
 	  /* First make sure wgt_avg has same missing value as variable */
 	  (void)nco_mss_val_cp(var_prc[idx],wgt_avg);
@@ -711,19 +762,19 @@ main(int argc,char **argv)
 	  default: nco_dfl_case_nc_type_err(); break;
 	  } /* end switch */
 	  /* Second mask wgt_avg where variable is missing value */
-	  (void)nco_var_mask(wgt_avg->type,wgt_avg->sz,var_prc[idx]->has_mss_val,var_prc[idx]->mss_val,mss_val_dbl,nco_op_ne,var_prc[idx]->val,wgt_avg->val);
+	  (void)nco_var_msk(wgt_avg->type,wgt_avg->sz,var_prc[idx]->has_mss_val,var_prc[idx]->mss_val,mss_val_dbl,nco_op_ne,var_prc[idx]->val,wgt_avg->val);
 	} /* endif weight must be checked for missing values */
 	
 	if(msk_nm != NULL && DO_CONFORM_MSK){
-	  /* Must mask weight in same fashion as variable was masked */
-	  /* If msk and var did not conform then do not mask wgt */
-	  /* Ensure wgt_avg has a missing value */
+	  /* Must mask weight in same fashion as variable was masked
+	     If msk and var did not conform then do not mask wgt
+	     Ensure wgt_avg has a missing value */
 	  if(!wgt_avg->has_mss_val){
 	    wgt_avg->has_mss_val=True;
 	    wgt_avg->mss_val=nco_mss_val_mk(wgt_avg->type);
 	  } /* end if */
 	  /* Mask by changing weight to missing value where condition is false */
-	  (void)nco_var_mask(wgt_avg->type,wgt_avg->sz,wgt_avg->has_mss_val,wgt_avg->mss_val,msk_val,op_typ_rlt,msk_out->val,wgt_avg->val);
+	  (void)nco_var_msk(wgt_avg->type,wgt_avg->sz,wgt_avg->has_mss_val,wgt_avg->mss_val,msk_val,op_typ_rlt,msk_out->val,wgt_avg->val);
 	} /* endif weight must be masked */
 	
 	/* fxm: temporary kludge to make sure weight has tally space
@@ -731,14 +782,15 @@ main(int argc,char **argv)
 	   it is created, sometimes, before the tally array for var_prc_out[idx] is 
 	   created, and thus the nco_var_dpl() call in nco_var_cnf_dmn() does not copy
 	   a tally array into wgt_avg. See related note about this above. TODO #114.*/
-	if((wgt_avg->tally=(long *)nco_realloc(wgt_avg->tally,wgt_avg->sz*sizeof(long))) == NULL){
-	  (void)fprintf(stdout,"%s: ERROR Unable to realloc() %ld*%ld bytes for tally buffer for weight %s in main()\n",prg_nm_get(),wgt_avg->sz,(long)sizeof(long),wgt_avg->nm);
-	  nco_exit(EXIT_FAILURE); 
-	} /* end if */
+	if(wgt_avg->sz > 0)
+	  if((wgt_avg->tally=(long *)nco_realloc(wgt_avg->tally,wgt_avg->sz*sizeof(long))) == NULL){
+	    (void)fprintf(fp_stdout,"%s: ERROR Unable to realloc() %ld*%ld bytes for tally buffer for weight %s in main()\n",prg_nm_get(),wgt_avg->sz,(long)sizeof(long),wgt_avg->nm);
+	    nco_exit(EXIT_FAILURE); 
+	  } /* end if */
 	/* Average weight over specified dimensions (tally array is set here) */
 	wgt_avg=nco_var_avg(wgt_avg,dmn_avg,nbr_dmn_avg,nco_op_avg);
 	if(MULTIPLY_BY_TALLY){
-	  /* Currently this is not implemented */
+	  /* NB: Currently this is not implemented */
 	  /* Multiply numerator (weighted sum of variable) by tally 
 	     We deviously accomplish this by dividing denominator by tally */
 	  (void)nco_var_nrm(wgt_avg->type,wgt_avg->sz,wgt_avg->has_mss_val,wgt_avg->mss_val,wgt_avg->tally,wgt_avg->val);
@@ -746,7 +798,7 @@ main(int argc,char **argv)
 	/* Divide numerator by denominator */
 	/* Diagnose common PEBCAK before it causes core dump */
 	if(var_prc_out[idx]->sz == 1L && var_prc_out[idx]->type == NC_INT && var_prc_out[idx]->val.lp[0] == 0){
-	  (void)fprintf(stdout,"%s: ERROR Weight in denominator weight = 0.0, will cause SIGFPE\n%s: HINT Sum of masked, averaged weights must be non-zero\n%s: HINT A possible workaround is to remove variable \"%s\" from output file using \"%s -x -v %s ...\"\n%s: Expecting core dump...now!\n",prg_nm,prg_nm,prg_nm,var_prc_out[idx]->nm,prg_nm,var_prc_out[idx]->nm,prg_nm);
+	  (void)fprintf(fp_stdout,"%s: ERROR Weight in denominator weight = 0.0, will cause SIGFPE\n%s: HINT Sum of masked, averaged weights must be non-zero\n%s: HINT A possible workaround is to remove variable \"%s\" from output file using \"%s -x -v %s ...\"\n%s: Expecting core dump...now!\n",prg_nm,prg_nm,prg_nm,var_prc_out[idx]->nm,prg_nm,var_prc_out[idx]->nm,prg_nm);
 	} /* end if */
 	/* Divide numerator by masked, averaged, weights */
 	switch(nco_op_typ){
@@ -763,7 +815,8 @@ main(int argc,char **argv)
 	case nco_op_ttl: /* Total is already in buffer, do nothing */	
 	  break;
 	default:
-	  (void)fprintf(stdout,"%s: ERROR Illegal nco_op_typ in weighted normalization\n",prg_nm);
+	  (void)fprintf(fp_stdout,"%s: ERROR Illegal nco_op_typ in weighted normalization\n",prg_nm);
+	  nco_exit(EXIT_FAILURE);
 	  break;
 	} /* end switch */
 	/* Free wgt_avg, but keep wgt_out, after each use */
@@ -786,14 +839,17 @@ main(int argc,char **argv)
 	case nco_op_min: /* Minimum is already in buffer, do nothing */
 	case nco_op_max: /* Maximum is already in buffer, do nothing */	
 	case nco_op_ttl: /* Total is already in buffer, do nothing */	
+	  break;
 	default:
+	  (void)fprintf(fp_stdout,"%s: ERROR Illegal nco_op_typ in non-weighted normalization\n",prg_nm);
+	  nco_exit(EXIT_FAILURE);
 	  break;
 	} /* end switch */
       }else if(!NRM_BY_DNM){
 	/* Normalization has been turned off by user, we are done */
 	;
       }else{
-	(void)fprintf(stdout,"%s: ERROR Unforeseen logical branch in main()\n",prg_nm);
+	(void)fprintf(fp_stdout,"%s: ERROR Unforeseen logical branch in main()\n",prg_nm);
 	nco_exit(EXIT_FAILURE);
       } /* end if */
       /* Some non-linear operations require additional processing */
@@ -812,7 +868,7 @@ main(int argc,char **argv)
       /* Free tally buffer */
       var_prc_out[idx]->tally=(long *)nco_free(var_prc_out[idx]->tally);
       
-      /* Revert any arithmetic promotion but leave unpacking (for now) */
+      /* Revert any arithmetic promotion but leave unpacked (for now) */
       var_prc_out[idx]=nco_var_cnf_typ(var_prc_out[idx]->typ_upk,var_prc_out[idx]);
       
       /* Free current input buffer */
@@ -822,13 +878,14 @@ main(int argc,char **argv)
 #pragma omp critical
 #endif /* _OPENMP */
       { /* begin OpenMP critical */
-	/* Copy average to output file and free averaging buffer */
+	/* Copy average to output file then free averaging buffer */
 	if(var_prc_out[idx]->nbr_dim == 0){
 	  (void)nco_put_var1(out_id,var_prc_out[idx]->id,var_prc_out[idx]->srt,var_prc_out[idx]->val.vp,var_prc_out[idx]->type);
 	}else{ /* end if variable is scalar */
 	  (void)nco_put_vara(out_id,var_prc_out[idx]->id,var_prc_out[idx]->srt,var_prc_out[idx]->cnt,var_prc_out[idx]->val.vp,var_prc_out[idx]->type);
 	} /* end if variable is array */
       } /* end OpenMP critical */
+      /* Free current output buffer */
       var_prc_out[idx]->val.vp=nco_free(var_prc_out[idx]->val.vp);
       
     } /* end (OpenMP parallel for) loop over idx */
@@ -848,7 +905,7 @@ main(int argc,char **argv)
     /* Remove local copy of file */
     if(FILE_RETRIEVED_FROM_REMOTE_LOCATION && REMOVE_REMOTE_FILES_AFTER_PROCESSING) (void)nco_fl_rm(fl_in);
     
-  } /* end loop over idx_fl */
+  } /* end loop over fl_idx */
   
   /* Close output file and move it from temporary to permanent location */
   (void)nco_fl_out_cls(fl_out,fl_out_tmp,out_id);
@@ -856,3 +913,5 @@ main(int argc,char **argv)
   nco_exit_gracefully();
   return EXIT_SUCCESS;
 } /* end main() */
+
+
