@@ -1,9 +1,9 @@
-// $Header: /cvsroot/nco/nco/src/nco_c++/nco_att.cc,v 1.4 2003/01/16 09:26:55 zender Exp $ 
+// $Header: /cvsroot/nco/nco/src/nco_c++/nco_att.cc,v 1.11 2004/07/08 01:40:11 zender Exp $ 
 
 // Implementation (declaration) of C++ interface to netCDF attribute routines
 
-/* Copyright (C) 2001--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 2001--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include <nco_att.hh> // C++ interface to netCDF attribute routines
@@ -34,7 +34,7 @@ nco_put_att // [fnc] Create attribute
 (const int &nc_id, // I [enm] netCDF file ID
  const int &var_id, // I [id] Variable ID
  const std::string &att_nm, // I [sng] Attribute name
- const size_t &att_sz, // I [sng] Attribute length
+ const size_t &att_sz, // I [nbr] Attribute length
  const float * const &att_val, // I [frc] Attribute value
  const nc_type &att_typ) // I [enm] Attribute type 
 {
@@ -54,6 +54,9 @@ nco_put_att // [fnc] Create attribute
 {
   // Purpose: Wrapper for nco_put_att<float>()
   const size_t att_sz(1);
+  /* fxm: NB: float is the source from which all other nco_put_att<type> functions should
+     be copied/templated, once the best method for handling single value puts is determined, 
+     that is */
   //  int rcd=nco_put_att(nc_id,var_id,att_nm,att_sz,&(const_cast<float &>(att_val)),att_typ);
   // float foo_val(att_val);
   // int rcd=nco_put_att(nc_id,var_id,att_nm,att_sz,&foo_val,att_typ);
@@ -66,7 +69,7 @@ nco_put_att // [fnc] Create attribute
 (const int &nc_id, // I [enm] netCDF file ID
  const int &var_id, // I [id] Variable ID
  const std::string &att_nm, // I [sng] Attribute name
- const size_t &att_sz, // I [sng] Attribute length
+ const size_t &att_sz, // I [nbr] Attribute length
  const double * const &att_val, // I [frc] Attribute value
  const nc_type &att_typ) // I [enm] Attribute type 
 {
@@ -76,6 +79,104 @@ nco_put_att // [fnc] Create attribute
   return rcd;
 } // end nco_put_att<double>()
 
+int // O [enm] Return success code
+nco_put_att // [fnc] Create attribute
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const std::string &att_nm, // I [sng] Attribute name
+ const double &att_val, // I [frc] Attribute value
+ const nc_type &att_typ) // I [enm] Attribute type 
+{
+  // Purpose: Wrapper for nc_put_att_double()
+  const size_t att_sz(1);
+  int rcd=nc_put_att_double(nc_id,var_id,att_nm.c_str(),att_typ,att_sz,&(const_cast<double &>(att_val)));
+  return rcd;
+} // end nco_put_att<double>()
+
+int // O [enm] Return success code
+nco_put_att // [fnc] Create attribute
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const std::string &att_nm, // I [sng] Attribute name
+ const size_t &att_sz, // I [nbr] Attribute length
+ const int * const &att_val, // I [frc] Attribute value
+ const nc_type &att_typ) // I [enm] Attribute type 
+{
+  // Purpose: Wrapper for nc_put_att_int()
+  int rcd=nc_put_att_int(nc_id,var_id,att_nm.c_str(),att_typ,att_sz,att_val);
+  return rcd;
+} // end nco_put_att<int>()
+
+int // O [enm] Return success code
+nco_put_att // [fnc] Create attribute
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const std::string &att_nm, // I [sng] Attribute name
+ const int &att_val, // I [frc] Attribute value
+ const nc_type &att_typ) // I [enm] Attribute type 
+{
+  // Purpose: Wrapper for nc_put_att_int()
+  const size_t att_sz(1);
+  int rcd=nc_put_att_int(nc_id,var_id,att_nm.c_str(),att_typ,att_sz,&(const_cast<int &>(att_val)));
+  return rcd;
+} // end nco_put_att<int>()
+
+int // O [enm] Return success code
+nco_put_att // [fnc] Create attribute
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const std::string &att_nm, // I [sng] Attribute name
+ const size_t &att_sz, // I [nbr] Attribute length
+ const short * const &att_val, // I [frc] Attribute value
+ const nc_type &att_typ) // I [enm] Attribute type 
+{
+  // Purpose: Wrapper for nc_put_att_short()
+  int rcd=nc_put_att_short(nc_id,var_id,att_nm.c_str(),att_typ,att_sz,att_val);
+  return rcd;
+} // end nco_put_att<short>()
+
+int // O [enm] Return success code
+nco_put_att // [fnc] Create attribute
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const std::string &att_nm, // I [sng] Attribute name
+ const short &att_val, // I [frc] Attribute value
+ const nc_type &att_typ) // I [enm] Attribute type 
+{
+  // Purpose: Wrapper for nc_put_att_short()
+  const size_t att_sz(1);
+  int rcd=nc_put_att_short(nc_id,var_id,att_nm.c_str(),att_typ,att_sz,&(const_cast<short &>(att_val)));
+  return rcd;
+} // end nco_put_att<short>()
+
+int // O [enm] Return success code
+nco_put_att // [fnc] Create attribute
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const std::string &att_nm, // I [sng] Attribute name
+ const size_t &att_sz, // I [nbr] Attribute length
+ const long * const &att_val, // I [frc] Attribute value
+ const nc_type &att_typ) // I [enm] Attribute type 
+{
+  // Purpose: Wrapper for nc_put_att_long()
+  int rcd=nc_put_att_long(nc_id,var_id,att_nm.c_str(),att_typ,att_sz,att_val);
+  return rcd;
+} // end nco_put_att<long>()
+
+int // O [enm] Return success code
+nco_put_att // [fnc] Create attribute
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const std::string &att_nm, // I [sng] Attribute name
+ const long &att_val, // I [frc] Attribute value
+ const nc_type &att_typ) // I [enm] Attribute type 
+{
+  // Purpose: Wrapper for nc_put_att_long()
+  const size_t att_sz(1);
+  int rcd=nc_put_att_long(nc_id,var_id,att_nm.c_str(),att_typ,att_sz,&(const_cast<long &>(att_val)));
+  return rcd;
+} // end nco_put_att<long>()
+
 int // O [enm] Return success code
 nco_put_att // [fnc] Create attribute
 (const int &nc_id, // I [enm] netCDF file ID
@@ -178,6 +279,7 @@ nco_inq_attid // [fnc] Inquire attribute ID
   // Purpose: Wrapper for nco_inq_attid()
   int att_id; // O [id] Attribute ID
   int rcd=nco_inq_attid(nc_id,var_id,att_nm,att_id);
+  rcd+=0; // CEWI
   return att_id;
 } // end nco_inq_attid() 
 
@@ -190,6 +292,7 @@ nco_inq_attid // [fnc] Inquire attribute ID
   // Purpose: Wrapper for nco_inq_attid()
   int att_id; // O [id] Attribute ID
   int rcd=nco_inq_attid(nc_id,nco_inq_varid(nc_id,var_nm),att_nm,att_id);
+  rcd+=0; // CEWI
   return att_id;
 } // end nco_inq_attid() 
 
@@ -222,6 +325,7 @@ nco_inq_attname // [fnc] Inquire attribute name
   // Purpose: Wrapper for nco_inq_attname()
   std::string att_nm; // O [id] Variable name
   int rcd=nco_inq_attname(nc_id,var_id,att_id,att_nm);
+  rcd+=0; // CEWI
   return att_nm;
 } // end nco_inq_attname() 
 
@@ -234,6 +338,7 @@ nco_inq_attname // [fnc] Inquire attribute name
   // Purpose: Wrapper for nco_inq_attname()
   std::string att_nm; // O [id] Variable name
   int rcd=nco_inq_attname(nc_id,nco_inq_varid(nc_id,var_nm),att_id,att_nm);
+  rcd+=0; // CEWI
   return att_nm;
 } // end nco_inq_attname() 
 
@@ -264,6 +369,7 @@ nco_inq_atttype // [fnc] Inquire variable type
   // Purpose: Wrapper for nco_inq_atttype()
   nc_type att_typ; // O [enm] Variable type
   int rcd=nco_inq_atttype(nc_id,nco_inq_varid(nc_id,var_nm),att_nm,att_typ);
+  rcd+=0; // CEWI
   return att_typ;
 } // end nco_inq_atttype() 
 
@@ -295,6 +401,7 @@ nco_inq_attlen // [fnc] Inquire attribute length
   // Purpose: Wrapper for nco_inq_attlen()
   size_t att_sz; // O [nbr] Attribute size
   int rcd=nco_inq_attlen(nc_id,var_id,att_nm,att_sz);
+  rcd+=0; // CEWI
   return att_sz;
 } // end nco_inq_attlen() 
 
@@ -307,6 +414,7 @@ nco_inq_attlen // [fnc] Inquire attribute length
   // Purpose: Wrapper for nco_inq_attlen()
   size_t att_sz; // O [nbr] Attribute size
   int rcd=nco_inq_attlen(nc_id,var_id,nco_inq_attname(nc_id,var_id,att_id),att_sz);
+  rcd+=0; // CEWI
   return att_sz;
 } // end nco_inq_attlen() 
 
@@ -319,6 +427,7 @@ nco_inq_attlen // [fnc] Inquire attribute length
   // Purpose: Wrapper for nco_inq_attlen(nc_inq_dimid())
   size_t att_sz; // O [nbr] Attribute size
   int rcd=nco_inq_attlen(nc_id,nco_inq_varid(nc_id,var_nm),att_nm,att_sz);
+  rcd+=0; // CEWI
   return att_sz;
 } // end nco_inq_attlen() 
 
@@ -334,8 +443,14 @@ nco_get_att // [fnc] Get attribute value
 {
   // Purpose: Wrapper for nc_get_att_text()
   size_t att_sz=nco_inq_attlen(nc_id,var_id,att_nm);
-  char *att_val_chr=new char[att_sz];
+  if(att_sz == 0) nco_wrn_prn("nco_get_att<std::string> will attempt to read NC_CHAR of size zero");
+  // Allocate one extra byte for manual NUL-termination
+  char *att_val_chr=new char[att_sz+1];
   int rcd=nc_get_att_text(nc_id,var_id,att_nm.c_str(),att_val_chr);
+  /* Attribute on disk may or may not be NUL-terminated
+     In either case, NUL-terminate it before assigning it to string
+     Otherwise, overloaded string:=(char *) assignment will not terminate */
+  att_val_chr[att_sz]='\0';
   att_val=att_val_chr;
   delete []att_val_chr;
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_att<std::string>");
