@@ -1,9 +1,9 @@
-// $Header: /cvsroot/nco/nco/src/nco_c++/nco_hgh.cc,v 1.3 2003/01/16 09:26:55 zender Exp $ 
+// $Header: /cvsroot/nco/nco/src/nco_c++/nco_hgh.cc,v 1.7 2004/07/08 06:18:31 zender Exp $ 
 
 // Implementation (declaration) of C++ interface to high-level NCO utilities
 
-/* Copyright (C) 2001--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 2001--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include <nco_hgh.hh> // High-level NCO utilities
@@ -56,7 +56,7 @@ nco_var_dfn // [fnc] Define variables in output netCDF file
   // Allow file to already be in define mode
   rcd=nco_redef(nc_id,NC_EINDEFINE); // [fnc] Put open netCDF dataset into define mode
 
-  register long dmn_idx; // [idx] Counting index for dmn
+  int dmn_idx; // [idx] Counting index for dmn
   for(idx=0;idx<var_mtd_nbr;idx++){
     // Set dmn_nbr_max to, e.g., 1, to limit output file size at expense of losing all 2-D variables
     if(var_mtd[idx].dmn_nbr <= dmn_nbr_max){
