@@ -1,9 +1,9 @@
-// $Header: /cvsroot/nco/nco/src/nco_c++/nco_utl.cc,v 1.7 2003/01/16 09:26:55 zender Exp $ 
+// $Header: /cvsroot/nco/nco/src/nco_c++/nco_utl.cc,v 1.13 2004/07/06 05:22:27 zender Exp $ 
 
 // Implementation (declaration) of C++ interface utilities for netCDF routines
 
-/* Copyright (C) 2001--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 2001--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include <nco_utl.hh> // C++ interface utilities for netCDF routines
@@ -34,18 +34,18 @@ nco_err_exit // [fnc] Lookup, print netCDF error message, exit
  const std::string &msg_opt) // I [sng] Optional supplemental error message
 {
   /* Purpose: Print netCDF error message, routine name, and exit 
-     Routine is called by all wrappers when a fatal error is encountered
+     Routine is called by all wrappers when fatal error is encountered
      msg variable allows wrapper to pass more descriptive information than 
-     is contained in the netCDF-defined error message.
-     Use msg to print, e.g., the name of the variable which caused the error */
-  std::string sbr_nm("nco_err_exit()");
+     netCDF-defined error message contains.
+     Use msg to print, e.g., name of variable which caused error */
+  const std::string sbr_nm("nco_err_exit()");
   if(rcd != NC_NOERR){
     std::cout << sbr_nm << ": ERROR netCDF library returned error code " << rcd << std::endl;
     std::cout << sbr_nm << ": ERROR " << msg << std::endl << nc_strerror(rcd) << std::endl;
     if(msg_opt != "") std::cout << sbr_nm << ": " << msg_opt << std::endl;
 #ifdef ABORT_ON_ERROR
     std::abort(); // [fnc] Produce core dump
-#else
+#else // !ABORT_ON_ERROR
 #ifndef SGIMP64 // fxm: SGI IRIX CC does not support std::exit nor have cstdlib
     std::exit(EXIT_FAILURE); // [fnc] Exit nicely
 #else // SGIMP64
@@ -55,6 +55,21 @@ nco_err_exit // [fnc] Lookup, print netCDF error message, exit
   } // endif error
 } // end nco_err_exit()
 
+void 
+nco_wrn_prn // [fnc] Print NCO warning message and return
+(const std::string &msg, // I [sng] Supplemental warning message
+ const std::string &msg_opt) // I [sng] Optional supplemental warning message
+{
+  /* Purpose: Print NCO warning message and return
+     Routine is called by some wrappers when non-fatal weirdness is encountered
+     msg variable allows wrapper to pass descriptive information
+     Use msg to print, e.g., name of variable which caused warning */
+  const std::string sbr_nm("nco_wrn_prn()");
+  std::cout << sbr_nm << ": WARNING " << msg << std::endl;
+  if(msg_opt != "") std::cout << sbr_nm << ": " << msg_opt << std::endl;
+  return;
+} // end nco_wrn_prn()
+
 // Begin nco_inq_varsz() overloads
 
 int // O [enm] Return success code
@@ -88,6 +103,7 @@ nco_inq_varsz // [fnc] Compute size of variable
   // Purpose: Compute size (number of elements) in specified variable
   size_t var_sz; // O [nbr] Variable size
   int rcd=nco_inq_varsz(nc_id,var_id,var_sz);
+  rcd+=0; /* CEWI */
   return var_sz;
 } // end nco_inq_varsz()
 
@@ -249,6 +265,7 @@ nco_get_xtype // [fnc] Determine external netCDF type
      Algorithm assumes internal representation is IEEE
      Thus algorithm fails for Crays, where native float is 8 B (double precision) */
   float flt_foo=var_val; // CEWI
+  flt_foo++; // Squelch "declared but never referenced" warnings
   return NC_FLOAT;
 } // end nco_get_xtype()
 
@@ -260,6 +277,7 @@ nco_get_xtype // [fnc] Determine external netCDF type
      Algorithm assumes internal representation is IEEE
      Thus algorithm fails for Crays, where native float is 8 B (double precision) */
   double dbl_foo=var_val; // CEWI
+  dbl_foo++; // Squelch "declared but never referenced" warnings
   return NC_DOUBLE;
 } // end nco_get_xtype()
 
