@@ -1,9 +1,9 @@
-// $Header: /cvsroot/nco/nco/src/nco_c++/nco_var.cc,v 1.7 2003/01/16 09:26:55 zender Exp $ 
+// $Header: /cvsroot/nco/nco/src/nco_c++/nco_var.cc,v 1.16 2004/09/18 23:24:31 zender Exp $ 
 
 // Purpose: Implementation (declaration) of C++ interface to netCDF variable routines
 
-/* Copyright (C) 2001--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 2001--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include <nco_var.hh> // C++ interface to netCDF variable routines
@@ -56,7 +56,7 @@ nco_def_var // [fnc] Create variable in netCDF file
   /* fxm: It would be nice if dmn_id could be passed as const, but I cannot
      figure out how to do this and pass a const int * to nc_def_var */
   //  int rcd=nco_def_var(nc_id,var_nm,var_xtype,dmn_id.size(),&dmn_id[0],var_id);
-  int rcd=nco_def_var(nc_id,var_nm,var_xtype,dmn_id.size(),&(const_cast<std::valarray<int> &>(dmn_id)[0]),var_id);
+  int rcd=nco_def_var(nc_id,var_nm,var_xtype,static_cast<int>(dmn_id.size()),&(const_cast<std::valarray<int> &>(dmn_id)[0]),var_id);
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_def_var");
   return rcd;
 } // end nco_def_var<std::valarray<int>>()
@@ -108,6 +108,7 @@ nco_inq_varid // [fnc] Inquire variable ID
   // Purpose: Wrapper for nco_inq_varid()
   int var_id; // O [id] Variable ID
   int rcd=nco_inq_varid(nc_id,var_nm,var_id);
+  rcd+=0; // CEWI
   return var_id;
 } // end nco_inq_varid() 
 
@@ -138,6 +139,7 @@ nco_inq_varname // [fnc] Inquire variable name
   // Purpose: Wrapper for nco_inq_varname()
   std::string var_nm; // O [id] Variable name
   int rcd=nco_inq_varname(nc_id,var_id,var_nm);
+  rcd+=0; // CEWI
   return var_nm;
 } // end nco_inq_varname() 
 
@@ -166,6 +168,7 @@ nco_inq_vartype // [fnc] Inquire variable type
   // Purpose: Wrapper for nco_inq_vartype()
   nc_type var_typ; // O [enm] Variable type
   int rcd=nco_inq_vartype(nc_id,var_id,var_typ);
+  rcd+=0; // CEWI
   return var_typ;
 } // end nco_inq_vartype() 
 
@@ -177,6 +180,7 @@ nco_inq_vartype // [fnc] Inquire variable type
   // Purpose: Wrapper for nco_inq_vartype()
   nc_type var_typ; // O [enm] Variable type
   int rcd=nco_inq_vartype(nc_id,nco_inq_varid(nc_id,var_nm),var_typ);
+  rcd+=0; // CEWI
   return var_typ;
 } // end nco_inq_vartype() 
 
@@ -205,6 +209,7 @@ nco_inq_varndims // [fnc] Inquire variable rank
   // Purpose: Wrapper for nco_inq_varndims()
   int dmn_nbr; // O [nbr] Number of dimensions
   int rcd=nco_inq_varndims(nc_id,var_id,dmn_nbr);
+  rcd+=0; // CEWI
   return dmn_nbr;
 } // end nco_inq_varndims() 
 
@@ -216,6 +221,7 @@ nco_inq_varndims // [fnc] Inquire variable rank
   // Purpose: Wrapper for nco_inq_varndims()
   int dmn_nbr; // O [nbr] Number of dimensions
   int rcd=nco_inq_varndims(nc_id,nco_inq_varid(nc_id,var_nm),dmn_nbr);
+  rcd+=0; // CEWI
   return dmn_nbr;
 } // end nco_inq_varndims() 
 
@@ -244,6 +250,7 @@ nco_inq_varnatts // [fnc] Inquire variable attributes
   // Purpose: Wrapper for nco_inq_varnatts()
   int att_nbr; // O [nbr] Number of attributes
   int rcd=nco_inq_varnatts(nc_id,var_id,att_nbr);
+  rcd+=0; // CEWI
   return att_nbr;
 } // end nco_inq_varnatts() 
 
@@ -255,6 +262,7 @@ nco_inq_varnatts // [fnc] Inquire variable attributes
   // Purpose: Wrapper for nco_inq_varnatts()
   int att_nbr; // O [nbr] Number of attributes
   int rcd=nco_inq_varnatts(nc_id,nco_inq_varid(nc_id,var_nm),att_nbr);
+  rcd+=0; // CEWI
   return att_nbr;
 } // end nco_inq_varnatts() 
 
@@ -275,6 +283,114 @@ nco_inq_vardimid // [fnc] Inquire variable dimension IDS
 } // end nco_inq_vardimid() 
 
 // End nco_inq_var() overloads
+// Begin nco_put_vara() overloads
+
+// Overload 1:  Write array given ID, start, count vectors
+int // O [enm] Return success code
+nco_put_vara // [fnc] Write variable to netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const std::valarray<size_t> &var_srt, // I [idx] Start vector
+ const std::valarray<size_t> &var_cnt, // I [nbr] Count vector
+ const float * const &var_val) // I [frc] Variable value
+{
+  // Purpose: Wrapper for nc_put_vara_float()
+  int rcd=nc_put_vara_float(nc_id,var_id,&(const_cast<std::valarray<size_t> &>(var_srt)[0]),&(const_cast<std::valarray<size_t> &>(var_cnt)[0]),var_val);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_vara<valarray,valarray,float *> failed with variable "+nco_inq_varname(nc_id,var_id));
+  return rcd;
+} // end nco_put_vara<valarray,valarray,float *>()
+
+int // O [enm] Return success code
+nco_put_vara // [fnc] Write variable to netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const size_t * const &var_srt, // I [idx] Start vector
+ const size_t * const &var_cnt, // I [nbr] Count vector
+ const float * const &var_val) // I [frc] Variable value
+{
+  // Purpose: Wrapper for nc_put_vara_float()
+  int rcd=nc_put_vara_float(nc_id,var_id,var_srt,var_cnt,var_val);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_vara<size_t *,size_t *,float *> failed with variable "+nco_inq_varname(nc_id,var_id));
+  return rcd;
+} // end nco_put_vara<size_t *,size_t *,float *>()
+
+int // O [enm] Return success code
+nco_put_vara // [fnc] Write variable to netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const size_t * const &var_srt, // I [idx] Start vector
+ const size_t * const &var_cnt, // I [nbr] Count vector
+ const double * const &var_val) // I [frc] Variable value
+{
+  // Purpose: Wrapper for nc_put_vara_double()
+  int rcd=nc_put_vara_double(nc_id,var_id,var_srt,var_cnt,var_val);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_vara<size_t *,size_t *,double *> failed with variable "+nco_inq_varname(nc_id,var_id));
+  return rcd;
+} // end nco_put_vara<size_t *,size_t *,double *>()
+
+int // O [enm] Return success code
+nco_put_vara // [fnc] Write variable to netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const size_t * const &var_srt, // I [idx] Start vector
+ const size_t * const &var_cnt, // I [nbr] Count vector
+ const long double * const &var_val) // I [frc] Variable value
+{
+  // Purpose: Wrapper for nc_put_vara_double()
+  const size_t var_sz(nco_inq_varsz(nc_id,var_id)); // [nbr] Number of elements
+  double *var_val_dbl=new double[var_sz]; // [frc] Double precision values
+  for(size_t lmn_idx=0;lmn_idx<var_sz;lmn_idx++){
+    var_val_dbl[lmn_idx]=var_val[lmn_idx];
+  } // end loop over idx
+  int rcd=nc_put_vara_double(nc_id,var_id,var_srt,var_cnt,var_val_dbl);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_vara<size_t *,size_t *,long double *> failed with variable "+nco_inq_varname(nc_id,var_id));
+  delete []var_val_dbl;
+  return rcd;
+} // end nco_put_vara<size_t *,size_t *,long double *>()
+
+int // O [enm] Return success code
+nco_put_vara // [fnc] Write variable to netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const size_t * const &var_srt, // I [idx] Start vector
+ const size_t * const &var_cnt, // I [nbr] Count vector
+ const int * const &var_val) // I [frc] Variable value
+{
+  // Purpose: Wrapper for nc_put_vara_int()
+  int rcd=nc_put_vara_int(nc_id,var_id,var_srt,var_cnt,var_val);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_vara<size_t *,size_t *,int *> failed with variable "+nco_inq_varname(nc_id,var_id));
+  return rcd;
+} // end nco_put_vara<size_t *,size_t *,int *>()
+
+int // O [enm] Return success code
+nco_put_vara // [fnc] Write variable to netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const size_t * const &var_srt, // I [idx] Start vector
+ const size_t * const &var_cnt, // I [nbr] Count vector
+ const long * const &var_val) // I [frc] Variable value
+{
+  // Purpose: Wrapper for nc_put_vara_long()
+  int rcd=nc_put_vara_long(nc_id,var_id,var_srt,var_cnt,var_val);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_vara<size_t *,size_t *,long *> failed with variable "+nco_inq_varname(nc_id,var_id));
+  return rcd;
+} // end nco_put_vara<size_t *,size_t *,long *>()
+
+int // O [enm] Return success code
+nco_put_vara // [fnc] Write variable to netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const size_t * const &var_srt, // I [idx] Start vector
+ const size_t * const &var_cnt, // I [nbr] Count vector
+ const short * const &var_val) // I [frc] Variable value
+{
+  // Purpose: Wrapper for nc_put_vara_short()
+  int rcd=nc_put_vara_short(nc_id,var_id,var_srt,var_cnt,var_val);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_vara<size_t *,size_t *,short *> failed with variable "+nco_inq_varname(nc_id,var_id));
+  return rcd;
+} // end nco_put_vara<size_t *,size_t *,short *>()
+
+// End nco_put_vara() overloads
 // Begin nco_put_var() overloads
 
 // Overload 1:  Write array given ID
@@ -286,7 +402,7 @@ nco_put_var // [fnc] Write variable to netCDF file
 {
   // Purpose: Wrapper for nc_put_var_float()
   int rcd=nc_put_var_float(nc_id,var_id,var_val);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<float *>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<float *> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_put_var<float *>()
 
@@ -298,7 +414,7 @@ nco_put_var // [fnc] Write variable to netCDF file
 {
   // Purpose: Wrapper for nc_put_var_double()
   int rcd=nc_put_var_double(nc_id,var_id,var_val);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<double *>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<double *> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_put_var<double *>()
 
@@ -315,7 +431,7 @@ nco_put_var // [fnc] Write variable to netCDF file
     var_val_dbl[lmn_idx]=var_val[lmn_idx];
   } // end loop over idx
   int rcd=nc_put_var_double(nc_id,var_id,var_val_dbl);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<long double *>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<long double *> failed with variable "+nco_inq_varname(nc_id,var_id));
   delete []var_val_dbl;
   return rcd;
 } // end nco_put_var<long double *>()
@@ -328,7 +444,7 @@ nco_put_var // [fnc] Write variable to netCDF file
 {
   // Purpose: Wrapper for nc_put_var_int()
   int rcd=nc_put_var_int(nc_id,var_id,var_val);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<int *>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<int *> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_put_var<int *>()
 
@@ -340,7 +456,7 @@ nco_put_var // [fnc] Write variable to netCDF file
 {
   // Purpose: Wrapper for nc_put_var_long()
   int rcd=nc_put_var_long(nc_id,var_id,var_val);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<long *>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<long *> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_put_var<long *>()
 
@@ -352,7 +468,7 @@ nco_put_var // [fnc] Write variable to netCDF file
 {
   // Purpose: Wrapper for nc_put_var_short()
   int rcd=nc_put_var_short(nc_id,var_id,var_val);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<short *>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<short *> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_put_var<short *>()
 
@@ -365,9 +481,9 @@ nco_put_var // [fnc] Write variable to netCDF file
 {
   // Purpose: Wrapper for nc_put_var1_float()
   // fxm: Decide which method is best, this one or the one used for longs */
-  std::valarray<size_t> srt(static_cast<size_t>(0),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
+  std::valarray<size_t> srt(static_cast<size_t>(0U),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
   int rcd=nc_put_var1_float(nc_id,var_id,&srt[0],&(const_cast<float &>(var_val)));
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<float>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<float> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_put_var<float>()
 
@@ -378,9 +494,9 @@ nco_put_var // [fnc] Write variable to netCDF file
  const double &var_val) // I [frc] Variable value
 {
   // Purpose: Wrapper for nc_put_var1_double()
-  std::valarray<size_t> srt(static_cast<size_t>(0),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
+  std::valarray<size_t> srt(static_cast<size_t>(0U),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
   int rcd=nc_put_var1_double(nc_id,var_id,&srt[0],&(const_cast<double &>(var_val)));
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<double>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<double> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_put_var<double>()
 
@@ -392,9 +508,9 @@ nco_put_var // [fnc] Write variable to netCDF file
 {
   // Purpose: Wrapper for nc_put_var1_double()
   const double var_val_dbl(var_val); // [frc] Double precision value
-  std::valarray<size_t> srt(static_cast<size_t>(0),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
+  std::valarray<size_t> srt(static_cast<size_t>(0U),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
   int rcd=nc_put_var1_double(nc_id,var_id,&srt[0],&(const_cast<double &>(var_val_dbl)));
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<long double>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<long double> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_put_var<long double>()
 
@@ -405,9 +521,9 @@ nco_put_var // [fnc] Write variable to netCDF file
  const int &var_val) // I [frc] Variable value
 {
   // Purpose: Wrapper for nc_put_var1_int()
-  std::valarray<size_t> srt(static_cast<size_t>(0),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
+  std::valarray<size_t> srt(static_cast<size_t>(0U),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
   int rcd=nc_put_var1_int(nc_id,var_id,&srt[0],&(const_cast<int &>(var_val)));
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<int>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<int> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_put_var<int>()
 
@@ -430,10 +546,10 @@ nco_put_var // [fnc] Write variable to netCDF file
     rcd=nc_put_var1_long(nc_id,var_id,srt_dmn_0_ptr,&(const_cast<long &>(var_val)));
   }else{
     // Default starting index is 0
-    std::valarray<size_t> srt(static_cast<size_t>(0),static_cast<size_t>(dmn_nbr));
+    std::valarray<size_t> srt(static_cast<size_t>(0U),static_cast<size_t>(dmn_nbr));
     rcd=nc_put_var1_long(nc_id,var_id,&srt[0],&(const_cast<long &>(var_val)));
   } // endelse
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<long>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<long> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_put_var<long>()
 
@@ -444,9 +560,9 @@ nco_put_var // [fnc] Write variable to netCDF file
  const short &var_val) // I [frc] Variable value
 {
   // Purpose: Wrapper for nc_put_var1_short()
-  std::valarray<size_t> srt(static_cast<size_t>(0),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
+  std::valarray<size_t> srt(static_cast<size_t>(0U),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
   int rcd=nc_put_var1_short(nc_id,var_id,&srt[0],&(const_cast<short &>(var_val)));
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<short>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<short> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_put_var<short>()
 
@@ -466,7 +582,7 @@ nco_get_var // [fnc] Ingest variable from netCDF file
   // Allocate space for variable: User is responsible for freeing space
   var_val=new float[var_sz]; // [frc] Variable value
   rcd=nc_get_var_float(nc_id,var_id,var_val);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var<float>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var<float> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_get_var<float>()
 
@@ -482,7 +598,7 @@ nco_get_var // [fnc] Ingest variable from netCDF file
   // Allocate space for variable: User is responsible for freeing space
   var_val=new double[var_sz]; // [frc] Variable value
   rcd=nc_get_var_double(nc_id,var_id,var_val);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var<double>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var<double> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_get_var<double>()
 
@@ -499,7 +615,7 @@ nco_get_var // [fnc] Ingest variable from netCDF file
   var_val=new long double[var_sz]; // [frc] Variable value
   double *var_val_dbl=new double[var_sz]; // [frc] Double precision values
   rcd=nc_get_var_double(nc_id,var_id,var_val_dbl);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var<long double>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var<long double> failed with variable "+nco_inq_varname(nc_id,var_id));
   for(size_t lmn_idx=0;lmn_idx<var_sz;lmn_idx++){
     var_val[lmn_idx]=var_val_dbl[lmn_idx];
   } // end loop over idx
@@ -519,7 +635,7 @@ nco_get_var // [fnc] Ingest variable from netCDF file
   // Allocate space for variable: User is responsible for freeing space
   var_val=new int[var_sz]; // [frc] Variable value
   rcd=nc_get_var_int(nc_id,var_id,var_val);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var<int>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var<int> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_get_var<int>()
 
@@ -535,7 +651,7 @@ nco_get_var // [fnc] Ingest variable from netCDF file
   // Allocate space for variable: User is responsible for freeing space
   var_val=new long[var_sz]; // [frc] Variable value
   rcd=nc_get_var_long(nc_id,var_id,var_val);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var<long>");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var<long> failed with variable "+nco_inq_varname(nc_id,var_id));
   return rcd;
 } // end nco_get_var<long>()
 
@@ -548,6 +664,7 @@ nco_get_var // [fnc] Ingest variable from netCDF file
   // Purpose: Wrapper for nco_get_var()
   float *var_val; // [frc] Variable value
   int rcd=nco_get_var(nc_id,var_id,var_val);
+  rcd+=0; // CEWI
   return var_val;
 } // end nco_get_var<float>()
 
@@ -560,6 +677,7 @@ nco_get_var // [fnc] Ingest variable from netCDF file
   // Purpose: Wrapper for nco_get_var()
   float *var_val; // [frc] Variable value
   int rcd=nco_get_var(nc_id,nco_inq_varid(nc_id,var_nm),var_val);
+  rcd+=0; // CEWI
   return var_val;
 } // end nco_get_var<float>()
 
