@@ -1,9 +1,9 @@
-// $Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.11 2003/01/16 09:26:55 zender Exp $
+// $Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.15 2004/06/03 05:52:43 zender Exp $
 
 // Purpose: Test for C++ programs using libnco_c++ API
 
-/* Copyright (C) 2001--2003 Charlie Zender
-   This software is distributed under the terms of the GNU General Public License
+/* Copyright (C) 2001--2004 Charlie Zender
+   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
    See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 /* Compilation
@@ -50,10 +50,10 @@ int main(int argc,char **argv)
   long sz_nbr(1); // [nbr] Number of sizes
   register long idx; // [idx] Counting index
 
-  const std::string CVS_Date("$Date: 2003/01/16 09:26:55 $"); // [sng] CVS date string
-  const std::string CVS_Header("$Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.11 2003/01/16 09:26:55 zender Exp $"); // [sng] CVS header string
-  const std::string CVS_Id("$Id: tst.cc,v 1.11 2003/01/16 09:26:55 zender Exp $"); // [sng] CVS identification string
-  const std::string CVS_Revision("$Revision: 1.11 $"); // [sng] CVS revision string
+  const std::string CVS_Date("$Date: 2004/06/03 05:52:43 $"); // [sng] CVS date string
+  const std::string CVS_Header("$Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.15 2004/06/03 05:52:43 zender Exp $"); // [sng] CVS header string
+  const std::string CVS_Id("$Id: tst.cc,v 1.15 2004/06/03 05:52:43 zender Exp $"); // [sng] CVS identification string
+  const std::string CVS_Revision("$Revision: 1.15 $"); // [sng] CVS revision string
   const std::string date_cvs(CVS_Date.length() > 7 ? CVS_Date.substr(7,19) : static_cast<std::string>("Unknown")); // [sng] Date from CVS
   const std::string sbr_nm("main"); // [sng] Subroutine name
   const std::string prg_nm("libnco_c++"); // [sng] Program name
@@ -163,6 +163,9 @@ int main(int argc,char **argv)
   std::cerr << "Wrote results to " << fl_out << std::endl;
   std::cerr << "ncks: ncks -C -H -F -m -u -d wvl,0.5e-6 -v wvl " << fl_out << std::endl;
 
+  // Fix all unused identifiers at end where no harm can be done
+  idx=0*(rcd+lat_nbr+lon_nbr+dmn_bnd_sz[0]+argc+sizeof(argv)); // [idx] Counting index CEWI
+
   return EXIT_SUCCESS;
 } // end main()
 
